#========================================================================================
# package      DE_EPAGES::AppStore::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides shop functions of AppStore.
#========================================================================================
package DE_EPAGES::AppStore::API::Shop;
use base Exporter;
use strict;

use DE_EPAGES::AppStore::API::Constants qw (
    UNITY_SCRIPT_TAGS_APP_NAME
);
use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Order::API::Constants  qw (
    UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE
);
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use HTTP::Status qw( HTTP_NOT_FOUND );

our @EXPORT_OK = qw (
    CreateUnityScriptTagsApp
    DeleteUnityScriptTagsApp
    GetUnityScriptTagsApp
    GetRegisterSessionScriptUrl
);

sub ErrorDefinition {
    return {
        'AppAlreadyExists' => 'App \'#Name\' not created because it exists already',
        'AppNotInstalled' => 'App \'#Name\' not deleted because it is not installed'
    };
}

#========================================================================================
# function     GetUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Get the app used to hand over script-tags to Unity (epages-ui)
#               for the given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub GetUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');

    my $ScriptTagAppClient;
    if (defined $UnityScriptTagsAppGUID) {
        # get credentials from existing app
        $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});
    } else {
        # create new app, if there has not been any so far
        $ScriptTagAppClient = CreateUnityScriptTagsApp($Shop);
    }

    return $ScriptTagAppClient;
}

#========================================================================================
# function     CreateUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Creates a new private developer app used to hand over script-tags to
#               Unity (epages-ui).
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub CreateUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});
    my $UnityScriptTagsApp;

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');
    if (defined $UnityScriptTagsAppGUID) {
        eval {
            # try to get existing app
            $UnityScriptTagsApp = $ScriptTagAppClient->getApp();
        };
        if (ExistsError()) {
            my $Error = GetError();
            # ignore error '404 not found' and create a new app instead
            Error($Error) unless $Error->vars->{'Status'} == HTTP_NOT_FOUND;
        } else {
            # do not create a new app in case one already exists
            Error('AppAlreadyExists', {'Name' => UNITY_SCRIPT_TAGS_APP_NAME});
        }
    }

    # create a new app
    $ScriptTagAppClient->createApp();

    # add initial script-tags
    $ScriptTagAppClient->addScriptTag(GetRegisterSessionScriptUrl($Shop));

    return $ScriptTagAppClient;
}

#========================================================================================
# function     DeleteUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Deletes the Unity script-tags app for the given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub DeleteUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');
    # withput ID we do not know which app to delete
    Error('AppNotInstalled', {'Name' => UNITY_SCRIPT_TAGS_APP_NAME}) unless defined $UnityScriptTagsAppGUID;

    my $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});

    $ScriptTagAppClient->deleteApp();

    return $ScriptTagAppClient;
}

#========================================================================================
# function     GetRegisterSessionScriptUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetRegisterSessionScriptUrl($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the script URL for the register session script tag
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $RegisterSessionScriptUrl | relative URL | string
#========================================================================================
sub GetRegisterSessionScriptUrl {
    my ($Shop) = @_;

    my $RegisterSessionScriptPageType = LoadPageTypeByAlias(UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE);
    my $RegisterSessionScriptViewAction = $RegisterSessionScriptPageType->get('ViewAction')->alias;
    return $Shop->get('WebServerScriptName') . '?ViewAction=' . $RegisterSessionScriptViewAction;
}

1;
