use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp GetRegisterSessionScriptUrl );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Order::API::Constants  qw ( UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );

#========================================================================================
# function         PatchRegisterSessionScriptTag
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchRegisterSessionScriptTag();
#----------------------------------------------------------------------------------------
# description      Deletes the absolute script tag Url for the register session script
#                   tag and adds a relative one
#========================================================================================
sub PatchRegisterSessionScriptTag {
    my ($ShopAlias, $Force) = @_;

    my $RegisterSessionScriptPageType = LoadPageTypeByAlias(UNITY_REGISTER_SESSION_SCRIPT_PAGE_TYPE);
    my $RegisterSessionScriptViewAction = $RegisterSessionScriptPageType->get('ViewAction')->alias;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            _PatchRegisterSessionScriptTag($Shop, $RegisterSessionScriptViewAction, $Force);
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                _PatchRegisterSessionScriptTag($Shop, $RegisterSessionScriptViewAction, $Force);
            };
            warn('Patch of the RegisterSession ScriptTag failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    return;
}

sub _PatchRegisterSessionScriptTag {
    my ($Shop, $RegisterSessionScriptViewAction, $Force) = @_;

    my $RegisterSessionScriptUrl = GetRegisterSessionScriptUrl($Shop);
    my $ScriptTagAppClient = GetUnityScriptTagsApp($Shop);
    my $aScriptTags = $ScriptTagAppClient->getAllScriptTags();
    my $IsScriptTagPresent = 0;

    foreach my $hScriptTag (@$aScriptTags) {
        # do nothing if (relative) RegisterSessionScriptUrl is already present and force flag is not set
        return if (!$Force && $hScriptTag->{'scriptUrl'} eq $RegisterSessionScriptUrl);

        if ($hScriptTag->{'scriptUrl'} =~ m/ViewAction=$RegisterSessionScriptViewAction/) {
            # delete existing script url
            $ScriptTagAppClient->client->delete($ScriptTagAppClient->singleScriptTagUrl($hScriptTag->{'scriptTagId'}),
                $ScriptTagAppClient->requestHeaders );
            $IsScriptTagPresent = 1;
        }
    }

    # add relative script url
    $ScriptTagAppClient->addScriptTag($RegisterSessionScriptUrl) if $IsScriptTagPresent;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Force;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'force'       => \$Force,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchRegisterSessionScriptTag($ShopAlias, $Force); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Deletes the absolute script tag Url for the register session script
    tag and adds a relative one.
Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -force      delete relative Url, if it exists already
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );