package DE_EPAGES::CustomForm::UI::CustomForm;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateShortGUID );
use DE_EPAGES::Core::API::TestParameter qw ( TestMultipleEMailAddresses );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use DE_EPAGES::Object::API::Template qw (ProcessFileTemplate);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);

#==============================================================================
# save fields in MBO
sub AddField {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomForm  = $Servlet->object;
    my $Form        = $Servlet->form;

    my $hFormValues = $Form->form($CustomForm, 'AddField');
    my $Folder      = $CustomForm->folder('Fields');

    # check if FieldClassID is given (since it's necessary)
    unless ($hFormValues->{'FieldClassID'}) {
        $Form->addFormError({
            'Name'   => 'AddField',
            'Reason' => 'FieldClassIDIsNecessary'
        });
        $Form->executeFormError;
    }

    my $FieldClassID = $hFormValues->{'FieldClassID'};
    my $FieldClass   = $CustomForm->getSite()->loadObject($FieldClassID, 'Class', {'AllowSystemSite' => 1});

    # check if FieldName is given (since it's mandatory)
    if (not $hFormValues->{'FieldName'} and $FieldClass->alias ne 'CaptionFormField') {
        $Form->addFormError({
            'Name'   => 'AddField',
            'Reason' => 'FieldNameIsMandatory'
        });
        $Form->executeFormError;
    }

    # create field
    my $NewFieldObject = $Folder->insertChild($FieldClass, {
        'Alias'    => $FieldClass->alias . GenerateShortGUID()
    });

    # save attributes
    $NewFieldObject->set({
        IsAlterable => 1,
        Position => 9999,
        IsMandatory => $hFormValues->{IsMandatory}
    });
    $NewFieldObject->set({
        FieldName => $hFormValues->{'FieldName'},
        AdditionalHelpText => $hFormValues->{AdditionalHelpText}
    }, $Servlet->languageID);

    # update positions
    $Folder->renumberChildren;

    # insert two options if it's an option field
    if ($FieldClass->get('BaseClass')->alias eq 'CustomFormFieldWithOptions') {
        my $OptionsFolder = $NewFieldObject->folder('Options');
        my $OptionClass = LoadClassByAlias('CustomFormFieldOption');

        my $Option1 = $OptionsFolder->insertChild($OptionClass, {
            'Alias'    => 'Option' . GenerateShortGUID(),
            'Position' => 10
        });
        my $Option2 = $OptionsFolder->insertChild($OptionClass, {
            'Alias'    => 'Option' . GenerateShortGUID(),
            'Position' => 20
        });

        # {Option} lang tags for RadioButton, {Choice} for SelectBox
        my $Flavour = $FieldClass->alias eq 'RadioButtonFormField' ? 'OptionContent.txt' : 'ChoiceContent.txt';
        my $OptionContent = join '', $ENV{'EPAGES_CARTRIDGES'}, '/DE_EPAGES/CustomForm/Templates/MBO/', $Flavour;

        foreach my $LanguageID (@{$CustomForm->getSite()->languages}) {
            local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
            my $Code2 = GetInfoLanguage($LanguageID)->{'Code2'};
            my $OptionTranslated = ProcessFileTemplate($OptionContent, $Code2, {});

            $Option1->set({
                'OptionName' => $OptionTranslated . ' 1'
            }, $LanguageID);

            $Option2->set({
                'OptionName' => $OptionTranslated . ' 2'
            }, $LanguageID);
        }
    }

    return;
}

#==============================================================================
# saves recipients settings in MBO
sub SaveRecipients {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;

    # form validation
    my $hFormValues = $Form->form($Servlet->object, 'SaveRecipients');

    foreach my $Test ('ToEMail', 'CcEMail', 'BccEMail') {
        eval {
            TestMultipleEMailAddresses($Test, $hFormValues->{$Test}) if defined $hFormValues->{$Test};
        };
        if (ExistsError()) {
            $Form->addFormError({
                'Name'   => 'SaveRecipients',
                'Reason' => $Test.'InputError'
            });
        }
    }
    $Form->executeFormError();

    # save standard attributes
    return $self->SUPER::Save($Servlet);
}

#==============================================================================
# save fields in MBO
sub SaveFields {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomForm = $Servlet->object;
    my $Form = $Servlet->form;
    my $hFormValues = $Form->form($CustomForm, 'SaveFields');
    my @MandatoryFields = $Form->formValue('IsMandatoryObjectID');

    $CustomForm->saveFields( $hFormValues->{'ListPositions'}, \@MandatoryFields, $hFormValues->{'ListDeletings'} );

    return;
}

#==============================================================================
# submit form in SF
sub Submit {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomForm = $Servlet->object;
    my $Form = $Servlet->form;
    my $User = $Servlet->user;

    $self->GoogleCaptcha($Servlet);

    # validate input fields
    my $aFields = $CustomForm->get('Fields');
    $_->validateFormInput($Servlet) foreach @$aFields;
    $Form->executeFormError;

    my $IsRegistrated = 0;
    my $CustomerNo;
    my $CustomerEMail;

    if ($User and not $User->get('IsAnonymous')) {
        my $Customer = $User->get('Customers')->[0];
        if( defined $Customer ) {
            $CustomerNo = $Customer->alias;
            $IsRegistrated = 1;
        }
        $CustomerEMail = $User->get('EMail');
    }

    # create structure for email tle's
    my @FormFields = ();
    foreach my $Field (@$aFields) {
        my $FieldName = $Field->get('FieldName', $Servlet->languageID);
        if ($Field->class->alias eq 'EMailFormField') {
            $CustomerEMail = $Form->formValue($Field->alias) unless ($CustomerEMail);
        }
        # don't send fields where the fieldname isn't translated
        elsif (defined $FieldName and $FieldName ne '') {
            push(@FormFields, {
                'CustomFormField' => $Field->tleHash,
                'Value'           => $Form->formValue($Field->alias) // ''
            });
        }
    }

    # check if CustomerEMail is defined
    die 'no customer email found' unless ($CustomerEMail);

    # fill email metadata
    my $Shop = $CustomForm->getSite();
    my $Merchant = $Shop->get('SuperMerchant');
    my $LanguageID = $Servlet->languageID;

    my %CustomerRegionalParams = (
        'Locale'   => $Servlet->locale,
        'Language' => GetCodeByLanguageID($LanguageID)
    );

    my %MerchantMailParams = (
        'TO'          => $CustomForm->get('ToEMail') // $Merchant->get('EMail') // $Shop->get('MailFrom'),
        'CC'          => $CustomForm->get('CcEMail'),
        'BCC'         => $CustomForm->get('BccEMail'),
        'FROM'        => $CustomForm->get('ToEMail') // $Merchant->get('EMail') // $Shop->get('MailFrom'),
        'REPLY'       => $CustomerEMail,
        'HTMLAllowed' => 1,
        'Subject'     => $CustomForm->get('MailSubject', $LanguageID),
    );
    my %CustomerMailParams = (
        'TO'          => $CustomerEMail,
        'FROM'        => $Shop->get('MailFrom'),
        'HTMLAllowed' => 1,
        'Subject'     => $CustomForm->get('MailSubject', $LanguageID),
    );

    my %MailVars = (
        'FormFields'    => \@FormFields,
        'IsRegistrated' => $IsRegistrated,
        'CustomerNo'    => $CustomerNo,
        'CustomerEMail' => $CustomerEMail,
    );

    my $MailType = LoadRootObject()->folder('MailTypes')->child('CustomForm');

    # send email to merchant
    eval { $MailType->send($CustomForm, \%MerchantMailParams, \%CustomerRegionalParams, \%MailVars) };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->code eq 'Invalid_Sender') {
            $Form->addFormError({
                'Name'   => 'InvalidSender',
                'Reason' => 'InvalidSender'
            });
            $Form->executeFormError;
        } else {
            Error($Error);
        }
    }

    # send email to customer if he/she's registrated and checked the corresponding option
    my $SendToCustomer = $Form->formValue('CustomerCopy');
    if (defined $SendToCustomer and $SendToCustomer and $User) {
        $MailType->send($CustomForm, \%CustomerMailParams, \%CustomerRegionalParams, \%MailVars);
    }

    return;
}

#==============================================================================
# enable ssl for custom forms
sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->redirectToSecureConnection($Servlet->object->getSite->get('WebServerScriptUrlSSL'));
    return $self->SUPER::View($Servlet);
}

1;
