#========================================================================================
# package      DE_EPAGES::Design::Hooks::Shop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::Design::Hooks::Shop;

use strict;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Design::API::Cartridge;
use DE_EPAGES::Core::API::Image               qw ( ResizeImage );
use DE_EPAGES::Core::API::File                qw ( fsplit ExistsFile );
use DE_EPAGES::Design::API::Style             qw ( AddStyleLogo DeleteStyleLogo );

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Imports default shop data into a new shop when it is created.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $Cartridge = DE_EPAGES::Design::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);
    return;
}

#========================================================================================
# function     OnAfterUpdateShop
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Removes obsolete style sheet files, if PageCacheLastInvalidation will be
#               set to undef.
#               (Hook OBJ_AfterUpdateShop)
#----------------------------------------------------------------------------------------
# input        $hParams |  hook arguments, used keys are
#               <ul>
#                   <li>Object - shop - object
#                   <li>Values - update values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    if (exists $hParams->{'Values'}{'PageCacheLastInvalidation'} &&
      !defined $hParams->{'Values'}{'PageCacheLastInvalidation'} ) {
        $_->removeObsoleteStyleSheets() foreach @{$Shop->get('Styles')};
    }

    unless (defined $Shop->get('Logo')){
        DeleteStyleLogo($_) foreach @{$Shop->get('Styles')};
    }

    #resize logos for all styles
    return unless defined $hParams->{'Values'}{'Logo'};
    return if $Shop->get('IsUnityRegistered');

    my $ShopLogo = $Shop->get('Logo');
    my ($Path, undef, $Extension) = fsplit($ShopLogo);

    $ShopLogo = $Shop->get('PublicPath').'/'.$ShopLogo;
    if (ExistsFile($ShopLogo)){
        if ($Path){
            ResizeImage($ShopLogo, $ShopLogo, undef, 90);
        }
        AddStyleLogo($_, $ShopLogo, $Extension) foreach @{$Shop->get('Styles')};
    }
    return;
}

#========================================================================================
# function     OnLoadCheckoutCSS
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnLoadCheckoutCSS($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook OBJ_LoadCheckoutCSS.
#               Checks if the CSS for the checkout should be requested from Unity via REST
#               or the related shop attribute is taken
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - shop object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnLoadCheckoutCSS {
    my ($hParams) = @_;

    # skip if another hook function already wants to request the CSS from Unity
    return if $hParams->{'Values'}->{'RequestCSSPathFromUnity'};

    if( defined $hParams->{'Values'}->{'ChangeAction'}
        && $hParams->{'Values'}->{'ChangeAction'} eq 'PickupBasket') {
        $hParams->{'Values'}->{'RequestCSSPathFromUnity'} = 1;
    }

    return;
}

1;
