#========================================================================================
# package      DE_EPAGES::GoogleAdWords::Hooks::Shop::OnBeforeUpdateShop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::GoogleAdWords::Hooks::Shop;

use strict;

use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::GoogleAdWords::API::Constants  qw (
    GOOGLE_AD_WORDS_SCRIPT_URL_QUERY
);

#========================================================================================
# function     OnBeforeUpdateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  check if GoogleAdWordsHTMLCode has changed,
#               parse the related GoogleAdWordsID out of it
#               and regeister the script-tag via the REST-API
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object        - shop object          - object
#                 <li>ObjectID      - shop object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    if ($Shop->get('IsUnityRegistered')) {
        my $hValues = $hParams->{'Values'};
        if (exists $hValues->{'GoogleAdWordsHTMLCode'}) {
            my $GoogleAdWordsHTMLCode = $hValues->{'GoogleAdWordsHTMLCode'};

            # get app client to communicate with the epagesJ REST API
            my $AppsClient = GetUnityScriptTagsApp($Shop);

            # get existing script tag, if any
            my $GoogleAdwordsScriptTagUrl = $Shop->get('WebServerScriptName') . GOOGLE_AD_WORDS_SCRIPT_URL_QUERY;

            # parse id from Google AdWords script
            my $GoogleAdWordsID = undef;
            if (defined $GoogleAdWordsHTMLCode) {
                # parse id=xxx" or id=xxx& and gtag('config', 'xxx');
                # check that both ids are identical
                if ($GoogleAdWordsHTMLCode =~ m/id=([^\"&]*)[\"&][\s\S]*gtag\(\'config\', \'([^']*)\'\)/
                    && $1 eq $2) {
                    $GoogleAdWordsID = $1;
                } else {
                    # throw error if not id found while parsing --> either wrong merchant input
                    # or google ad words script format changed --> parsing has fo be adapted
                    Error('NoGoogleAdWordsIDFound');
                }
                # post new script tag, in case there has not been any so far
                $AppsClient->addScriptTag($GoogleAdwordsScriptTagUrl)
            } else {
                # no GoogleAdWordsHTMLCode given any more
                # delete existing script-tag
                $AppsClient->deleteScriptTag($GoogleAdwordsScriptTagUrl)
            }

            $Shop->set({'GoogleAdWordsID' => $GoogleAdWordsID});
        }
    }

    return;
}

1;
