use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::AppStore::API::Constants  qw (
    SHOPS_RESOURCE_PATH
    SCRIPT_TAGS_RESOURCE_PATH
    MY_SCRIPT_TAGS_URL_QUERY
);
use DE_EPAGES::AppStore::API::Object::DeveloperAppsClient;
use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath LoadRootObject );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );

#========================================================================================
# function         PatchUnityScriptTagsApp
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchUnityScriptTagsApp();
#----------------------------------------------------------------------------------------
# description      Patches the Google AdWords script tag for Unity shops to the
#                   EpagesUIScriptTags app and deletes the GoogleAdWords script-tag app.
#========================================================================================
sub PatchUnityScriptTagsApp {
    my ($ShopAlias, $Force) = @_;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            PatchUnityGoogleAdWordsScriptTagsApp($Shop);
        } else {
            # non Unity shop
            print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                PatchUnityGoogleAdWordsScriptTagsApp($Shop);
            };
            warn('Unity Storefront App renewal failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    return;
}

sub PatchUnityGoogleAdWordsScriptTagsApp {
    my ($Shop) = @_;

    my $hInfoShop = $Shop->get(['UnityGoogleAdWordsAppGUID', 'WebServerScriptNamePart']);
    return unless defined $hInfoShop->{'UnityGoogleAdWordsAppGUID'};

    # get app
    my $AppsClient = DE_EPAGES::AppStore::API::Object::DeveloperAppsClient->create({'Shop' => $Shop});
    my $GoogleAdWordsApp = $AppsClient->getApp($hInfoShop->{'UnityGoogleAdWordsAppGUID'});

    # get script-tags
    my $ScriptTagUrl = LoadRootObject->get('EPagesJBaseURL') . SHOPS_RESOURCE_PATH
        . '/' . $hInfoShop->{'WebServerScriptNamePart'}  . '/' . SCRIPT_TAGS_RESOURCE_PATH;
    my $hHeaders = {
        'Authorization' => 'Bearer ' . $GoogleAdWordsApp->{'token'},
        'User-Agent'    => 'epages6 via GoogleAdWords app'
    };
    my $RequestURL = $ScriptTagUrl . MY_SCRIPT_TAGS_URL_QUERY;
    my $Response = $AppsClient->client->get( $RequestURL, $hHeaders );
    my $hScriptTags = $AppsClient->decodeResponse($Response->decoded_content);

    # get new scrip-tag app
    my $ScriptTagAppClient = GetUnityScriptTagsApp($Shop);

    # delete app
    $AppsClient->deleteApp($GoogleAdWordsApp->{'privateAppId'});
    $Shop->set({ 'UnityGoogleAdWordsAppGUID' => undef });

    # add script-tags with new app
    foreach my $hScriptTag (@{ $hScriptTags->{'items'} }) {
        $ScriptTagAppClient->addScriptTag($hScriptTag->{'scriptUrl'});
    }
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Force;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchUnityScriptTagsApp($ShopAlias, $Force); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Patches the Google AdWords script tag for Unity shops to the EpagesUIScriptTags app
    and deletes the GoogleAdWords script-tag app.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );