#========================================================================================
# package      DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaFinancing
# base         DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase
# state        public
# description  object interface for PaymentMethodKlarnaFinancing.
#========================================================================================
package DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaFinancing;
use base DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase;

use strict;
use DE_EPAGES::Core::API::Error qw (GetError ExistsError);
use DE_EPAGES::ExternalPayment::API::Log qw(LogPayment);
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Klarna::API::Constants qw (KLARNA_FINANCING);
use DE_EPAGES::Klarna::API::Pclass qw (GetBasketPclasses);
use DE_EPAGES::Klarna::API::KlarnaCalc qw (CalcMonthlyFinancingCost CalcTotalFinancingPurchaseCost);

use constant DEFAULT_CURRENCY    => 'EUR';
use constant DEFAULT_INVOICE_FEE => 0.00;

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodPaynet');
#               my $Object = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Overrides and calls PaymentMethodKlarnaBase->insert to add a Template Path
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object
# return       $Object | new object | object
#========================================================================================
sub insert {
    my ($Class, $hInfo) = @_;

    my $InitialNoteTemplatePath = '/DE_EPAGES/Klarna/Templates/InitialNoteContentFinancing.html';
    my $self = $Class->SUPER::insert($hInfo, $InitialNoteTemplatePath);
    return $self;
}

#========================================================================================
# function      recalculateLineItem
# state         public
#----------------------------------------------------------------------------------------
# syntax        $Package->recalculateLineItem($LineItem);
#----------------------------------------------------------------------------------------
# description   Recalculates lineitem. Saves default Klarna payment type if
#                it is yet missing.
#----------------------------------------------------------------------------------------
# input         $LineItem | line item | object
# return        $object | created object | object
#========================================================================================
sub recalculateLineItem {
    my $self = shift;
    my ($LineItem) = @_;

    if (!($LineItem->get('InternalMethod'))) {
        my $SelectedMethod = KLARNA_FINANCING;
        $LineItem->set({ 'InternalMethod' => $SelectedMethod });
    }

    $self->SUPER::recalculateLineItem($LineItem);

    my $InvoiceNo = $LineItem->get('Kr_InvoiceNo');
    return if(isValid($InvoiceNo));

    my $PaymentMethod = $LineItem->get('PaymentMethod');
    my $Container = $LineItem->get('LineItemContainer');
    my $Basket = $Container->parent;
    my $aCampaigns = GetBasketPclasses($PaymentMethod, $Basket);
    my $Sum = $Container->get('GrandTotal');

    # Selected periodic cost
    my $OldSelectedPeriodicCostAlias;
    if (defined $LineItem->get('KlarnaPeriodicCost')) {
        $OldSelectedPeriodicCostAlias = $LineItem->get('KlarnaPeriodicCost')->get('Alias');
    }

    $LineItem->set({ 'Kr_PeriodicCosts' => [] });
    return unless defined($aCampaigns);
    foreach my $Pclass (@$aCampaigns) {
        if ($Container->get('GrandTotal') >= $Pclass->get('MinPurchase')) {
            my $PeriodicCost;
            my $hParams = {
                'Pclass' => $Pclass,
                'Sum' => $Sum,
            };
            eval {
                $PeriodicCost = CalcMonthlyFinancingCost($hParams);
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogPayment('KlarnaFinancing', 'Error in GetPeriodicCost', $Error->message);
            }
            my $NewCost = $self->SavePeriodicCost($LineItem,$PeriodicCost,$Pclass);
            $LineItem->set({'KlarnaPeriodicCost' => $NewCost}) unless(defined $LineItem->get('KlarnaPeriodicCost'));
            # If customer is from Norway, also total price for part payment has to be visible
            my $CurrencyID = $Pclass->get('CurrencyID');
            if ($CurrencyID eq 'NOK') {
                my $TotalCost;
                eval {
                    $TotalCost = CalcTotalFinancingPurchaseCost($hParams,'KlarnaFinancing');
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogPayment('KlarnaFinancing', 'Error in GetTotalCreditPurchaseCost', $Error->message);
                }
                $NewCost->set({ 'TotalCost' => $TotalCost });
            }
        }
    }

    if ($LineItem->get('InternalMethod') == KLARNA_FINANCING && scalar(@$aCampaigns) == 1) {
        my $Folder = $LineItem->folder('Kr_PeriodicCosts');
        if ($Folder->existsChild($aCampaigns->[0]->alias)) {
            my $PCost = $Folder->child($aCampaigns->[0]->alias);
            $LineItem->set({
                'KlarnaPeriodicCost' => $PCost,
                'Kr_TotalCost'       => $PCost->get('TotalCost')
            });
        }
    }

    if (defined $OldSelectedPeriodicCostAlias) {
        my $Parent = $LineItem->folder('Kr_PeriodicCosts');

        if ($Parent->existsChild($OldSelectedPeriodicCostAlias)) {
            my $SelectedCost =  $Parent->child($OldSelectedPeriodicCostAlias);
            $LineItem->set({ 'KlarnaPeriodicCost' => $SelectedCost })
        }
    }

    return;
}

#========================================================================================
# function      default
# state         private
#----------------------------------------------------------------------------------------
# syntax        $hValues = DE_EPAGES::Klarna::API::Object::PaymentMethodKlarna->default( $Parent, $hInfo );
# example       $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description   Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#               sets default fee,currency, testmode
#----------------------------------------------------------------------------------------
# input         $Parent | parent object of the new object | object
# input         $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return        $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return $class->SUPER::default($Parent, $hInfo, DEFAULT_INVOICE_FEE, DEFAULT_CURRENCY);
}
#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars));
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               The payment method can be added to the container if the CurrencyID
#               attribute matches the currency of the container.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;
    return 0 unless $self->checkGeneral($hVars);

    my $Container = $hVars->{'LineItemContainer'};
    return 0 unless defined($Container);

    my $Pclasses = GetBasketPclasses($self, $Container->parent);
    return 0 unless defined($Pclasses);

    my $GrandTotal = $Container->get('GrandTotal');
    foreach my $Cost (@$Pclasses) {
        unless ($Cost->get('IsAccount')) {
            if ($Cost->get('MinPurchase') <= $GrandTotal) {
                return 1;
            }
        }
    }

    return 0;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Funtion tests if a payment method is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | CurrencyID - alpha currency code (ISO 4217) - char(3),
#                        TaxModel - gross or net price - int                  | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $IsUsable =  $self->SUPER::usableAtStorefront($hVars);

    if ($IsUsable) {
        my $aCampaigns = $self->folder('Kr_Pclasses')->children;
        my $Total = scalar(@$aCampaigns);

        if ($Total <= 0) {
            LogPayment('KlarnaFinancing', 'Klarna usableAtStorefront', 'Campaign data is not available.');
            return 0;
        }

        return 1;
    }

    return 0;
}
1;