#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization
# state        public
# description  hard core synchronization module regarding multistores
#               it is possible to add or update product types to shops, using direct sql connection
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization;
use base DE_EPAGES::MultiStoreConnector::API::MSSynchronization;

use strict;
use feature 'switch';
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Object::API::Factory qw (
    LoadObject
    LoadClassByAlias
    LoadObjectByGUID
    LoadObjectByPath
    Transaction
    );
use DE_EPAGES::Product::API::Constants qw (PRODUCT_ATTRIBUTE_TYPES);
use JSON::XS;

# We interpret private methods as protected
## no critic (ProhibitPrivateSubroutineUse)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Synchronization = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new synchronization object of DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization.
#----------------------------------------------------------------------------------------
# input        %options | options for synchronization | hash
# return       $Synchronization | synchronization object | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    #super constructor
    my $self = __PACKAGE__->SUPER::new(%options);

    #info of attributes and their expected data types
    $self->{'AttributeParameters'} = {
        # object
        'alias' => 'string',
        'position' => 'integer',

        # attribute
        'type' => 'string',
        'length' => 'integer',
        'is_visible' => 'boolean',
        'is_html' => 'boolean',
        'show_in_own_tab' => 'boolean',
        'is_search_filter' => 'boolean',

        # predefattribute settings
        'pre_def_attributes' => 'array',
    };

    # info of predefattributes and their expected data types
    $self->{'PreDefAttributeAttributeParameters'} = {
        'alias' => 'string',
        'position' => 'integer',
    };

    #info of attributes mandatory
    $self->{'AttributeMandatory'} = {
        # object
        'alias' => 1,
        'position' => 1,

        # attribute
        'type' => 1,
        'length' => 0,
        'is_visible' => 1,
        'is_html' => 1,
        'show_in_own_tab' => 0,
        'is_search_filter' => 0,

        # predefattribute settings
        'pre_def_attributes' => 0,
    };

    # info of attributes mandatory for a predefattribute
    $self->{'PreDefAttributeAttributeMandatory'} = {
        'alias' => '1',
        'position' => '1',
    };

    return $self;

}

#========================================================================================
# function     addProductTypeAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Result = $Synchronization->addProductTypeAttributes($ShopAlias, $ahProductTypeAttributesInfo);
#----------------------------------------------------------------------------------------
# description  Function to add product types attributes to a specific subshop.
#----------------------------------------------------------------------------------------
# input        $ShopAlias | alias of shop | string
# input        $ahProductTypeAttributesInfo | array with attribute information | ref.array.hash
#----------------------------------------------------------------------------------------
# return       $Success | returns true if everything is correct | boolean
#========================================================================================
sub addProductTypeAttributes {
    my $self = shift;
    my ($ShopAlias, $ahProductTypeAttributesInfo) = @_;

    #empty cache
    $self->{'ShopCache'} = {};

    #get info
    my $ParentID;
    my $ProductType;

    my $ShopPath = '/Shops/' . $ShopAlias;
    my $Shop;

    eval {$Shop = LoadObjectByPath($ShopPath);};
    if(ExistsError){
        my $MyError = GetError();

        if($MyError->code eq 'PATH_NOT_FOUND'){
                Error('ShopNotFoundInStore', {
                    'Message' => 'shop not found in store',
                    'Shop' => $ShopAlias,
                });
        }
        else{
            Error($MyError);
        }
    }

    # get parent ids for each attribute to insert
    foreach my $hProductTypeAttributeInfo (@$ahProductTypeAttributesInfo) {
        # load the product type
        my $ProductTypeGUID = $hProductTypeAttributeInfo->{'product_type'};
        eval{
            $ProductType = LoadObjectByGUID($ProductTypeGUID);
        };
        if(ExistsError){
            my $MyError = GetError();

            if($MyError->code eq 'NoResults_ObjectByGUID'){
                Error('ProductTypeNotFoundInStore', {
                    'Message' => 'product type not found',
                    'GUID' => $ProductTypeGUID,
                });
            }
            else{
                Error($MyError);
            }
        }
        # parent is the 'Attributes' child of the product type
        $ParentID = $ProductType->folder('Attributes')->id;

        # save the parent id
        $hProductTypeAttributeInfo->{'parentid'} = $ParentID;

        # save also the id of the product type, which is used as attributeclassid for the attribute
        $hProductTypeAttributeInfo->{'attributeclassid'} = $ProductType->id;
    }

    my $Return = $self->_handleProductTypeAttributesAdd($ahProductTypeAttributesInfo, $Shop);

    #reset cache for this product type attributes
    $self->resetObjectCache($ParentID);
    $self->resetObjectCache($ProductType->id);
    $self->clearSiteCache($Shop);

    # reset cache for attributes of this product type
    $self->resetAttributeCache($ProductType->id);

    #reset cache for product types
    $self->resetObjectCache($Shop->id);

    #return 1 if function is passed successfully
    return $Return;
}

#========================================================================================
# function     getProductTypeAttributesContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->getProductTypeAttributesContent($aGUIDs);
#----------------------------------------------------------------------------------------
# description  Returns the product type attributes for product types which are specified by their GUIDs
#----------------------------------------------------------------------------------------
# input        $aGUIDs | global unique id's of product types to check | ref.array
# return       $hProductTypes | product types | ref.hash
#========================================================================================
sub getProductTypeAttributesContent {
    my $self = shift;
    my ($aProductTypeGUIDs) = @_;  # GUIDs of the product types

    my $hResult;
    my $aProductTypeIDs;

    # map product type GUIDS to object IDs
    foreach my $ProductTypeGUID (@$aProductTypeGUIDs) {
        push(@$aProductTypeIDs, $self->_getIDFromGUID($ProductTypeGUID));

        # We can be sure that $aProductTypeIDs->[i] matches $aProductTypeGUIDs->[i] because
        # _getIDFromGUID throws an error if a GUID cannot be found. 
    }

    # get product type attributes by product type IDs
    my $hProductTypeAttributesResult = $self->_handleProductTypeAttributesGet($aProductTypeIDs);

    # Build result hash with product type GUIDs as keys and their attributes as values
    for (my $i = 0; $i < scalar @$aProductTypeGUIDs; $i++) {
        my $ProductTypeGUID = $aProductTypeGUIDs->[$i];
        my $ProductTypeID = $aProductTypeIDs->[$i];

        $hResult->{$ProductTypeGUID} = $hProductTypeAttributesResult->{$ProductTypeID};
    }

    return $hResult;
}

#========================================================================================
# function     getPreDefAttributeAttributeParameterType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ParameterType = $self->getPreDefAttributeAttributeParameterType($AttributeName);
#----------------------------------------------------------------------------------------
# description  get type of a predefattribute parameter
#----------------------------------------------------------------------------------------
# input        $AttributeName | name of predefattribute parameter | string
# return       $ParameterType | parameter type | string
#========================================================================================
sub getPreDefAttributeAttributeParameterType{
    my $self = shift;
    my ($AttributeName) = @_;

    return $self->{'PreDefAttributeAttributeParameters'}->{$AttributeName};
}

#========================================================================================
# function     getPreDefAttributeAttributeMandatory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mandatory = $self->getPreDefAttributeAttributeMandatory($AttributeName);
#----------------------------------------------------------------------------------------
# description  get mandatory value for predefattribute parameter
#----------------------------------------------------------------------------------------
# input        $AttributeName | name of predefattribute parameter | string
# return       $Mandatory | true if predefattribute parameter is mandatory  | bool
#========================================================================================
sub getPreDefAttributeAttributeMandatory{
    my $self = shift;
    my ($AttributeName) = @_;

    return $self->{'PreDefAttributeAttributeMandatory'}->{$AttributeName};
}

#========================================================================================
# function     verifyPreDefAttributeContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->verifyPreDefAttributeContent($hContent);
#----------------------------------------------------------------------------------------
# description  verify parameters of predefattributes, throws error if verification fails
#----------------------------------------------------------------------------------------
# input        $hContent | ref.hash
#========================================================================================
sub verifyPreDefAttributeContent {
    my $self = shift;
    my ($hContent) = @_;

    #get info
    my $aPreDefAttributeAttributeNames = $self->getPreDefAttributeAttributeNames();
    my $Alias = $hContent->{'alias'};

    #for each attribute
    foreach my $PreDefAttributeAttributeName (@$aPreDefAttributeAttributeNames) {

        #get info
        my $ParameterType = $self->getPreDefAttributeAttributeParameterType($PreDefAttributeAttributeName);
        my $IsMandatory = $self->getPreDefAttributeAttributeMandatory($PreDefAttributeAttributeName);
        my $PreDefAttributeAttributeValue = $hContent->{$PreDefAttributeAttributeName};
        my $RaiseError;

        #check parameter type
        if ((not defined $PreDefAttributeAttributeValue) and $IsMandatory){
            Error('MissingMandatoryAttribute', {
                'Message' => 'mandatory attribute not exists',
                'Attribute' => $PreDefAttributeAttributeName,
                'Object' => $Alias,
            })
        }
        elsif ((not defined $PreDefAttributeAttributeValue) and (not $IsMandatory)){$RaiseError = 0}
        elsif($ParameterType eq 'string') { $RaiseError = 1 unless ($self->isString($PreDefAttributeAttributeValue))}
        elsif($ParameterType eq 'integer') { $RaiseError = 1 unless ($self->isInteger($PreDefAttributeAttributeValue))}
        elsif($ParameterType eq 'number') { $RaiseError = 1 unless ($self->isNumber($PreDefAttributeAttributeValue))}
        elsif($ParameterType eq 'boolean') { $RaiseError = 1 unless ($self->isBool($PreDefAttributeAttributeValue))}
        elsif($ParameterType eq 'object') { $RaiseError = 1 unless ($self->isObject($PreDefAttributeAttributeValue))}
        elsif($ParameterType eq 'array') { $RaiseError = 1 unless ($self->isArray($PreDefAttributeAttributeValue))}
        else { Error('AttributeNameNotValid', {'Message' => $PreDefAttributeAttributeName . ' \'' . $PreDefAttributeAttributeValue . '\' is not a valid attribute name. \''} ) }

        #raise error if not correct
        Error('AttributeValueVerificationError', {
            'Message' => 'wrong parameter type for predefattribute',
            'Object' => $Alias,
            'Attribute' => $PreDefAttributeAttributeName,
            'Value' => $PreDefAttributeAttributeValue,
            'ParameterType' => $ParameterType
        } ) if ($RaiseError);
    }
}

#========================================================================================
# function     getPreDefAttributeAttributeNames
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributeNames = $self->getPreDefAttributeAttributeNames();
#----------------------------------------------------------------------------------------
# description  get all existing predefattribute parameter names
#----------------------------------------------------------------------------------------
# return       $aAttributeNames | allexisting predefattribute parameter names  | ref.array
#========================================================================================
sub getPreDefAttributeAttributeNames {
    my $self = shift;

    my @Return = keys %{$self->{'PreDefAttributeAttributeParameters'}};

    return \@Return;

}

#========================================================================================
# function     _handleProductTypeAttributesAdd
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->_handleProductTypeAttributesAdd($ahProductTypeAttributesInfo, Shop);
#----------------------------------------------------------------------------------------
# description  Performs parameter validation and inserts the given product types into the
#               database.
#----------------------------------------------------------------------------------------
# input        $ahProductTypeAttributesInfo | list of attributes to add | ref.array.hash
# input        $Shop | shop that contains the product types | object
#----------------------------------------------------------------------------------------
# return       $Success | returns true if everything is correct | boolean
#========================================================================================
sub _handleProductTypeAttributesAdd{
    my $self = shift;
    my ($ahProductTypeAttributesInfo, $Shop) = @_;

    #get info
    my $ProductAttributeClass = LoadClassByAlias('ProductAttribute');
    my $ShopID = $Shop->id();
    my $ProductAttributeClassID = $ProductAttributeClass->id;
    my $ShowInOwnTabID = $ProductAttributeClass->folder('Attributes')->child('ShowInOwnTab')->id;
    my $SQLStatementObject;
    my $SQLStatementAttribute;
    my $SQLStatementBooleanAttribute;
    my $SQLStatementFacetedSearchAttribute;
    my $ahAliasObjectIDCache;
    my $haResult;
    my $aRollbackRelevantIDs;
    my $TemporaryObjectID;

    # holds attribrutes that have predefattributes
    my $ahSelectionAttributes;


    # do parameter verification and transformation
    foreach my $hProductTypeAttributeInfo (@ $ahProductTypeAttributesInfo){

        #check if parameter verification is enabled
        if ($self->isParamVerificationActivated()){

            #verify content
            $self->verifyContent($hProductTypeAttributeInfo);
        }

        #transform parameters
        $self->transformInputParameters($hProductTypeAttributeInfo);
    }


    #lock tables and execute code while locked
    $self->dbi->lockTables(
        [
            { 'table' => 'object', 'mode' => 'WRITE'},
            { 'table' => 'attribute', 'mode' => 'WRITE'},
            { 'table' => 'facetedsearchattribute', 'mode' => 'WRITE'},
            { 'table' => 'booleanattribute', 'mode' => 'WRITE'},
        ],
        sub {
            #initialize variables
            my $Counter = 0;
            my @Row = $self->dbh->selectrow_array('SELECT AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = \'object\'');
            my $NextID = $Row[0];
            my $aSQLParametersObject;
            my $aSQLParametersAttribute;
            my $aSQLParametersFacetedSearchAttribute = [];
            my $aSQLParametersBooleanAttribute = [];


            #prepare sql statement for object table
            $SQLStatementObject = 'INSERT INTO object (' .
                'objectid, ' .
                'classid, ' .
                'parentid, ' .
                'alias, ' .
                'position, ' .
                'guid, ' .
                'inherit, ' .
                'creationdate, ' .
                'siteid' .
                ")\nVALUES";


            #prepare sql statement for attribute table
            $SQLStatementAttribute = 'INSERT INTO attribute(' .
                'attributeid, ' .
                'attributeclassid, ' .
                'type, ' .
                'length, ' .
                'package, ' .
                'isarray, ' .
                'isreadonly, ' .
                'isexportable, ' .
                'iscacheable, ' .
                'isobject, ' .
                'ismandatory, ' .
                'isuserdefined, ' .
                'ishtml, ' .
                'isvisible, ' .
                'exportlevel' .
                ")\nVALUES";


            #prepare sql statement for facetedsearchattribute table
            #this is needed to save the IsSearchFilter attribute value
            #for a product type attribute
            $SQLStatementFacetedSearchAttribute = 'INSERT INTO facetedsearchattribute(' .
                'attributeid, ' .
                'issearchfilter' .
                ")\nVALUES";


            #prepare sql statement for booleanattribute table
            #this is needed to save the ShowInOwnTab attribute value
            #for a product type attribute
            $SQLStatementBooleanAttribute = 'INSERT INTO booleanattribute(' .
                'objectid, ' .
                'attributeid, ' .
                'value' .
                ")\nVALUES";


            foreach my $hProductTypeAttributeInfo (@ $ahProductTypeAttributesInfo){

                #get all needed infos
                $TemporaryObjectID = $NextID + $Counter;

                #remember id's for rollback
                push(@$aRollbackRelevantIDs, $TemporaryObjectID);

                #remember result
                push(@$ahAliasObjectIDCache, {
                    'product_type' => $hProductTypeAttributeInfo->{'product_type'},
                    'alias' => $hProductTypeAttributeInfo->{'alias'},
                    'objectid' => $TemporaryObjectID,
                });


                #### sql statement for object table
                $SQLStatementObject .= "\n(";

                #objectd
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$TemporaryObjectID);

                #classid
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$ProductAttributeClassID);

                #parentid
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$hProductTypeAttributeInfo->{'parentid'});

                #alias
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$hProductTypeAttributeInfo->{'alias'});

                #positionclear
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$hProductTypeAttributeInfo->{'position'});

                #guid
                $SQLStatementObject .= 'UPPER(UUID()), ';

                #isinherit
                $SQLStatementObject .= '1, ';

                #creationdate
                $SQLStatementObject .= 'NOW(), ';

                #shopid
                $SQLStatementObject .= '?),';
                push(@$aSQLParametersObject,$ShopID);


                ###sql statement for attribute table
                $SQLStatementAttribute .= "\n(";

                my $AttributeType = $hProductTypeAttributeInfo->{'type'};
                my $ProductAttributeTypes = PRODUCT_ATTRIBUTE_TYPES();
                my $hProductAttributeTypeInfo = $ProductAttributeTypes->{$AttributeType};

                Error('AttributeTypeNotDefined', {
                    'Message' => "Attribute type $AttributeType is not defined",
                    'Type' => $AttributeType,
                    'Attribute' => $hProductTypeAttributeInfo->{'alias'},
                }) unless defined($hProductAttributeTypeInfo);

                # remember attributes with PreDefAttributes
                if ($self->_isSelectionAttribute($AttributeType)) {
                    $hProductTypeAttributeInfo->{'objectid'} = $TemporaryObjectID;
                    push(@$ahSelectionAttributes, $hProductTypeAttributeInfo);
                }

                # do we have to store the IsSearchFilter attribute?
                if ($AttributeType eq 'PreDefString' || $AttributeType eq 'PreDefLocalizedString' ||
                    $AttributeType eq 'PreDefMultiString' || $AttributeType eq 'PreDefMultiLocalizedString' ||
                    $AttributeType eq 'Integer' || $AttributeType eq 'Float' ||
                    $AttributeType eq 'Boolean') {

                    $SQLStatementFacetedSearchAttribute .= "\n(";

                    #attributeid
                    $SQLStatementFacetedSearchAttribute .= '?, ';
                    push(@$aSQLParametersFacetedSearchAttribute,$TemporaryObjectID);

                    #issearchfilter
                    $SQLStatementFacetedSearchAttribute .= '?),';
                    push(@$aSQLParametersFacetedSearchAttribute,$hProductTypeAttributeInfo->{'is_search_filter'});

                }

                #do we have to store the ShowInOwnTab attribute?
                elsif ($AttributeType eq 'String' || $AttributeType eq 'LocalizedString') {

                    $SQLStatementBooleanAttribute .= "\n(";

                    #objectid
                    $SQLStatementBooleanAttribute .= '?, ';
                    push(@$aSQLParametersBooleanAttribute,$TemporaryObjectID);

                    #attributeid
                    $SQLStatementBooleanAttribute .= '?, ';
                    push(@$aSQLParametersBooleanAttribute,$ShowInOwnTabID);

                    #value
                    $SQLStatementBooleanAttribute .= '?),';
                    push(@$aSQLParametersBooleanAttribute,$hProductTypeAttributeInfo->{'show_in_own_tab'});

                }

                #attributeid
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$TemporaryObjectID);

                #attributeclassid
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductTypeAttributeInfo->{'attributeclassid'});

                #type
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$AttributeType);

                #length
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductTypeAttributeInfo->{'length'});

                #package
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductAttributeTypeInfo->{'Package'});

                #isarray
                my $IsArray = $self->_isArray($AttributeType);
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$IsArray);

                #isreadonly -> 0
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,0);

                #isexportable -> 1
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,1);

                #iscacheable -> 1
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,1);

                #isobject
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductAttributeTypeInfo->{'IsObject'});

                #ismandatory -> 0
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,1);

                #isuserdefined -> 1
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,1);

                #ishtml
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductTypeAttributeInfo->{'is_html'});

                #isvisible
                $SQLStatementAttribute .= '?, ';
                push(@$aSQLParametersAttribute,$hProductTypeAttributeInfo->{'is_visible'});

                #exportlevel -> 1
                $SQLStatementAttribute .= '?),';
                push(@$aSQLParametersAttribute,1);


                #increase counter
                $Counter ++;

            }

            chop($SQLStatementObject);
            chop($SQLStatementAttribute);
            chop($SQLStatementFacetedSearchAttribute);
            chop($SQLStatementBooleanAttribute);


            #start transaction
            Transaction(
                sub{
                    $self->dbi()->execute(
                        $SQLStatementObject,
                        $aSQLParametersObject
                    );

                    $self->dbi()->execute(
                        $SQLStatementAttribute,
                        $aSQLParametersAttribute
                    );

                    $self->dbi()->execute(
                        $SQLStatementFacetedSearchAttribute,
                        $aSQLParametersFacetedSearchAttribute
                    ) if (@$aSQLParametersFacetedSearchAttribute > 0);

                    $self->dbi()->execute(
                        $SQLStatementBooleanAttribute,
                        $aSQLParametersBooleanAttribute
                    ) if (@$aSQLParametersBooleanAttribute > 0)
                },
                #rollback
                sub {
                    #rollback code
                    $self->_deleteProductTypeAttributesFromDB($aRollbackRelevantIDs);
                }
            );
        }
    );

    #prepare result
    foreach my $hAliasObjectIDCacheEntry (@$ahAliasObjectIDCache){
        my $ProductTypeGUID = $hAliasObjectIDCacheEntry->{'product_type'};
        my $AttributeAlias = $hAliasObjectIDCacheEntry->{'alias'};
        my $AttributeObjectID = $hAliasObjectIDCacheEntry->{'objectid'};

        push(@{$haResult->{$ProductTypeGUID}}, {
            'alias' => $AttributeAlias,
            'guid' => LoadObject($AttributeObjectID)->get('GUID'),
            'epages6_objectid' => $AttributeObjectID,
        }) if $self->_validProductTypeAttributeDBCheck($AttributeObjectID);
    }


    # for attributes with predefattributes, insert the predefattributes
    my $hPreDefAttributesResult;
    eval {
        $hPreDefAttributesResult = $self->_insertPreDefAttributes($ahSelectionAttributes, $Shop) if defined($ahSelectionAttributes);
    };

    # if something goes wrong, we have to rollbacck all attributes
    if(ExistsError) {
        my $MyError = GetError();
        Transaction(
            sub {
                $self->_deleteProductTypeAttributesFromDB($aRollbackRelevantIDs);
            }
        );
        Error($MyError);
    }

    # merge predefattribute ids with attribute ids
    foreach my $ProductTypeGUID (keys %$haResult) {
        foreach my $hAttribute (@{$haResult->{$ProductTypeGUID}}) {
            my $AttributeAlias = $hAttribute->{'alias'};
            my $aPreDefAttributes = $hPreDefAttributesResult->{$ProductTypeGUID}->{$AttributeAlias};
            $hAttribute->{'pre_def_attributes'} = $aPreDefAttributes if defined($aPreDefAttributes);
        }
    }

    return $haResult;
}

#========================================================================================
# function     _handleProductTypeAttributesGet
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aResult = $self->_handleProductTypeAttributesGet($aProductTypeIDs);
#----------------------------------------------------------------------------------------
# description  Gets product type attribute contents from the database.
#----------------------------------------------------------------------------------------
# input        $aProductTypeIDs | IDs of product types to get the attributes | ref.array
# return       $haResult | product type attributes 
#               keys
#               <ul>
#                   <li>product type IDs</li>
#               </ul>
#               values
#               <ul>
#                   <li>arrays of attributes</li>
#                   <li>each attribute is a hash itsself</li>
#               </ul> | ref.hash.array
#========================================================================================
sub _handleProductTypeAttributesGet {
    my $self = shift;
    my ($aProductTypeIDs) = @_;  # IDs of the product types
    my $aResult;
    my $aAttributeIDs;

    # Get ID of the attribute 'showinowntab', which is an optional boolean attribute for product types.
    # The attribute s an object and therefore has an objectid. It has also an entry in the table 'attribute', where
    # the attributeid = objectid. We need the objectid / attributeid to set this attribute for our product type attributes
    my $ProductAttributeClass = LoadClassByAlias('ProductAttribute');
    my $ShowInOwnTabID = $ProductAttributeClass->folder('Attributes')->child('ShowInOwnTab')->id;

    #prepare sql statement for product type attributes
    my $SQLStatement = 'SELECT ' .
        'o.guid, ' .
        'o.alias, ' .
        'o.position, ' .
        'a.type, ' .
        'a.length, ' .
        'a.ishtml, ' .
        'a.isvisible, ' .
        'b.value, ' .   # value of attribute ShowInOwnTab
        'f.issearchfilter, ' .  # attribute that is needed for faceted search. It is saved in a separate table
        'a.attributeclassid, ' .
        'o.objectid ' .

        'FROM ' . 
        '(object o INNER JOIN attribute a ' . 
        'ON o.objectid = a.attributeid) ' . 
        'LEFT JOIN booleanattribute b ' .   # booleanattribute contains value for ShowInOwnTab
        'ON o.objectid = b.objectid ' . 
        'LEFT JOIN facetedsearchattribute f ' . # facetedsearchattribute contains value for the parameter 'is_search_filter'
        'ON o.objectid = f.attributeid ' .
        "WHERE (b.attributeid IS NULL OR b.attributeid = ?) " .
        'AND a.attributeclassid IN(';

    #build sql statement
    foreach my $ID (@$aProductTypeIDs){
        $SQLStatement .= '?,';
    }

    #finalize statement
    chop($SQLStatement);
    $SQLStatement .= ')';

    # prepare SQL parameters
    my $aSQLParameters;
    push(@$aSQLParameters, $ShowInOwnTabID);
    push(@$aSQLParameters, @$aProductTypeIDs);


    #execute sql statements
    my $aDBResult = $self->dbi()->execute($SQLStatement, $aSQLParameters);


    #for each result row
    foreach my $Row (@$aDBResult){

        #placeholder
        my $hRowResult;

        #save content from db result
        $hRowResult->{'guid'} = $Row->[0];
        $hRowResult->{'alias'} = $Row->[1];
        $hRowResult->{'position'} = $Row->[2];
        $hRowResult->{'type'} = $Row->[3];
        $hRowResult->{'length'} = $Row->[4];
        $hRowResult->{'is_html'} = $Row->[5];
        $hRowResult->{'is_visible'} = $Row->[6];
        $hRowResult->{'show_in_own_tab'} = $Row->[7];
        $hRowResult->{'is_search_filter'} = $Row->[8];
        $hRowResult->{'product_type_id'} = $Row->[9];
        $hRowResult->{'attribute_id'} = $Row->[10];

        # remember object ids of the attributes
        # we need them to get their predef attributes
        push(@$aAttributeIDs, $Row->[10]);

        #transform parameters to valid JSON values
        $self->transformOutputParametersGet($hRowResult);

        # Remember result
        push(@$aResult, $hRowResult);
    }

    #get predef attributes
    my $hPreDefAttributesResult = $self->_getPreDefAttributes($aAttributeIDs);

    # merge predefattributes with their attributes and build final result
    my $haResult;

    foreach my $hProductTypeAttribute (@$aResult) {
        my $ProductTypeAttributeID = $hProductTypeAttribute->{'attribute_id'};

        if(defined($hPreDefAttributesResult)) {
            my $aProductTypePreDefAttributes = $hPreDefAttributesResult->{$ProductTypeAttributeID};

            $hProductTypeAttribute->{'pre_def_attributes'} = $aProductTypePreDefAttributes if defined($aProductTypePreDefAttributes);
        }
        
        # now build the final result hash
        # this is a hash with product type IDs as keys and an array of their attributes as values
        my $ProductTypeID = $hProductTypeAttribute->{'product_type_id'};

        # remove attribute id and product type id from final result;
        # these were only needed during processing
        delete $hProductTypeAttribute->{'product_type_id'};
        delete $hProductTypeAttribute->{'attribute_id'};

        push(@{$haResult->{$ProductTypeID}}, $hProductTypeAttribute);
    }

    # return result

    # result has the format
    # {
    #     $ProductTypeID => [
    #       {
    #         'alias' => $AttributeAlias,
    #         'guid' => $AttributeType,
    #          ...
    #         'pre_def_attributes' => [
    #           {
    #           ...
    #           }]
    #       }]
    #     ]
    # }
    return $haResult;
}

#========================================================================================
# function     _validProductTypeAttributeDBCheck
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->_validProductTypeAttributeDBCheck($ObjectID);
#----------------------------------------------------------------------------------------
# description  checks if the object exist in the tables tables object and attribute
#               check directly into the data base
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | integer
# return       $Success | returns true if everything is correct | boolean
#========================================================================================
sub _validProductTypeAttributeDBCheck {
    my $self = shift;
    my ($ObjectID) = @_;

    #execute on data base
    my @Row = $self->dbh->selectrow_array(
        'SELECT o.alias FROM object AS o ' .
        'INNER JOIN attribute AS a ON o.objectid = a.attributeid ' .
        'WHERE objectid=?',
        {},
        $ObjectID
    );

    #check result
    return 1 if (scalar @Row > 0);
    return 0;
}

#========================================================================================
# function     _isArray
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->_isArray($AttributeType);
#----------------------------------------------------------------------------------------
# description  Returns the value for the 'isarray' attribute of the 'attribute' table
#----------------------------------------------------------------------------------------
# input        $AttributeType | attribute type | string
# return       $Success | the desired value for the isarray | boolean
#========================================================================================
sub _isArray {
    my $self = shift;
    my ($AttributeType) = @_;

    return ($AttributeType eq 'PreDefMultiString' || $AttributeType eq 'PreDefLocalizedMultiString')? 1 : 0
}

sub _isSelectionAttribute {
    my $self = shift;
    my ($AttributeType) = @_;

    return $AttributeType =~ /^PreDef/;
}

#========================================================================================
# function     _insertPreDefAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->_insertPreDefAttributes($ahSelectionAttributes, $Shop);
#----------------------------------------------------------------------------------------
# description  checks if the object exist in all three tables class, object, producttype
#               check directly into the data base
#----------------------------------------------------------------------------------------
# input        $ahSelectionAttributes | attributes with predefattributes | ref.array.hash
# input        $Shop | shop of the product type | object
# return       $hResult | hash of added predefattributes and their IDs
#               <ul>
#                 <li>Keys: GUIDs of product types</li>
#                 <li>Values: Hashes with attribute aliases as keys and array predefattributes as value</li>
#               </ul>
#========================================================================================
sub _insertPreDefAttributes {
    my $self = shift;
    my ($ahSelectionAttributes, $Shop) = @_;

    my $ShopID = $Shop->id;

    my $SQLStatementObject;
    my $SQLStatementPreDefAttribute;
    my $aRollbackRelevantIDs = [];
    my $aPreDefAttributesObjectIDs = [];
    my $PreDefAttributeClassID = LoadClassByAlias('PreDefAttribute')->id;
    my $ProductAttributeClassID = LoadClassByAlias('ProductAttribute')->id;
    my $ObjectClassID = LoadClassByAlias('Object')->id;
    my $ahAliasObjectIDCache;

    # do parameter verification and transformation
    foreach my $hProductTypeAttributeInfo (@ $ahSelectionAttributes){
        my $ahPreDefAttributes = $hProductTypeAttributeInfo->{'pre_def_attributes'};

        foreach my $hPreDefAttribute (@ $ahPreDefAttributes) {

            #check if parameter verification is enabled
            if ($self->isParamVerificationActivated()){

                #verify content
                $self->verifyPreDefAttributeContent($hPreDefAttribute);
            }

            #transform parameters
            $self->transformInputParameters($hPreDefAttribute);
        }
    }

    #lock tables and execute code while locked
    $self->dbi->lockTables(
        [
            { 'table' => 'object', 'mode' => 'WRITE'},
            { 'table' => 'predefattribute', 'mode' => 'WRITE'},
        ],
        sub {
            #initialize variables
            my $Counter = 0;
            my @Row = $self->dbh->selectrow_array('SELECT AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = \'object\'');
            my $NextID = $Row[0];
            my $aSQLParametersObject;
            my $aSQLParametersPreDefAttribute;


            #prepare sql statement for object table
            $SQLStatementObject = 'INSERT INTO object (' .
                'objectid, ' .
                'classid, ' .
                'parentid, ' .
                'alias, ' .
                'position, ' .
                'guid, ' .
                'inherit, ' .
                'creationdate, ' .
                'siteid' .
                ")\nVALUES";

            #prepare sql statement for predefattribute table
            $SQLStatementPreDefAttribute = 'INSERT INTO predefattribute (' .
                'predefattributeid, ' .
                'attributeid, ' .
                'predefclassid' .
                ")\nVALUES";


            foreach my $hProductTypeAttributeInfo (@ $ahSelectionAttributes){
                my $TemporaryObjectID = $NextID + $Counter;

                ### sql statement for object table: insert predefattributes object
                $SQLStatementObject .= "\n(";

                #objectid
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$TemporaryObjectID);

                #classid
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$ObjectClassID);

                #parentid
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,$hProductTypeAttributeInfo->{'objectid'});

                #alias
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,'PreDefAttributes');

                #position
                $SQLStatementObject .= '?, ';
                push(@$aSQLParametersObject,0);

                #guid
                $SQLStatementObject .= 'UPPER(UUID()), ';

                #isinherit
                $SQLStatementObject .= '1, ';

                #creationdate
                $SQLStatementObject .= 'NOW(), ';

                #shopid
                $SQLStatementObject .= '?),';
                push(@$aSQLParametersObject,$ShopID);

                #remember id's for rollback
                push(@$aPreDefAttributesObjectIDs, $TemporaryObjectID);

                $hProductTypeAttributeInfo->{'pre_def_attributes_id'} = $TemporaryObjectID;

                $Counter++;


                my $ahPreDefAttributes = $hProductTypeAttributeInfo->{'pre_def_attributes'};

                # insert every predefattribute as object
                foreach my $hPreDefAttribute (@ $ahPreDefAttributes) {

                    #get all needed infos
                    $TemporaryObjectID = $NextID + $Counter;

                    #remember id's for rollback
                    push(@$aRollbackRelevantIDs, $TemporaryObjectID);

                    #remember result
                    push(@$ahAliasObjectIDCache, {
                        'product_type' => $hProductTypeAttributeInfo->{'product_type'},
                        'attribute' => $hProductTypeAttributeInfo->{'alias'},
                        'alias' => $hPreDefAttribute->{'alias'},
                        'objectid' => $TemporaryObjectID,
                    });

                    #### sql statement for object table
                    $SQLStatementObject .= "\n(";

                    #objectid
                    $SQLStatementObject .= '?, ';
                    push(@$aSQLParametersObject,$TemporaryObjectID);

                    #classid
                    $SQLStatementObject .= '?, ';
                    push(@$aSQLParametersObject,$PreDefAttributeClassID);

                    #parentid
                    $SQLStatementObject .= '?, ';
                    push(@$aSQLParametersObject,$hProductTypeAttributeInfo->{'pre_def_attributes_id'});

                    #alias
                    $SQLStatementObject .= '?, ';
                    push(@$aSQLParametersObject,$hPreDefAttribute->{'alias'});

                    #position
                    $SQLStatementObject .= '?, ';
                    push(@$aSQLParametersObject,$hPreDefAttribute->{'position'});

                    #guid
                    $SQLStatementObject .= 'UPPER(UUID()), ';

                    #isinherit
                    $SQLStatementObject .= '1, ';

                    #creationdate
                    $SQLStatementObject .= 'NOW(), ';

                    #shopid
                    $SQLStatementObject .= '?),';
                    push(@$aSQLParametersObject,$ShopID);


                    ###sql statement for predefattribute table
                    $SQLStatementPreDefAttribute .= "\n(";

                    #predefattributeid
                    $SQLStatementPreDefAttribute .= '?, ';
                    push(@$aSQLParametersPreDefAttribute,$TemporaryObjectID);

                    #attributeid
                    $SQLStatementPreDefAttribute .= '?, ';
                    push(@$aSQLParametersPreDefAttribute,$hProductTypeAttributeInfo->{'objectid'});

                    #predefclassid
                    $SQLStatementPreDefAttribute .= '?),';
                    push(@$aSQLParametersPreDefAttribute,$ProductAttributeClassID);


                    #increase counter
                    $Counter ++;
                }
            }

            chop($SQLStatementObject);
            chop($SQLStatementPreDefAttribute);


            #start transaction
            Transaction(
                sub{
                    $self->dbi()->execute(
                        $SQLStatementObject,
                        $aSQLParametersObject
                    );
                    $self->dbi()->execute(
                        $SQLStatementPreDefAttribute,
                        $aSQLParametersPreDefAttribute
                    );

                },
                #rollback
                sub {
                    #rollback code
                    foreach my $ID (@$aRollbackRelevantIDs){
                        #rollback for predefattribute - delete all the added predefattribute
                        $self->dbi()->do(
                            'DELETE FROM predefattribute WHERE predefattributeid=?',
                            [$ID]
                        );

                        #rollback for object
                        $self->dbi()->do(
                            'DELETE FROM object WHERE objectid=?',
                            [$ID]
                        );
                    }

                    #rollback for 'pre_def_attributes' container objects
                    foreach my $ID (@$aPreDefAttributesObjectIDs){
                        #rollback for object
                        $self->dbi()->do(
                            'DELETE FROM object WHERE objectid=?',
                            [$ID]
                        );
                    }
                }
            );
        }
    );

    my $hResult;

    # iterate over list of added objectids of predefattributes
    foreach my $hAliasObjectIDCacheEntry (@$ahAliasObjectIDCache){
        # get GUID for the product type of this predefattribute
        my $ProductTypeGUID = $hAliasObjectIDCacheEntry->{'product_type'};

        # get alias of the attribute of this predefattribute
        my $AttributeAlias = $hAliasObjectIDCacheEntry->{'attribute'};

        # get alias of the predefattribute
        my $PreDefAttributeAlias = $hAliasObjectIDCacheEntry->{'alias'};
        my $PreDefAttributeObjectID = $hAliasObjectIDCacheEntry->{'objectid'};

        # result has the format
        # {
        #     $ProductTypeGUID => {
        #         $AttributeAlias => [{
        #             'alias' => $PreDefAttributeAlias,
        #             'guid' => $PreDefAttributeGUID,
        #             'epages6_objectid' => $PreDefAttributeObjectID,
        #         }]
        #     }
        # }
        push(@{$hResult->{$ProductTypeGUID}->{$AttributeAlias}}, {
            'alias' => $PreDefAttributeAlias,
            'guid' => LoadObject($PreDefAttributeObjectID)->get('GUID'),
            'epages6_objectid' => $PreDefAttributeObjectID,
        }) if $self->_validProductTypePreDefAttributeDBCheck($PreDefAttributeObjectID);
    }
    return $hResult;
}

#========================================================================================
# function     _getPreDefAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $haResult = $self->_getPreDefAttributes($aProductTypeAttributeIDs);
#----------------------------------------------------------------------------------------
# description  Gets predefattributes of the given product type attributes
#----------------------------------------------------------------------------------------
# input        $aProductTypeAttributeIDs | IDs of product type attributes to get the predefattributes | ref.array
# return       $haResult | predefattributes
#               <ul>
#                   <li> hash with attribute IDs as keys </li>
#                   <li> values are arrays of the predefattributes </li>
#               </ul> | ref.hash.array
#========================================================================================
sub _getPreDefAttributes {
    my $self = shift;
    my ($aProductTypeAttributeIDs) = @_;
    my $haResult;

    # we have nothing to do if there are no attributes
    return if (not(defined($aProductTypeAttributeIDs)) or scalar(@$aProductTypeAttributeIDs) == 0);

    #prepare SQL statement
    my $SQLStatement = 'SELECT ' . 
        'p.attributeid, ' .
        'o.alias, ' .
        'o.guid, ' .
        'o.position ' .

        'FROM predefattribute p ' .
        'JOIN object o ' .
        'ON p.predefattributeid = o.objectid ' .
        'WHERE p.attributeid in(';

    # build sql statement
    # We do not care if an attribute really has predefattributes or not, this is handled by SQL.
    # If an attribute has no predefattributes, there are simply no result sets for this ID.
    foreach my $ID (@$aProductTypeAttributeIDs){
        $SQLStatement .= '?,';
    }

    #finalize statement
    chop($SQLStatement);
    $SQLStatement .= ')';


    #execute sql statements
    my $aDBResult = $self->dbi()->execute($SQLStatement, $aProductTypeAttributeIDs);


    #for each result row
    foreach my $Row (@$aDBResult){

        #placeholder
        my $hRowResult;

        #save content from db result
        $hRowResult->{'alias'} = $Row->[1];
        $hRowResult->{'guid'} = $Row->[2];
        $hRowResult->{'position'} = $Row->[3];


        #transform parameters to valid JSON values
        $self->transformOutputParametersGet($hRowResult);

        # Remember result.
        # This is a hash with attribute IDs as keys and an arrays of their
        # predefattributes as values.
        my $ProductTypeAttributeID = $Row->[0];
        push(@{$haResult->{$ProductTypeAttributeID}}, $hRowResult);
    }

    return $haResult;
}

#========================================================================================
# function     _validProductTypePreDefAttributeDBCheck
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = $self->_validProductTypePreDefAttributeDBCheck($ObjectID);
#----------------------------------------------------------------------------------------
# description  checks if the object exist in tables object and predefattribute
#               check directly into the data base
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | integer
# return       $Success | returns true if everything is correct | boolean
#========================================================================================
sub _validProductTypePreDefAttributeDBCheck {
    my $self = shift;
    my ($ObjectID) = @_;

    my @Row = $self->dbh->selectrow_array(
        'SELECT o.alias FROM object AS o ' .
        'INNER JOIN predefattribute AS p ON o.objectid = p.predefattributeid ' .
        'WHERE objectid=?',
        {},
        $ObjectID
    );

    #check result
    return 1 if (scalar @Row > 0);
    return 0;
}

#========================================================================================
# function     _deleteProductTypeAttributesFromDB
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->_deleteProductTypeAttributesFromDB($aRollbackRelevantIDs);
#----------------------------------------------------------------------------------------
# description  performes a rollback for the given attribute ids
#----------------------------------------------------------------------------------------
# input        $aRollbackRelevantIDs | ids of product types to roll back | ref.array
#========================================================================================
sub _deleteProductTypeAttributesFromDB {
    my $self = shift;
    my ($aRollbackRelevantIDs) = @_;

    foreach my $ID (@$aRollbackRelevantIDs){
        #rollback for booleanattribute
        $self->dbi()->do(
            'DELETE FROM booleanattribute WHERE objectid=?',
            [$ID]
        );

        #rollback for facetedsearchattribute
        $self->dbi()->do(
            'DELETE FROM facetedsearchattribute WHERE attributeid=?',
            [$ID]
        );

        #rollback for attribute
        $self->dbi()->do(
            'DELETE FROM attribute WHERE attributeid=?',
            [$ID]
        );

        #rollback for child objects
        $self->dbi()->do(
            'DELETE FROM object WHERE parentid=?',
            [$ID]
        );

        #rollback for object
        $self->dbi()->do(
            'DELETE FROM object WHERE objectid=?',
            [$ID]
        );
    }
}

1;
