#========================================================================================
# package      DE_EPAGES::Order::API::PickupTokenValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Tools for PickupToken validation.
#========================================================================================
package DE_EPAGES::Order::API::PickupTokenValidation;

use DE_EPAGES::Presentation::API::DereferrerToken qw (ValidateDereferrerToken);
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use URI::QueryParam;
use URI::Escape;
use URI;

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Order::API::PickupTokenValidation
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}



#========================================================================================
# function     getUrlForPickupTokenValidation
# state        public
#----------------------------------------------------------------------------------------
# syntax       getUrlForPickupTokenValidation( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the request URL without the PickupToken parameter.
#----------------------------------------------------------------------------------------
# input        $Servlet | The request servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       The request url | String
#========================================================================================
sub getUrlForPickupTokenValidation {
    my $self = shift;
    my ($Servlet) = @_;

    my $Host = $Servlet->params('HTTP_HOST');
    my $Path = $Servlet->request()->uri();
    my $IsSSL = $Servlet->request()->header('x-epages-needs-ssl');
    my $Protocol = $IsSSL ? 'https' : 'http';
    my $Url = "$Protocol://$Host$Path";

    my $Uri = URI->new($Url);
    $Uri->query_param_delete('PickupToken');
    my $UrlWithoutPickupToken = uri_unescape($Uri->as_string());
    $UrlWithoutPickupToken =~ s/@/%40/g;
    return $UrlWithoutPickupToken;
}



#========================================================================================
# function     checkPickupToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       checkPickupToken( $Token, $URL );
#----------------------------------------------------------------------------------------
# description  Checks if the pickup token belongs to the given url.
#----------------------------------------------------------------------------------------
# input        $Token | the pickup token | String
# input        $URL | the request url | String
# return       is token valid | boolean
#========================================================================================
sub checkPickupToken {
    my $self = shift;
    my ($Token, $URL) = @_;

    my $IsTokenValid = 0;
    eval {
        $self->_ValidateDereferrerToken($Token, $URL);
        $IsTokenValid = 1;
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
    }
    return $IsTokenValid;
}



#========================================================================================
# function     _ValidateDereferrerToken
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ValidateDereferrerToken($DereferrerToken, $URL);
#----------------------------------------------------------------------------------------
# description  Wrapper for function
#----------------------------------------------------------------------------------------
# input        $Token | valid base64 encoded security token | String
# input        $URL   | url | string
#========================================================================================
sub _ValidateDereferrerToken {
    my $self = shift;
    my ($Token, $URL) = @_;
    ValidateDereferrerToken($Token, $URL);
}

1;