#========================================================================================
# package      DE_EPAGES::Order::API::Resource::Cart
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for a single cart.
#========================================================================================
package DE_EPAGES::Order::API::Resource::Cart;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::Formatter;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Presentation::API::DereferrerToken qw (GenerateDereferrerToken);
use DE_EPAGES::Order::API::LineItemContainerData;
use DE_EPAGES::Order::API::PickupTokenExpiration;
use DE_EPAGES::Order::API::BasketPatch;
use DE_EPAGES::Shop::API::Constants qw (TAX_MODEL_GROSS);
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToJava GetAddressAttributeNames );
use feature 'switch';

use strict;



sub service {
    return LoadObjectByPath('/RestServices/Cart');
}



sub class {
    return 'Basket';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
        PATCH => [
             ['patch', 'application/vnd.epages.v1+json'],
             ['patch', 'application/json'],
         ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/carts/([^/]+)/?$|;
    return $1;
}



sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    given( $ActionName ) {
        when( 'get' ) {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $IncludeTax = $hQuery->{'includeTax'}->[0];
            return [ $Object, $IncludeTax ];
        }
        default {
            return [ $Object ];
        }
    }
}



sub get {
    my $self = shift;
    my ($Basket, $IncludeTax) = @_;

    if($self->_isBasketExpired($Basket)) {
        Error('ResourceNotFound');
    }

    $self->_validateLineItems($Basket);

    my $LocaleID = $Basket->container()->get('LocaleID');
    my $Locale = DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $Formatter = $self->_createFormatter($Basket->getSite, $LocaleID);

    my @Attributes = qw(GUID Path BillingAddress ShippingAddress LineItemContainer WebUrlSSL PickupToken MinOrderAmount IsMinOrderAmountReached);
    my $hBasket = $Basket->get(\@Attributes, $LanguageID);

    my @AddrAttributes = $self->_GetAddressAttributeNames();
    my $hBillingAddr = $hBasket->{BillingAddress} ? $hBasket->{BillingAddress}->get(\@AddrAttributes, $LanguageID) : undef;
    my $hShippingAddr = $hBasket->{ShippingAddress} ? $hBasket->{ShippingAddress}->get(\@AddrAttributes, $LanguageID) : undef;

    my $LineItemContainerData = DE_EPAGES::Order::API::LineItemContainerData->new($Basket->get('LineItemContainer'));
    my $hContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID, $IncludeTax);

    # create basket pickup token and url    
    my $BasketPickupTokenURL = $self->_getBasketPickupTokenURL($hBasket->{'WebUrlSSL'});
    my $BasketPickupToken    = $self->_generatePickupToken($BasketPickupTokenURL);
    my $CheckoutUrl          = $self->_createUrlWithToken($BasketPickupTokenURL, $BasketPickupToken);

    # create register session token and url
    my $RegisterSessionTokenURL = $self->_getRegisterSessionTokenURL($hBasket->{'WebUrlSSL'});
    my $RegisterSessionToken    = $self->_generatePickupToken($RegisterSessionTokenURL);
    my $RegisterSessionUrl      = $self->_createUrlWithToken($RegisterSessionTokenURL, $RegisterSessionToken);


    my $CurrencyId = $LineItemContainerData->getCurrencyOfLineItemContainer($LanguageID);
    my $MinCartValue = $self->_minCartValue($hBasket->{'MinOrderAmount'}, $LocaleID, $CurrencyId, $Formatter);
    my $MinCartCode = $self->_minCartValueCode($Basket);
    my $MinCart = $self->_minCart($MinCartValue, $Formatter->bool($hBasket->{'IsMinOrderAmountReached'}), $MinCartCode );
    my $aLinks = $self->_getCartLinks($Basket->getSite(), $hContainerData);
    return {
        'cartId'             => $hBasket->{GUID},
        'shippingAddress'    => $self->_ConvertAddressToJava( $hShippingAddr ),
        'billingAddress'     => $self->_ConvertAddressToJava( $hBillingAddr ),
        'checkoutUrl'        => $CheckoutUrl,
        'pickupToken'        => $BasketPickupToken,
        'registerSessionUrl' => $RegisterSessionUrl,
        'lineItemContainer'  => $hContainerData,
        'shippingData'       => $LineItemContainerData->getShippingData($LanguageID, $LocaleID, $IncludeTax),
        'paymentData'        => $LineItemContainerData->getPaymentData($LanguageID, $LocaleID, $IncludeTax),
        'minimumCart'        => $MinCart,
        'links'              => $aLinks,
    };
}

sub _getCartLinks{
    my $self = shift;
    my ($Shop, $hContainerData) = @_;

    my $aLinks = [];
    if($Shop->get('FeatureMaxValue')->{'Coupon'} && $Shop->get('hasRunningCouponCampaigns') && !defined $hContainerData->{'couponLineItem'}){
        push @$aLinks, {'rel' => 'coupon-add',
                        'href' => '',};
    }
    return $aLinks;
}

sub _minCart{
    my $self = shift;
    my ($Value, $Status, $Code) = @_;

    return {
        'amount'    => $Value,
        'status'    => $Status,
        'code'      => $Code,
    }
}

sub _minCartValueCode{
    my $self = shift;
    my ($Basket) = @_;

    my $AttributeName = $Basket->getSite()->get('MinOrderSubTotalAttributeName');
    given ($AttributeName){
        when('LineItemsSubTotal')
            {return 'SumProduct';}
        when('SalesDiscountsSubTotal')
            {return 'SumCoupon';}
        when('DiscountsSubTotal')
            {return 'SumDiscount';}
        when('ShippingSubTotal')
            {return 'SumDelivery';}
        when('GrandTotal')
            {return 'SumPayment';}
    }
}

sub _minCartValue{
    my $self = shift;
    my ($MinOrderAmount, $LocaleID, $CurrencyId, $Formatter) = @_;
    return undef unless $MinOrderAmount;
    return $Formatter->price($MinOrderAmount, $CurrencyId, TAX_MODEL_GROSS);
}

sub patch {
    my $self = shift;
    my ($Basket, $ahRequestBody) = @_;

    if($self->_isBasketExpired($Basket)) {
        Error('ResourceNotFound', {
            'GUID' => $Basket->guid(),
            'Class' => $self->class(),
            'Site' => $Basket->getSite()->pathString(),
        });
    }

    $self->_validateLineItems($Basket);

    my $Patch = DE_EPAGES::Order::API::BasketPatch->new($Basket);
    foreach my $hPatchOperation (@{$ahRequestBody}) {
        $Patch->patch($hPatchOperation);
    }

    my $LocaleID = $Basket->container()->get('LocaleID');
    return $self->get($Basket, $LocaleID);
}

sub _isBasketExpired {
    my $self = shift;
    my ($Basket) = @_;

    my $BasketCreationDate = $Basket->get('CreationDate');
    my $PickupTokenExpiration = DE_EPAGES::Order::API::PickupTokenExpiration->new();
    my $IsExpired = $PickupTokenExpiration->isExpired($BasketCreationDate);

    return $IsExpired;
}

sub _validateLineItems{
    my $self = shift;
    my ($Basket) = @_;

    my $LineItemContainer = $Basket->get('LineItemContainer');
    if (defined $LineItemContainer){
        return;
    }
    Error('Gone', {'reason' => 'This cart was already used for creating an order!'});

}


sub _getBasketPickupTokenURL {
    my $self = shift;
    my ( $WebUrl ) = @_;

    return "$WebUrl\&ChangeAction=PickupBasket";
}

sub _getRegisterSessionTokenURL {
    my $self = shift;
    my ( $WebUrl ) = @_;

    return "$WebUrl\&ChangeAction=JSONRegisterSession\&ViewAction=JSON";
}

sub _createUrlWithToken {
    my $self = shift;
    my ($URL, $Token) = @_;

    return "$URL\&PickupToken=$Token";
}


sub _generatePickupToken {
    my $self = shift;
    my ( $PickupTokenUrl ) = @_;
    return GenerateDereferrerToken($PickupTokenUrl);
}

sub _createFormatter {
    my $self = shift;
    my ($Shop, $LocaleID) = @_;
    return DE_EPAGES::Shop::API::Resource::Formatter->new($Shop, $LocaleID);
}

sub _ConvertAddressToJava {
    my $self = shift;
    my ( $hAddress ) = @_;
    return ConvertAddressToJava($hAddress);
}

sub _GetAddressAttributeNames {
    return GetAddressAttributeNames();
}

1;
