#========================================================================================
# package      DE_EPAGES::Order::Hooks::Servlet
# state        public
# description  Additional hook function for OnIsEp6HandledUnityRequest hook.
#========================================================================================
package DE_EPAGES::Order::Hooks::Servlet;

use strict;

use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;

#========================================================================================
# function     OnIsEp6HandledUnityRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnIsEp6HandledUnityRequest($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook IsEp6HandledUnityRequest.
#               The checkout process should be rendered by epages6 for any Unity shop.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - servlet object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnIsEp6HandledUnityRequest {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Object'};

    # skip if another hook function already acknowledges the request as a unity request handled by epages6
    return if $hParams->{'Values'}->{'IsEp6HandledUnityRequest'};

    my $ViewAction= $Servlet->params('ViewAction');

    # do not show empty basket error messages in epages6
    my $ChangeObjectID = $Servlet->params('ChangeObjectID') // $Servlet->params('ObjectID') ;

    my $Basket = defined $ChangeObjectID ? LoadObject($ChangeObjectID) : undef ;

    if(not defined $Basket) {
        if($Servlet->params('ObjectPath')){
            $Basket = LoadObjectByPath($Servlet->params('ObjectPath'));
        }
    }

    $Basket = undef unless (defined $Basket and $Basket->instanceOf('Basket'));

    # do we have an empty basket, return to unity sf
    if ($Basket){
        return unless $Basket->container();
        return unless scalar(@{$Basket->container()->get('LineItems')});
    }

    # test if the current ViewAction must be handled by epages6 (without the basket itself)
    # we only will display the whole checkout process on the epages6 site

    # handle ViewMultiCheckoutBasket as Unity Request. We have only a UNITY basket
    if (defined $ViewAction and $ViewAction eq 'ViewMultiCheckoutBasket'){
        $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 0;
        return;
    }

    # check for change actions epages6 needs to handle
    my @ChangeActions = $Servlet->form->value('ChangeAction');

    my $aChangeActionsHandledByEp6 = [
        'PickupBasket',
        'JSONRegisterSession',
        'SaveMultiCheckoutConfirmation',
    ];

    if(scalar @ChangeActions > 0) {
        foreach my $ChangeAction (@ChangeActions) {
            if($ChangeAction ~~ @{$aChangeActionsHandledByEp6}) {
                $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 1;
                return;
            }
        }
    }

    # handle Actions starting with ViewMultiCheckout*  and SF-ViewJQueryConfig
    if (defined $ViewAction and ($ViewAction =~ m/^ViewMultiCheckout/
                            or ($ViewAction eq 'SF-ViewJQueryConfig')
                            or ($ViewAction eq 'JSONViewResponse'))) {
        $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 1;
        return;
    }

    # try to get the order process data and validate ViewActions not handled before like actions from payments etc.
    # check if we have a  BasketOrderProcess and have ViewActions not starting with "ViewMultiCheckout"

    # first test if we have a Basket for loading the order process
    if ($Basket){
        my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
        if (defined $BasketOrderProcess){
            my $OrderProcessSteps = $BasketOrderProcess->steps;
            if (defined $OrderProcessSteps) {
                foreach my $Step (@$OrderProcessSteps) {
                    if ($ViewAction eq $Step->{'Step'}->get('ViewAction')->alias() ){
                        $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 1;
                        return;
                    }
                }
            }
        }
    }

    # we have an unknown ViewAction, UNITY is responsible
    $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 0;
    return;

}

1;
