#==============================================================================
# package      DE_EPAGES::Order::UI::Basket_Ext
# state        public
# description  UI class for Basket
#==============================================================================
package DE_EPAGES::Order::UI::Basket_Ext;
use base DE_EPAGES::Customer::UI::Shop;
use base Exporter;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Customer::API::Shop qw (GetCustomerForUser);
use DE_EPAGES::Object::API::Factory qw ( ExistsObject Transaction );
use DE_EPAGES::Order::API::LineItemContainer qw( SavePayment SaveShipping );
use DE_EPAGES::Presentation::API::Table::Session qw (ExistsSessionBySessionID GetInfoSessionBySessionID);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Order::API::PickupTokenExpiration;
use DE_EPAGES::Order::API::PickupTokenValidation;
use JSON qw ( to_json );

use strict;

our @EXPORT_OK = qw (
    _RegisterSession
);

#==============================================================================
# Event handlers
#==============================================================================
sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    my $Container = $Basket->container;
    my $Shop      = $Basket->getSite();
    my $Form      = $Servlet->form->form($Basket, 'Save');

    my $Quantities = $Form->{'LineItems'};
    foreach my $QuantityEntry (@$Quantities) {
        # ignore lineitems that are already removed
        next unless ExistsObject($QuantityEntry->{'LineItemID'});

        my $LineItem = $Shop->loadObject($QuantityEntry->{'LineItemID'}, 'LineItem');
        die 'invalid LineItem:'.$LineItem->id.' used in Basket:'.$Basket->id
         if $LineItem->get('LineItemContainer')->id != $Container->id;
        delete $QuantityEntry->{'LineItemID'};
        my $Quantity = $QuantityEntry->{'Quantity'};
        if ($Quantity == 0) {
            $LineItem->delete;
            next;
        } else {
            $LineItem->set({'IsChanged' => 0});
        }
        my %HookVars = ('LineItem' => $LineItem,
                        'Basket'=>$Basket,
                        'LineItemInfo' => $QuantityEntry,
                       );
        TriggerHook('BasketBeforeUpdateLineItemForm',\%HookVars);

        $LineItem->recalculate($Container, $QuantityEntry);
    }

    # set the CustomerComment
    $Basket->set({'CustomerComment' => $Form->{'CustomerComment'}})
     if defined $Servlet->form->value('CustomerComment');

    # set tax area
    my $TaxArea = $Container->get('TaxArea');
    if (defined $Form->{'TaxArea'} and $Basket->getSite()->get('CanSelectTaxAreaOnBasket')
     and (not defined $TaxArea or $TaxArea->id != $Form->{'TaxArea'})) {
        $Container->set({'TaxArea'=> $Shop->loadObject($Form->{'TaxArea'}, 'TaxArea', { AllowSystemSite => 1 })});
    }

    # set shipping and payment
    my $aWarnings = BlockWarnings(sub{
        my $NewShippingMethod = (defined $Form->{'ShippingMethod'} and ExistsObject($Form->{'ShippingMethod'}))
         ? $Shop->loadObject($Form->{'ShippingMethod'}, 'ShippingMethod') : undef;
        SaveShipping($Container, $NewShippingMethod);

        my $NewPaymentMethod = (defined $Form->{'PaymentMethod'} and ExistsObject($Form->{'PaymentMethod'}))
         ? $Shop->loadObject($Form->{'PaymentMethod'}, 'PaymentMethod') : undef;
        SavePayment($Container, $NewPaymentMethod);

    }, [qw (ShippingMethodChanged PaymentMethodChanged NoShippingMethod NoPaymentMethod)]);
    $Servlet->form->mapFormError({
        'Form' => 'Save',
        },$_) foreach @$aWarnings;

    # recalc, get new prices from mbo
    $Basket->recalculate;
    $Servlet->form->executeFormError();
    return;
}



#========================================================================================
# function     PickupBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       PickupBasket( $Servlet );
#----------------------------------------------------------------------------------------
# description  Picks up a basket which was created via the REST API.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub PickupBasket {
    my $self = shift;
    my ($Servlet) = @_;

    $self->RegisterSession($Servlet);
    $Servlet->params('ViewAction', 'ViewMultiCheckoutAddress');

    return;
}


#========================================================================================
# function     RegisterSession
# state        public
#----------------------------------------------------------------------------------------
# syntax       RegisterSession( $Servlet );
#----------------------------------------------------------------------------------------
# description  Validates token and register session.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub RegisterSession {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object();

    my $OrderCreationDate = $Basket->get('CreationDate');
    my $PickupTokenExpiration = $self->_createPickupTokenExpiration();
    my $IsExpired = $PickupTokenExpiration->isExpired($OrderCreationDate);

    my $PickupToken = $Servlet->requestHeaders('X-PickupToken');
    if(! isValid($PickupToken)) {
        $PickupToken = $Servlet->params('PickupToken');
    }

    my $PickupTokenValidation = $self->_createPickupTokenValidation();
    my $UrlForValidation = $PickupTokenValidation->getUrlForPickupTokenValidation($Servlet);
    my $IsTokenValid = $PickupTokenValidation->checkPickupToken($PickupToken, $UrlForValidation);

    if ($IsTokenValid && !$IsExpired) {
        my  $hParams = _RegisterSession($Servlet, $Basket);
        $Servlet->params('RegisterSession', to_json($hParams));
    }

    return;
}

#========================================================================================
# function     _RegisterSession
# state        private
#----------------------------------------------------------------------------------------
# syntax       _RegisterSession( $Servlet, Basket );
#----------------------------------------------------------------------------------------
# description  Helper function for registering a session
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $Basket | Basket | basket object
# return       $hParams | ref.hash | result hash with info
#========================================================================================
sub _RegisterSession {
    my ($Servlet, $Basket) = @_;

    # get informations from session cookie
    my $CookieSessionID = $Servlet->cookies('SessionID');
    my $SessionUserID   = ($CookieSessionID and ExistsSessionBySessionID($CookieSessionID)) ?
                           GetInfoSessionBySessionID($CookieSessionID)->{'UserID'} : undef;
    my $SessionUser     = $SessionUserID ? $Basket->getSite()->loadObject($SessionUserID, 'User') : undef;
    my $hParams         = {};

    # we return if we have a registered basket user and a valid session cookie for his user
    my $BasketUser = $Basket->get('User');
    if($SessionUserID and ($SessionUserID eq $BasketUser->id)){
        $hParams->{'SessionUserIsBasketUser'} = 1;
        return $hParams;
    }

    # if we have no session we always register a new session
    if (not $CookieSessionID){
        $Servlet->registerSession($BasketUser) ;
        $hParams->{'SessionRegistered'} = 1;
        return $hParams;
    }

    # move the basket to the registered user if we have a registered user and an anonymous basket
    if( $SessionUser and $BasketUser->get('IsAnonymous') ){
        $hParams->{'MoveBasket'} = 1;
        _MoveBasket($Basket, $BasketUser, $SessionUser);
    }

    return $hParams;
}

sub _MoveBasket {
    my ($Basket, $BasketUser, $SessionUser) = @_;

    # we move the basket from the $BasketUser to the $SessionUser
    my $BasketFolder = $SessionUser->folder('Baskets');
    Transaction( sub {
        my $Alias = $Basket->alias;
        my $Customer = GetCustomerForUser($SessionUser, $Basket->getSite());
        $Basket->container->set({'TaxModel' => $Customer->get('TaxModel')}) if defined $Customer;
        $BasketFolder->child($Alias)->delete if $BasketFolder->existsChild($Alias);
        my $HasBillingAddress = 0;
        if (defined $Basket->get('BillingAddress')) {
            $Basket->get('BillingAddress')->delete;
            $HasBillingAddress = 1;
        }
        my $NewBillingAddress = defined $SessionUser->get('BillingAddress') ? $SessionUser->get('BillingAddress') : undef;

        $Basket->set({
            'Parent'   =>$BasketFolder,
            'User'     =>$SessionUser,
            'Customer' =>$Customer,
            'BillingAddress' => $HasBillingAddress ? $NewBillingAddress : undef,
        });
        $Basket->recalculate;
    } );

    return;
}

sub _createPickupTokenExpiration {
    return DE_EPAGES::Order::API::PickupTokenExpiration->new();
}

sub _createPickupTokenValidation {
    return DE_EPAGES::Order::API::PickupTokenValidation->new();
}

1;
