#========================================================================================
# package      DE_EPAGES::Packlink::API::AppStorePacklink
# state        public
# description  Functions related to Packlink's App.
#========================================================================================
package DE_EPAGES::Packlink::API::AppStorePacklink;

use strict;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Packlink::API::Constants qw (
    PACKLINK_DOMAIN
    PACKLINK_INSTALL_TRIGGER
    PACKLINK_UNINSTALL_TRIGGER
);
use DE_EPAGES::Packlink::API::ShippingMethodPacklink;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AppStorePacklink = DE_EPAGES::Packlink::API::AppStorePacklink->new( );
#----------------------------------------------------------------------------------------
# description  AppStorePacklink constructor
#----------------------------------------------------------------------------------------
# return       $AppStorePacklink | AppStorePacklink object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     _getPacklinkAppId
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getPacklinkAppId( $Shop );
#----------------------------------------------------------------------------------------
# description  Obtains Packlink App ID.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop. | object
# return       $appId | Packlink App ID or undef if it isn't installed. | String
#========================================================================================
sub _getPacklinkAppId {
    my $self = shift;
    my ($Shop) = @_;

    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    my $hShopParams = {
        'ShopType' => $Shop->get('ShopTypeRef')->alias(),
        'Shop'     => $Shop
    };

    my $AppsClient = $self->_createAppsClient($hShopParams);

    return $self->_findAppIdByUrl($AppsClient);
}

#========================================================================================
# function     isPacklinkAppInstalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       isPacklinkAppInstalled( $Shop, $appId );
#----------------------------------------------------------------------------------------
# description  Determines whether Packlink app is intalled or not.
#----------------------------------------------------------------------------------------
# input        $Shop                 | Shop.                              | object
# input        $appId                | appId. Optional parameter.         | String
# return       $isPacklinkAppInstalled | True = installed, otherwise false. | boolean
#========================================================================================
sub isPacklinkAppInstalled {
    my $self = shift;
    my ( $Shop, $appId ) = @_;

    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    unless ( $self->_isValid($appId) ) {
        $appId = $self->_getPacklinkAppId($Shop);
    }
    unless ( $self->_isValid($appId) ) {
        return 0;
    }
    my $hShopParams = {
        'ShopType' => $Shop->get('ShopTypeRef')->alias(),
        'Shop'     => $Shop
    };
    my $AppsClient = $self->_createAppsClient($hShopParams);
    $self->_LogShippingFulfillment( 'Packlink', 'appID', $appId, 'isPacklinkAppInstalled' );

    my $hLinks       = $self->_getAppLinks($AppsClient, $appId);
    my $aLinks       = $hLinks->{'links'};
    my $aLinksLength = scalar @$aLinks;
    for ( my $i = 0 ; $i < $aLinksLength ; $i++ ) {

        if ( $aLinks->[$i]->{'rel'} eq 'uninstallApp' ) {
            return 1;
        }
    }

    return 0;
}

#========================================================================================
# function     installPacklinkApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       installPacklinkApp( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Installs Packlink app.
#----------------------------------------------------------------------------------------
# input        $Servlet   | Servlet                            | object
# input        $Shop      | Current shop                       | object
# return       $installed | True = installed, otherwise false. | boolean
#========================================================================================
sub installPacklinkApp {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    return $self->_updatePacklinkAppStatus( $Servlet, $Shop, PACKLINK_INSTALL_TRIGGER );
}

#========================================================================================
# function     uninstallPacklinkApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       uninstallPacklinkApp( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Uninstalls Packlink app.
#----------------------------------------------------------------------------------------
# input        $Servlet             | Servlet object                     | object
# input        $Shop                | Current shop                       | object
# return       $installed           | True = installed, otherwise false. | boolean
#========================================================================================
sub uninstallPacklinkApp {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    return $self->_updatePacklinkAppStatus( $Servlet, $Shop, PACKLINK_UNINSTALL_TRIGGER );
}

#========================================================================================
# function     _updatePacklinkAppStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _updatePacklinkAppStatus( $Servlet, $Shop, $HookName );
#----------------------------------------------------------------------------------------
# description  Installs or uninstalls Packlink App.
#----------------------------------------------------------------------------------------
# input        $Servlet     | Servlet                            | object
# input        $Shop        | Current shop                       | object
# input        $HookName    | Hook which will be triggered       | String
# return       $installed   | True = installed, otherwise false. | boolean
#========================================================================================
sub _updatePacklinkAppStatus {
    my $self = shift;
    my ( $Servlet, $Shop, $HookName ) = @_;

    $self->_TestObject( 'Servlet', $Servlet, 'DE_EPAGES::Presentation::Servlet::Permission' );
    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $AppId = $Servlet->params('appID');
    unless ( $self->_isValid($AppId) ) {
        $AppId = $self->_getPacklinkAppId($Shop);
    }

    $self->_LogShippingFulfillment( 'Packlink', 'appID', $AppId, '_updatePacklinkAppStatus' );

    if ( $self->_isValid($AppId) ) {
        eval { $self->_TriggerHook( $HookName, { 'Shop' => $Shop, 'AppID' => $AppId, 'AppUrl' => PACKLINK_DOMAIN } ); };
        if ( ExistsError() ) {
            my $Error = GetError();
            GetLog->error($Error);
            return 0;
        }
    }

    return 1;
}

#----------------------------------------------------------------------------------------
# Wrappers
#========================================================================================
# function     _createAppsClient
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createAppsClient( $hShopParams );
#----------------------------------------------------------------------------------------
# description  Wrapper to create an instance of AppsClient.
#----------------------------------------------------------------------------------------
# input        $hShopParams | Shop and ShopType.                            | object
# return       $appId       | Packlink App ID or undef if it isn't installed. | String
#========================================================================================
sub _createAppsClient {
    my $self = shift;

    my ($hShopParams) = @_;

    my $AppsClient = DE_EPAGES::AppStore::API::Object::AppsClient->create($hShopParams);
    $self->_TestObject( 'AppsClient', $AppsClient, 'DE_EPAGES::AppStore::API::Object::AppsClient' );

    return $AppsClient;
}

#----------------------------------------------------------------------------------------
sub _TestObject {
    my $self = shift;

    my ( $Name, $Value, $ClassPackage ) = @_;

    TestObject( $Name, $Value, $ClassPackage );
}

#----------------------------------------------------------------------------------------
sub _isValid {
    my $self = shift;

    my ($Parameter) = @_;

    return isValid($Parameter);
}

#----------------------------------------------------------------------------------------
sub _LogShippingFulfillment {
    my $self = shift;

    my ( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section ) = @_;

    LogShippingFulfillment( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section );

    return;
}

#----------------------------------------------------------------------------------------
sub _TriggerHook {
    my $self = shift;

    my ( $HookName, $hArgs ) = @_;

    TriggerHook( $HookName, $hArgs );

    return;
}

#----------------------------------------------------------------------------------------
sub _findAppIdByUrl {
    my $self = shift;
    my ($AppsClient) = @_;

    return $AppsClient->findAppIdByUrl( 'APP_SOAP', PACKLINK_DOMAIN );;
}

#----------------------------------------------------------------------------------------
sub _getAppLinks {
    my $self = shift;
    my ($AppsClient,$appId) = @_;

    return $AppsClient->getAppLinks($appId);
}


1;
