#========================================================================================
# package      DE_EPAGES::Packlink::API::ShippingMethodPacklink
# state        public
# description  Class package to handle shipping method Packlink stuff.
#========================================================================================
package DE_EPAGES::Packlink::API::ShippingMethodPacklink;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject Transaction );
use DE_EPAGES::Shipping::API::Tools qw (
    SetLinkShippingProviderMethod
    SetLogoToShippingMethod
);
use DE_EPAGES::Packlink::API::Constants qw (
    SHIPPING_METHOD_NAME_IN_SHOP
);

use strict;
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new();
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self   | self   | DE_EPAGES::Packlink::API::ShippingMethodPacklink
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     getShippingMethodPacklink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethodPacklink = $ShippingMethodPacklink->getShippingMethodPacklink($Shop);
#----------------------------------------------------------------------------------------
# description  It returns the Packlink shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# return       $ShippingMethodPacklink | Packlink shipping method object | object
#========================================================================================
sub getShippingMethodPacklink {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethodPacklink = $self->_getPacklinkShippingMethod($Shop);
    if ( !isValid($ShippingMethodPacklink) ) {
        $ShippingMethodPacklink = $self->_createShippingMethodPacklink($Shop);
    }

    return $ShippingMethodPacklink;
}

#==================================================================================================================
# function     getShippingProviderMethodPacklink
# state        public
#------------------------------------------------------------------------------------------------------------------
# syntax       $ShippingProviderMethodPacklink = $ShippingMethodPacklink->getShippingProviderMethodPacklink($Shop);
#------------------------------------------------------------------------------------------------------------------
# description  It returns the Shipping provider method
#------------------------------------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       $ProviderMethodsFolder | Packlink shipping provider method object | object
#==================================================================================================================
sub getShippingProviderMethodPacklink {
    my $self = shift;
    my ($Shop) = @_;

    my $ProviderMethodsFolder = $Shop->folder('ShippingProviderMethods');

    return $ProviderMethodsFolder->existsChild('Packlink') ? $ProviderMethodsFolder->child('Packlink') : undef;
}

#========================================================================================
# function     _getPacklinkShippingMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getPacklinkShippingMethod( $Shop );
#----------------------------------------------------------------------------------------
# description  get the packlink Shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       $PacklinkShippingMethod | Packlink shipping method object | object
#========================================================================================
sub _getPacklinkShippingMethod {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethods = $Shop->folder('ShippingMethods')->children();

    foreach my $ShippingMethod (@$ShippingMethods) {
        my $SelectedShippingProviderMethod = $ShippingMethod->get('SelectedShippingProviderMethod');
        if ( isValid($SelectedShippingProviderMethod) ) {
            my $SelectedShippingProviderMethodAlias = $SelectedShippingProviderMethod->get('Class')->get('Alias');
            if ( $SelectedShippingProviderMethodAlias eq 'ShippingProviderMethodPacklink' ) {
                return $SelectedShippingProviderMethodAlias;
            }
        }
    }
    return;
}

#========================================================================================
# function     _createShippingMethodPacklink
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createShippingMethodPacklink( $Shop );
#----------------------------------------------------------------------------------------
# description  Create a Packlink Shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       Shipping Method Packlink | Shipping method | object
#========================================================================================
sub _createShippingMethodPacklink {
    my $self = shift;
    my ($Shop) = @_;

    LogShippingFulfillment( 'Packlink', 'Creating a new Shipping method (Packlink) for', SHIPPING_METHOD_NAME_IN_SHOP );
    my $NewShippingMethodObject;
    Transaction(
        sub {
            # ShippingFullfillmentTypePacklink
            my $ShippingFullfillmentType = $self->_getShippingFulfillmentTypePacklink();
            my $ShippingProviderMethod = $self->_getOrCreateShippingProviderMethodPacklink( $Shop, $ShippingFullfillmentType );
            LogShippingFulfillment( 'Packlink', ' Getting the FullfillmentType with ID: ', $ShippingFullfillmentType->id );

            # ShippingTypes:Fixed price
            my $ShippingType = $self->_getFixedPriceShippingType($Shop);
            LogShippingFulfillment( 'Packlink', ' Creating the shipping type with ID: ', $ShippingType->id );

            # Create Shipping method
            $NewShippingMethodObject = $self->_newShippingMethod( $Shop, $ShippingType );
            LogShippingFulfillment('Packlink', ' Creating new Shipping Method has been created with id: ',$NewShippingMethodObject->id);

            $self->_setLinkAndLogo( $NewShippingMethodObject, $ShippingFullfillmentType, $ShippingProviderMethod );
        }
    );

    return $NewShippingMethodObject;
}

#========================================================================================
# function     _setLinkAndLogo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setLinkAndLogo( $NewShippingMethodObject, $ShippingFullfillmentType );
#----------------------------------------------------------------------------------------
# description  It sets the link and logo for the shipping method
#----------------------------------------------------------------------------------------
# input        $NewShippingMethodObject  | Shipping method          | object
# input        $ShippingFullfillmentType | Shipping fulfillmentType | object
# input        $ShippingProviderMethod   | Shipping provider method | object
#========================================================================================
sub _setLinkAndLogo {
    my $self = shift;
    my ( $NewShippingMethodObject, $ShippingFullfillmentType, $ShippingProviderMethod ) = @_;

    if ( isValid($NewShippingMethodObject) && isValid($ShippingProviderMethod) ) {
        SetLinkShippingProviderMethod( $NewShippingMethodObject, $ShippingProviderMethod );
    }
    else {
        LogError(' ShippingFullfillmentType or ShippingMethod are not valid');
    }

    SetLogoToShippingMethod( $NewShippingMethodObject, $ShippingFullfillmentType );
    return;
}

#========================================================================================
# function     _newShippingMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _newShippingMethod( $Shop, $ShippingType  );
#----------------------------------------------------------------------------------------
# description  It creates a new shipping method
#----------------------------------------------------------------------------------------
# input        $Shop                    | Shop            | object
# input        $ShippingType            | ShippingType    | object
# return       $NewShippingMethodObject | shipping method | object
#========================================================================================
sub _newShippingMethod {
    my $self = shift;
    my ( $Shop, $ShippingType ) = @_;

    my %Form;

    my $ShopMethods = $Shop->get('ShippingMethods');
    my $TaxMatrix   = $Shop->get('TaxMatrix');
    $Form{'Shop'}      = $Shop;
    $Form{'IsDefault'} = 1 unless scalar @$ShopMethods;
    $Form{'TaxClass'}  = $TaxMatrix->defaultTaxClass unless defined $Form{'TaxClass'};
    $Form{'Alias'}     = SHIPPING_METHOD_NAME_IN_SHOP;

    my $NewShippingMethodObject;
    eval { $NewShippingMethodObject = $ShippingType->newShippingMethod( $Shop, \%Form ); };

    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
    }

    $Shop->child('ShippingMethods')->renumberChildren;

    return $NewShippingMethodObject;
}

#========================================================================================
# function     _getShippingFulfillmentTypePacklink
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentTypePacklink = $self->_getShippingFulfillmentTypePacklink( );
#----------------------------------------------------------------------------------------
# description  It returns the Packlink shipping fulfillment type
#----------------------------------------------------------------------------------------
# return       $ShippingFulfillmentTypePacklink | Packlink shipping fulfillment type | object
#========================================================================================
sub _getShippingFulfillmentTypePacklink {
    my $self = shift;

    return LoadObjectByPath('/ShippingFulfillmentTypes/Packlink');
}

#========================================================================================
# function     _getOrCreateShippingProviderMethodPacklink
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getOrCreateShippingProviderMethodPacklink( $Shop, $ShippingFulfillmentType );
#----------------------------------------------------------------------------------------
# description  It returns existing Packlink Shipping Provider Method or a new one
#----------------------------------------------------------------------------------------
# input        $Shop                      | Shop                     | object
# input        $ShippingFulfillmentType   | Shipping Fulfillment type | object
# return       $ShippingFulfillmentMethod | fulfillment method       | object
#========================================================================================
sub _getOrCreateShippingProviderMethodPacklink {
    my $self = shift;
    my ( $Shop, $ShippingFulfillmentType ) = @_;

    my $ShippingProviderMethods = $Shop->get('ShippingProviderMethods');

    foreach my $ShippingProviderMethod (@$ShippingProviderMethods) {
        my $ShippingFulfillmentTypeInUse = $ShippingProviderMethod->get('ShippingFulfillmentType')->id();

        if ( $ShippingFulfillmentType->id() eq $ShippingFulfillmentTypeInUse ) {
            return $ShippingProviderMethod;
        }
    }

    my $NewShippingProviderMethod = $self->_newShippingProviderMethodPacklink( $Shop, $ShippingFulfillmentType );

    return $NewShippingProviderMethod;
}


#========================================================================================
# function     _newShippingProviderMethodPacklink
# state        private
#----------------------------------------------------------------------------------------
# syntax       _newShippingProviderMethodPacklink( $Shop,$ShippingFulfillmentType );
#----------------------------------------------------------------------------------------
# description  It creates a new shipping provider method Packlink
#----------------------------------------------------------------------------------------
# input        $Shop                       | Shop                      | object
# input        $ShippingFulfillmentType    | Shipping Fulfillment Type | object
# return       $NewShippingProviderMethod  | Shipping provider method  | object
#========================================================================================
sub _newShippingProviderMethodPacklink {
    my $self = shift;
    my ($Shop,$ShippingFulfillmentType) = @_;

    my $ShippingFulfillmentTypeAlias = $ShippingFulfillmentType->alias();
    my $hInfo = { 'Alias' => $ShippingFulfillmentTypeAlias } ;
    my $NewShippingProviderMethod = $ShippingFulfillmentType->newShippingProviderMethod( $Shop, $hInfo );

    return $NewShippingProviderMethod ;
}

#========================================================================================
# function     _getFixedPriceShippingType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getFixedPriceShippingType( $Shop );
#----------------------------------------------------------------------------------------
# description  It returns the Fixed Price Shipping Type
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# return       $ShippingType | shipping type | object
#========================================================================================
sub _getFixedPriceShippingType {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingTypesList = LoadRootObject()->get('ShippingTypes');

    foreach my $ShippingType (@$ShippingTypesList) {
        my $IsActivated             = $ShippingType->get('IsActivated');
        my $ShippingTypeMethodClass = $ShippingType->get('MethodClass')->get('Alias');
        if ( isValid($ShippingType) && $IsActivated ) {
            if ( $ShippingTypeMethodClass eq 'ShippingMethodFixPrice' ) {
                return $ShippingType;
            }
        }
    }
    return;
}

1;
