#========================================================================================
# package      DE_EPAGES::Packlink::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::Packlink::UI::Shop;
use base DE_EPAGES::ExternalShipping::UI::Shop;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Packlink::API::AppStorePacklink;
use DE_EPAGES::Packlink::API::CredentialsPacklink;
use DE_EPAGES::Packlink::API::ShippingMethodPacklink;
use DE_EPAGES::Packlink::API::Tools;
use DE_EPAGES::Shipping::UI::Shop;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;


#========================================================================================
# function     GrantPacklinkAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->GrantPacklinkAuthorization($Servlet);
#----------------------------------------------------------------------------------------
# description  ChangeAction function implemented authorise Packlink connections to
#               Merchant's Shop.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub GrantPacklinkAuthorization {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $redirectUrl = $Shop->get('WebUrlAdminSSL') . '/?ViewAction=MBO-ViewShippingList&ObjectID=' . $Shop->id ;
    $self->_redirectServlet($Servlet, $redirectUrl);

    return;
}

#========================================================================================
# function     ViewPacklinkAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewPacklinkAuthorization($Servlet);
#----------------------------------------------------------------------------------------
# description  ViewAction function implemented as part of the landing page for the Packlink
#               redirection performed right after generating a new Packlink ID for the merchant
#               or when uninstalling packlink.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub ViewPacklinkAuthorization {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $IsPacklinkAppAvailable = $self->_isPacklinkAppAvailable($Shop);
    if ( $self->_validateConnection($Servlet, $Shop) 
         && !($self->_validatePacklinkInstalled($Servlet, $Shop, $IsPacklinkAppAvailable)) ) {
        $self->_setupPacklink($Servlet, $Shop, $IsPacklinkAppAvailable);
    }

    if ($IsPacklinkAppAvailable) {
        $self->_validateUninstallRequest($Servlet);
    } else {
        $self->_redirectToShippingList($Servlet, $Shop);
    }

    return;
}

#========================================================================================
# function     ViewPacklinkConsecution
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewPacklinkConsecution($Servlet);
#----------------------------------------------------------------------------------------
# description  ViewAction function implemented as part of the landing page for the Packlink.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub ViewPacklinkConsecution {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $appID = $Servlet->params('appID');
    my $cancelButtonViewURL;
    if (isValid($appID) && !$self->_isMaxDeliveryMethodsReached($Servlet, $Servlet->object())) {
        $cancelButtonViewURL = '?ViewAction=MBO-ViewAppDetails&amp;appID='.$appID;
    } else {
        $cancelButtonViewURL = $Shop->isUnity() ?
                '?ViewAction=UnityMBO-ViewShippingList' : '?ViewAction=MBO-ViewShippingList';
    }

    $Servlet->vars('cancelButtonViewURL', $cancelButtonViewURL);
    $Servlet->vars('appID', $appID);

    return;
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------

#========================================================================================
# function     _isMaxDeliveryMethodsReached
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isMaxDeliveryMethodsReached( $Servlet,$Shop );
#----------------------------------------------------------------------------------------
# description  Tells us if the maximum number of delivery methods is reached
#----------------------------------------------------------------------------------------
# input        $Servlet                  | Servlet object                        | object
# input        $Shop                     | Shop object                           | object
# return       $ReturnVariable           | Comparison between feature max Value 
#                                             and actual value                    | boolean
#========================================================================================
sub _isMaxDeliveryMethodsReached {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $System = $self->_LoadRootObject();
    my $Feature = $System->child('Features')->child('ShippingMethods');

    my $CurrentValue = $self->_featureCurrentValue($Shop, $Feature);
    my $MaxValue = $self->_featureMaxValue($Shop, $Feature);

    if($CurrentValue >= $MaxValue) {
        my $hParams = {
            'ViewAction' => 'MBO-ViewShippingList',
        };
        my $hOptions = {
            'Type' => 'admin',
            'UseSSL' => 1,
        };
        my $URL = BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
        $self->_redirectServlet($Servlet, $URL);
    }

    return;
}

#========================================================================================
# function     _createPacklinkCredentials
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createPacklinkCredentials( $Servlet,$Shop,$ShippingMethodPacklink );
#----------------------------------------------------------------------------------------
# description  Create credentials for Packlink.
#----------------------------------------------------------------------------------------
# input        $Servlet                  | Servlet object                        | object
# input        $Shop                     | Shop object                           | object
# input        $ShippingMethodPacklink     | Shipping Method Packlink                | object
# return       $PacklinkCredentialsCreated | Result of Packlink credentials creation | boolean
#========================================================================================
sub _createPacklinkCredentials {
    my $self = shift;
    my ($Servlet,$Shop,$ShippingMethodPacklink) = @_;

    my $Form = $Servlet->form();
    my $CredentialsPacklink = DE_EPAGES::Packlink::API::CredentialsPacklink->new($ShippingMethodPacklink);

    return $CredentialsPacklink->createPacklinkCredentials($Form,$Shop);
}

#========================================================================================
# function     _redirectToShippingList
# state        private
#----------------------------------------------------------------------------------------
# syntax       _redirectToShippingList( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Redirect flow to shipping list URL
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# input        $Shop | Shop object | object
#========================================================================================
sub _redirectToShippingList {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $CurrentViewAction = $Servlet->params('ViewAction');
    my $NextViewAction = 'MBO-ViewShippingList';
    if ($CurrentViewAction =~ m/^Unity.*/) {
        $NextViewAction = 'Unity' . $NextViewAction;
    }
    my $URL = BuildShopUrl($Shop,
        {'ViewAction' => $NextViewAction, 'ObjectId' => $Shop->id()},
        {'Type' => 'admin','UseSSL' => 1},
        $Servlet);

    $self->_redirectServlet($Servlet, $URL);

    return;
}

#========================================================================================
# function     _setupPacklink
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setupPacklink( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Set up credentials and install packlink app store (if necessary).
#               Also, set servlet installation variables accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# input        $Shop | Shop object | object
# input        $IsPacklinkAppAvailable | wether packlink app is available in store | boolean
#========================================================================================
sub _setupPacklink {
    my $self = shift;
    my ($Servlet, $Shop, $IsPacklinkAppAvailable) = @_;

    my $ShippingMethodPacklink = DE_EPAGES::Packlink::API::ShippingMethodPacklink->new();
    my $ShippingMethod = $ShippingMethodPacklink->getShippingMethodPacklink($Shop);

    my $IsInstalledOk = 0;
    if (isValid($ShippingMethod)) {
        my $PacklinkCredentialsCreated = $self->_createPacklinkCredentials($Servlet, $Shop, $ShippingMethodPacklink);
        if ($PacklinkCredentialsCreated) {
            if ($IsPacklinkAppAvailable) {
                my $AppStorePacklink = $self->_createPacklinkAppStore();
                $IsInstalledOk = $AppStorePacklink->installPacklinkApp($Servlet, $Shop);
            } else {
                $IsInstalledOk = 1;
            }
        }
    }
    $Servlet->vars('IsInstalledOk', $IsInstalledOk);
    $Servlet->vars('PacklinkConnectionValidated', $IsInstalledOk);

    return;
}


#========================================================================================
# function     _validateConnection
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsValidConnection = $self->_validateConnection( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Check components for a successful connection to Packlink and set servlet var accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# return       $IsValidConnection | valid connection flag | boolean
#========================================================================================
sub _validateConnection {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $PacklinkConnectionValidated = 0;
    if (isValid($Servlet->params('PacklinkID'))) {
        $Servlet->vars('PacklinkID', $Servlet->params('PacklinkID'));
        $PacklinkConnectionValidated = 1;
    } else {
        $self->_validatePacklinkProviderMethod($Servlet, $Shop);
    }
    $Servlet->vars('PacklinkConnectionValidated', $PacklinkConnectionValidated);

    return $PacklinkConnectionValidated;
}

#========================================================================================
# function     _validatePacklinkProviderMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsValidProviderMethod = $self->_validatePacklinkProviderMethod( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  it checks if packlink provider method already exists and sets servlet vars accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# return       $IsValidProviderMethod | provider method validation | boolean
#========================================================================================
sub _validatePacklinkProviderMethod {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $ShippingMethodPacklink = DE_EPAGES::Packlink::API::ShippingMethodPacklink->new();
    my $PacklinkProviderMethod = $ShippingMethodPacklink->getShippingProviderMethodPacklink($Shop);

    if (!defined($PacklinkProviderMethod)) {
        $Servlet->vars('NoPacklinkProviderMethod', 1);
        return 0;
    }

    return 1;
}

#========================================================================================
# function     _validatePacklinkInstalled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsPacklinkInstalled = $self->_validatePacklinkInstalled( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Checks if packlink is installed and sets servlet var accordingly.
#               Check is done differently depending on packlink app availability.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# input        $IsPacklinkAppAvailable | if Packlink app store is available | boolean
# return       $IsPacklinkInstalled | packlink installed flag | boolean
#========================================================================================
sub _validatePacklinkInstalled {
    my $self = shift;
    my ($Servlet, $Shop, $IsPacklinkAppAvailable) = @_;

    my $IsPacklinkInstalled;
    if ($IsPacklinkAppAvailable) {
        my $AppStorePacklink = $self->_createPacklinkAppStore();
        $IsPacklinkInstalled =  $AppStorePacklink->isPacklinkAppInstalled($Shop);
    } else {
        $IsPacklinkInstalled = $self->_isInstallRequest($Servlet) ? 0 : 1;
    }
    $Servlet->vars('IsPacklinkInstalled', $IsPacklinkInstalled);

    return $IsPacklinkInstalled;
}

#========================================================================================
# function     _isPacklinkAppAvailable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsPacklinkAppAvailable = $self->_isPacklinkAppAvailable( $Shop );
#----------------------------------------------------------------------------------------
# description  It provides availability for packlink app
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $IsPacklinkAppAvailable | packlink app availability flag | boolean
#========================================================================================
sub _isPacklinkAppAvailable {
    my $self = shift;
    my ($Shop) = @_;

    my $Tools = $self->_createTools();
   return $Tools->IsPacklinkAppAvailable($Shop);
}

#========================================================================================
# function     _validateUninstallRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsUninstall = $self->_validateUninstallRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  It returns if AppStore request is an uninstall or not (install in this case)
#               and sets servlet variables accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# return       $IsUninstall | is an uninstall or not | boolean
#========================================================================================
sub _validateUninstallRequest {
    my $self = shift;
    my ($Servlet) = @_;

    my $UninstallFlag = $Servlet->params('uninstall');
    if (!defined($UninstallFlag)) {
        $UninstallFlag = 0;
    }
    $Servlet->vars('UninstallPacklink', $UninstallFlag);
    return $UninstallFlag;
}

#========================================================================================
# function     _isInstallRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsInstall = $self->_isInstallRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  It returns if request is an install or uninstall
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# return       $IsInstall | is an install or not | boolean
#========================================================================================
sub _isInstallRequest {
    my $self = shift;
    my ($Servlet) = @_;

    my $UninstallFlag = $Servlet->params('uninstall');
    if ($UninstallFlag) {
        return 0;
    }
    return 1;
}

#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------

#----------------------------------------------------------------------------------------
sub _createTools {
    my $self = shift;

    return DE_EPAGES::Packlink::API::Tools->new();
}

sub _createPacklinkAppStore {
    my $self = shift;

    return DE_EPAGES::Packlink::API::AppStorePacklink->new();
}

#----------------------------------------------------------------------------------------
sub _featureCurrentValue {
    my $self = shift;
    my ($Shop, $Feature) = @_;

    return $Shop->featureCurrentValue($Feature);
}

#----------------------------------------------------------------------------------------
sub _featureMaxValue {
    my $self = shift;
    my ($Shop, $Feature) = @_;

    return $Shop->featureMaxValue($Feature);
}

#----------------------------------------------------------------------------------------
sub _redirectServlet {
    my $self = shift;
    my ($Servlet,$redirectUrl) = @_;

    $Servlet->redirect($redirectUrl);
    return;
}

#----------------------------------------------------------------------------------------
sub _LoadRootObject {
    my $self = shift;

    return LoadRootObject();
}

1;
