#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::UI_Shop
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::Packlink::t::unit::UI_Shop;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify::Matcher qw ( HashRef Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use strict;

use constant TEST_PACKLINK_APP_ID => '346789406743';
use constant TEST_PACKLINK_ID => '345623908';
use constant TEST_REDIRECT_URL => '/this/is/redirected/built_url';
use constant TEST_SHOP_ID => '23456';
use constant TEST_WEB_URL_ADMIN_SSL => 'https://test.admin.url';


sub test {
    my $self = shift;

    $self->test_ViewPacklinkAuthorization_PacklinkAppNotAvailable_InstallRequest();
    $self->test_ViewPacklinkAuthorization_PacklinkAppNotAvailable_UninstallRequest();
    $self->test_ViewPacklinkAuthorization_PacklinkAppAvailable_InstallRequest();
    $self->test_ViewPacklinkAuthorization_PacklinkAppAvailable_UninstallRequest();
    $self->test_ViewPacklinkConsecution_NoAppId_GoToShippingList();
    $self->test_ViewPacklinkConsecution_WithAppId_GoToAppDetails();
    $self->test_ViewPacklinkConsecution_WithAppId_GoToAppDetails_NoFeaturesLeft();
    $self->test_GrantPacklinkAuthorization();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkAuthorization_PacklinkAppNotAvailable_InstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 0;
    my $IsPacklinkAppInstalled = undef;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable, $IsPacklinkAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 1;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest);

    $SUT->ViewPacklinkAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isPacklinkAppAvailable'),
            "$SubTestName - Checks whether Packlink app is available in AppStore or not (it is NOT)."
    );
    is(
            $MockedServlet->vars('PacklinkConnectionValidated'),
            1,
            "$SubTestName - Packlink id has been provided (because this is an install process)."
    );
    ok(
            WasCalled($SUT, '_isInstallRequest'),
            "$SubTestName - Checks if this is an install request (IT IS)."
    );
    is(
            $MockedServlet->vars('IsPacklinkInstalled'),
            0,
            "$SubTestName - IsPacklinkInstalled var is set to zero (not yet installed)."
    );
    ok(
            WasCalled($SUT, '_setupPacklink'),
            "$SubTestName - Packlink setup private method gets called."
    );
    ok(
            WasCalled($SUT, '_createPacklinkCredentials'),
            "$SubTestName - Packlink credentials creation private method gets called."
    );
    ok(
            !WasCalled($SUT, '_createPacklinkAppStore'),
            "$SubTestName - Packlink App not set as installed because is not available."
    );
    is(
            $MockedServlet->vars('IsInstalledOk'),
            1,
            "$SubTestName - Packlink installation whent well."
    );
    ok(
            WasCalled($SUT, '_redirectToShippingList'),
            "$SubTestName - Flow redirected because this is an installation from shipment."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkAuthorization_PacklinkAppNotAvailable_UninstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 0;
    my $IsPacklinkAppInstalled = undef;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable, $IsPacklinkAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 0;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsPacklinkAppAvailable);

    $SUT->ViewPacklinkAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isPacklinkAppAvailable'),
            "$SubTestName - Checks whether Packlink app is available in AppStore or not (it is NOT)."
    );
    ok(
            WasCalled($SUT, '_validatePacklinkProviderMethod'),
            "$SubTestName - Packlink provider method gets validated because there's no PacklinkID available."
    );
    is(
            $MockedServlet->vars('PacklinkConnectionValidated'),
            0,
            "$SubTestName - Packlink connection not validated."
    );
    ok(
            !WasCalled($SUT, '_validatePacklinkInstalled'),
            "$SubTestName - Do NOT validate Packlink installation on unsuccessful connection validation (uninstall)."
    );
    ok(
            WasCalled($SUT, '_redirectToShippingList'),
            "$SubTestName - Flow redirected because this is an installation from shipment."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkAuthorization_PacklinkAppAvailable_InstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 1;
    my $IsPacklinkAppInstalled = 0;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable, $IsPacklinkAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 1;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsPacklinkAppAvailable);

    $SUT->ViewPacklinkAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isPacklinkAppAvailable'),
            "$SubTestName - Checks whether Packlink app is available in AppStore or not (IT IS)."
    );
    is(
            $MockedServlet->vars('PacklinkConnectionValidated'),
            1,
            "$SubTestName - Packlink id has been provided (because this is an install process)."
    );
    ok(
            WasCalled($SUT, '_createPacklinkAppStore'),
            "$SubTestName - Get AppStore module to know whether Packlink App appears as installed or not."
    );
    is(
            $MockedServlet->vars('IsPacklinkInstalled'),
            0,
            "$SubTestName - IsPacklinkInstalled var is set to zero (not yet installed)."
    );
    ok(
            WasCalled($SUT, '_setupPacklink'),
            "$SubTestName - Packlink setup private method gets called."
    );
    ok(
            WasCalled($SUT, '_createPacklinkCredentials'),
            "$SubTestName - Packlink credentials creation private method gets called."
    );
    ok(
            WasCalled($SUT, '_createPacklinkAppStore'),
            "$SubTestName - Packlink App is set as installed."
    );
    is(
            $MockedServlet->vars('IsInstalledOk'),
            1,
            "$SubTestName - Packlink installation whent well."
    );
    ok(
            WasCalled($SUT, '_validateUninstallRequest'),
            "$SubTestName - Call to check and setup variables about Packlink App (if this is an install or uninstall."
    );
    is(
            $MockedServlet->vars('UninstallPacklink'),
            0,
            "$SubTestName - This is an install, so uninstall flag for Packlink App should be FALSE."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkAuthorization_PacklinkAppAvailable_UninstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 1;
    my $IsPacklinkAppInstalled = 1;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable, $IsPacklinkAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 0;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsPacklinkAppAvailable);

    $SUT->ViewPacklinkAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isPacklinkAppAvailable'),
            "$SubTestName - Checks whether Packlink app is available in AppStore or not (IT IS)."
    );
    ok(
            WasCalled($SUT, '_validatePacklinkProviderMethod'),
            "$SubTestName - Packlink provider method gets validated because there's no PacklinkID available."
    );
    is(
            $MockedServlet->vars('PacklinkConnectionValidated'),
            0,
            "$SubTestName - Packlink connection not validated."
    );
    ok(
            !WasCalled($SUT, '_validatePacklinkInstalled'),
            "$SubTestName - Do NOT validate Packlink installation on unsuccessful connection validation (uninstall)."
    );
    ok(
            WasCalled($SUT, '_validateUninstallRequest'),
            "$SubTestName - Call to check and setup variables about Packlink App (if this is an install or uninstall."
    );
    is(
            $MockedServlet->vars('UninstallPacklink'),
            1,
            "$SubTestName - This is an uninstall, so uninstall flag for Packlink App should be TRUE."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkConsecution_NoAppId_GoToShippingList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $MockedShop = $self->_createShopConsecution();
    my $hasAppIdParam = 0;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewPacklinkConsecution($MockedServlet);

    ok(
            !WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - No need to check feature resources because there IS NO appID parameter."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewShippingList/,
            "$SubTestName - Cancellation button resolves to shipping list because there IS NO appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkConsecution_WithAppId_GoToAppDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $FeatureCurrentValue = 0;
    my $FeatureMaxValue = 1;
    my $MockedShop = $self->_createShopConsecution($FeatureCurrentValue, $FeatureMaxValue);
    my $hasAppIdParam = 1;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewPacklinkConsecution($MockedServlet);

    ok(
            WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - Check feature resources because there IS an appID parameter."
    );
    ok(
            !WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - NO redirect call done because of reached/exceeded max features."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewAppDetails/,
            "$SubTestName - Cancellation button resolves to application details because there IS an appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewPacklinkConsecution_WithAppId_GoToAppDetails_NoFeaturesLeft {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $FeatureCurrentValue = 1;
    my $FeatureMaxValue = 1;
    my $MockedShop = $self->_createShopConsecution($FeatureCurrentValue, $FeatureMaxValue);
    my $hasAppIdParam = 1;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewPacklinkConsecution($MockedServlet);

    ok(
            WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - Check feature resources because there IS an appID parameter."
    );
    ok(
            WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - Redirect call done because of reached/exceeded max features."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewAppDetails/,
            "$SubTestName - Cancellation button resolves to application details because there IS an appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_GrantPacklinkAuthorization {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $MockedShop = $self->_createShopGrantAuthorization();
    my $MockedServlet = $self->_createServletGrantAuthorization($MockedShop);

    $SUT->GrantPacklinkAuthorization($MockedServlet);

    ok(
            WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - Servlet redirect done."
    );
}

#----------------------------------------------------------------------------------------
# helpers (Common)
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($IsPacklinkAppAvailable, $IsPacklinkAppInstalled) = @_;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Packlink::UI::Shop', $aParameterList);

    # mocks for Authorization tests
    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(
                    Object('DE_EPAGES::Shop::API::Object::Shop'),
                    HashRef(),
                    HashRef(),
                    Object()
            )->thenReturn(TEST_REDIRECT_URL);

    $MockifySut->spy('_isPacklinkAppAvailable')
            ->whenAny();
    if (defined($IsPacklinkAppAvailable)) {
        $MockifySut->mockStatic('DE_EPAGES::Packlink::API::Tools::IsPacklinkAppAvailable')
                ->when(
                        Object('DE_EPAGES::Packlink::API::Tools'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturn($IsPacklinkAppAvailable);
    } else {
        $MockifySut->mockStatic('DE_EPAGES::Packlink::API::Tools::IsPacklinkAppAvailable')
                ->when(
                        Object('DE_EPAGES::Packlink::API::Tools'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturnUndef();
    }
    $MockifySut->spy('_validatePacklinkProviderMethod')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::ShippingMethodPacklink::getShippingProviderMethodPacklink')
            ->when(
                    Object('DE_EPAGES::Packlink::API::ShippingMethodPacklink'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodPacklink());

    $MockifySut->spy('_validatePacklinkInstalled')
            ->whenAny();
    if (defined($IsPacklinkAppInstalled)) {
        $MockifySut->mockStatic('DE_EPAGES::Packlink::API::AppStorePacklink::isPacklinkAppInstalled')
                ->when(
                        Object('DE_EPAGES::Packlink::API::AppStorePacklink'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturn($IsPacklinkAppInstalled);
    } else {
        $MockifySut->mockStatic('DE_EPAGES::Packlink::API::AppStorePacklink::isPacklinkAppInstalled')
                ->when(
                        Object('DE_EPAGES::Packlink::API::AppStorePacklink'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturnUndef();
    }

    $MockifySut->spy('_isInstallRequest')
            ->whenAny();

    $MockifySut->spy('_setupPacklink')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::ShippingMethodPacklink::getShippingMethodPacklink')
            ->when(
                    Object('DE_EPAGES::Packlink::API::ShippingMethodPacklink'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodPacklink());

    $MockifySut->spy('_createPacklinkCredentials')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::CredentialsPacklink::createPacklinkCredentials')
            ->when(
                    Object('DE_EPAGES::Packlink::API::CredentialsPacklink'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn(1);

    $MockifySut->spy('_createPacklinkAppStore')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::AppStorePacklink::installPacklinkApp')
            ->when(
                    Object('DE_EPAGES::Packlink::API::AppStorePacklink'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn(1);

    $MockifySut->spy('_validateUninstallRequest')
            ->whenAny();

    $MockifySut->spy('_redirectToShippingList')
            ->whenAny();

    # additional mocks for Consecution tests
    $MockifySut->spy('_isMaxDeliveryMethodsReached')
            ->whenAny();

    # additional mocks for GrantAuthorization test
    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
            ->whenAny()->thenReturn($self->_createSystem());

    return $MockifySut->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (Authorization)
#----------------------------------------------------------------------------------------
sub _createShopAuthorization {
    my $self = shift;

    my $hAttributes = {
        'ID' => TEST_SHOP_ID,
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createServletAuthorization {
    my $self = shift;
    my ($MockedShop, $IsInstallRequest, $IsPacklinkAppAvailable) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $aParameters = [ { 'string' => TEST_REDIRECT_URL } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    if ($IsInstallRequest) {
        # when installing, PacklinkID must have been provided after contacting Packlink. Otherwise we are deactivating.
        $MockifyServlet->addParams( { 'PacklinkID' => TEST_PACKLINK_ID } );
    } else {
        if ($IsPacklinkAppAvailable) {
            $MockifyServlet->addParams( { 'uninstall' => 1 } );
        }
    }
    
    $MockifyServlet->addVars(
            {
                'IsPacklinkInstalled' => undef,
                'PacklinkConnectionValidated' => undef,
                'UninstallPacklink' => undef,
            }
    );
    return $MockifyServlet->getMockObject();
}

sub _createShippingProviderMethodPacklink {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyShippingProviderMethodPacklink = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $hAttributes
    );

    return $MockifyShippingProviderMethodPacklink->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (Consecution)
#----------------------------------------------------------------------------------------

sub _createShopConsecution {
    my $self = shift;
    my ($FeatureCurrentValue, $FeatureMaxValue) = @_;

    my $hAttributes = {};

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    my $aParameter = [ {'object' => 'DE_EPAGES::Shop::API::Object::Feature' } ];
    $MockifyShop->addMockWithReturnValueAndParameterCheck('featureCurrentValue', $FeatureCurrentValue, $aParameter);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('featureMaxValue', $FeatureMaxValue, $aParameter);
    $MockifyShop->addMockWithReturnValue('isUnity', 0);

    return $MockifyShop->getMockObject();
}

sub _createServletConsecution {
    my $self = shift;
    my ($MockedShop, $hasAppIdParam) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $aParameters = [ { 'string' => TEST_REDIRECT_URL } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    my $AppID = $hasAppIdParam ? TEST_PACKLINK_APP_ID : undef;
    $MockifyServlet->addParams( { 'appID' => $AppID } );

    return $MockifyServlet->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {};
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );
    my $aParameter = [ { 'string' => 'Features' } ];
    $MockifySystem->addMockWithReturnValueAndParameterCheck('child', $self->_createObject(), $aParameter);

    return $MockifySystem->getMockObject();
}

sub _createObject {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object', $hAttributes
    );
    my $aParameter = [ { 'string' => 'ShippingMethods' } ];
    $MockifyObject->addMockWithReturnValueAndParameterCheck('child', $self->_createFeature(), $aParameter);

    return $MockifyObject->getMockObject();
}

sub _createFeature {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyFeature = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Feature', $hAttributes
    );

    return $MockifyFeature->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (GrantAuthorization)
#----------------------------------------------------------------------------------------
sub _createShopGrantAuthorization {
    my $self = shift;

    my $hAttributes = {
        'ID' => TEST_SHOP_ID,
        'WebUrlAdminSSL' => TEST_WEB_URL_ADMIN_SSL
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createServletGrantAuthorization {
    my $self = shift;
    my ($MockedShop) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $ExpectedRedirectUrl = TEST_WEB_URL_ADMIN_SSL . '/?ViewAction=MBO-ViewShippingList&ObjectID=' . TEST_SHOP_ID;
    my $aParameters = [ { 'string' => $ExpectedRedirectUrl } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    return $MockifyServlet->getMockObject();
}

__PACKAGE__->runScript();