#========================================================================================
# package      DE_EPAGES::Paynet::API::Object::PaymentMethodPaynet
# state        public
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::Paynet::API::Object::PaymentMethodPaynet;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (trim);
use DE_EPAGES::Object::API::Template qw (ProcessFileTemplate);
use DE_EPAGES::Object::API::Language;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethodPaynet->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name 'Paynet'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'Paynet'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodPaynet');
#               my $Object = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Adds initial node content.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object
# return       $Object | new object | object
#========================================================================================
sub insert {
    my ($Class, $hInfo) = @_;

    my $self = $Class->SUPER::insert($hInfo);

    # set Note (EPG-7594), Name (EPG-14849), Reason1/2 (EPG-15949)
    my $FileNameReason1 = $ENV{'EPAGES_CARTRIDGES'}."/DE_EPAGES/Paynet/Templates/Reason1Content.txt";
    my $FileNameReason2 = $ENV{'EPAGES_CARTRIDGES'}."/DE_EPAGES/Paynet/Templates/Reason2Content.txt";
    my $FileNameContent = $ENV{'EPAGES_CARTRIDGES'}."/DE_EPAGES/Paynet/Templates/InitialNoteContent.html";
    
    my $PaymentType = $self->get('PaymentType');
    foreach my $hShopLanguage (@{$self->getSite()->get('Languages')}) {
        local $DE_EPAGES::Object::API::Language::LANGUAGEID = $hShopLanguage->{'LanguageID'};
        my $Reason1 = ProcessFileTemplate($FileNameReason1, $hShopLanguage->{'Code2'}, {});
        my $Reason2 = ProcessFileTemplate($FileNameReason2, $hShopLanguage->{'Code2'}, {});
        ($Reason1, $Reason2) = trim($Reason1, $Reason2);

        my $Note = ProcessFileTemplate($FileNameContent, $hShopLanguage->{'Code2'}, $self->tleHash);

        $self->set({
            'Name'    => $PaymentType->get('NameOrAlias', $hShopLanguage->{'LanguageID'}),
            'Reason1' => $Reason1,
            'Reason2' => $Reason2,
            'Note' => $Note
        }, $hShopLanguage->{'LanguageID'});
    }
    return $self;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method with the given currency is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    return $self->get('PaymentType')->get('IsActivated') &&
           defined($self->get('Paynet_CustomerID'))      &&
           defined($self->get('Paynet_ProjectID'));
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               This payment method can only be added to the container if currency EUR
#               has been chosen.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return (
        $hVars->{'CurrencyID'} eq 'EUR' ||
        $hVars->{'CurrencyID'} eq 'CHF' ||
        $hVars->{'CurrencyID'} eq 'GBP'
    )
    && $self->SUPER::canAddToBasket($hVars);
}

1;
