#========================================================================================
# package      DE_EPAGES::Presentation::Servlet::BasePermission
# base         DE_EPAGES::WebInterface::Servlet::BaseServlet
# state        public
#----------------------------------------------------------------------------------------
# description  This servlet acts as base class for all URL handling servlets. It is not
#               intended to be instanced directly.
#========================================================================================
package DE_EPAGES::Presentation::Servlet::BasePermission;
use base qw( DE_EPAGES::WebInterface::Servlet::BaseServlet );

use strict;
use HTTP::Status; # RC_OK, RC_MOVED_PERMANENTLY etc.
use DE_EPAGES::Database::API::Config        qw( DatabaseRedirect );
use DE_EPAGES::Core::API::EpagesConfig      qw( EpagesConfig );
use DE_EPAGES::Core::API::Error             qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::File              qw( GetFileContent );
use DE_EPAGES::Core::API::TestParameter     qw( TestObject );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet = new DE_EPAGES::Presentation::Servlet::BasePermission;
# example      my $BasePermissionServlet = new DE_EPAGES::Presentation::Servlet::BasePermission;
#----------------------------------------------------------------------------------------
# description  Create a new servlet, create Variables member hash.
#----------------------------------------------------------------------------------------
# return       $Servlet | blessed object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = $class->SUPER::new;
    $self->{'Variables'} = {};
    return $self;
}

#========================================================================================
# function     checkDatabaseRedirect
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ContinueWithServlet = $Servlet->checkDatabaseRedirect();
#----------------------------------------------------------------------------------------
# description  This method handles database redirection, e.g. show an error page if a
#               store is closed. It returns 0 if an error or a redirection is done, which
#               then should lead to an abort in the servlet request handler.
#----------------------------------------------------------------------------------------
# return       0 or 1 | 0 if a redirection/error handling is done, 1 if servlet
#               processing should be continued | boolean
#========================================================================================
sub checkDatabaseRedirect {
    my $self = shift;

    my $StoreName = $self->storeName;
    unless (defined $StoreName) {
        $self->handleStoreNotFound( DE_EPAGES::Core::API::Error->new( 'Code' => 'MissingDatabase' ) );
        return 0;
    }

    # remember the database name for error messages etc.
    $self->vars( 'StoreName', $StoreName );

    # check if there is a redirect file for the database
    my $RemoteAddr = $self->serverVariables->{'REMOTE_ADDR'};
    my $Redirect = eval { DatabaseRedirect($StoreName, undef, $RemoteAddr) };
    if( ExistsError() ) {
        my $Error = GetError();
        if( $Error->code eq 'MissingConfigSection') {
            $self->handleStoreNotFound( $Error );
            return 0;
        }
        else {
            Error( $Error );
        }
    }

    if( defined $Redirect ) {
        $self->setErrorCode503( 120 ); ## no critic qw(ProhibitMagicNumbers)
        my $Response = $self->response;
        $Response->content_type( 'text/html; charset=utf-8' );
        $Response->content(${GetFileContent($ENV{'EPAGES_WEBROOT'} .'/'. $Redirect)});
        return 0;
    }
    return 1;
}

#========================================================================================
# function     handleStoreNotFound
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->handleStoreNotFound( $Error );
#----------------------------------------------------------------------------------------
# description  This function sets the HTTP status 404 - "Not Found" and displays
#               an error message and logs a warning message containing QUERY_STRING
#               and HTTP_REFERER.
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
#========================================================================================
sub handleStoreNotFound {
    my $self = shift;
    my ($Error) = @_;
    TestObject( 'Error', $Error );

    $self->logUserFault( $Error );

    # send the status code 404 - File not found to remove the page from search engines
    $self->response->code( RC_NOT_FOUND );
    $self->viewError($Error);
    return;
}

#========================================================================================
# $function     setErrorCode503
# $state        public
#----------------------------------------------------------------------------------------
# $syntax       $Servlet->setErrorCode503;
# $syntax       $Servlet->setErrorCode503( $RetryAfter );
# $example      $Servlet->setErrorCode503( 60 );
#----------------------------------------------------------------------------------------
# $description  This function sets the HTTP status 503.
#               Optionally sets the header Retry-After to $RetryAfter.
#----------------------------------------------------------------------------------------
# input        $RetryAfter | (optional) value of the Retry-After header (in seconds) | int
#========================================================================================
sub setErrorCode503 {
    my $self = shift;
    my ($RetryAfter) = @_;

    $self->response->code( RC_SERVICE_UNAVAILABLE );
    $self->response->headers->header( 'Retry-After' => $RetryAfter ) if $RetryAfter;
}

#========================================================================================
# function     vars
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hVars = $Servlet->vars;
# syntax       $Value= $Servlet->vars($Name);
# syntax       $Servlet->vars($Name, $Value);
# example      $Servlet->vars('OutputEncoding', 'ISO-8859-1');
#----------------------------------------------------------------------------------------
# description  Without parameter returns all set variables.<br>
#               With one parameter (name of variable) the value of this variable will be
#               returned.<br>
#               With two parameter (name and value of variable) the value of this variable will be
#               set.<br>
#----------------------------------------------------------------------------------------
# input        $Name | name of variable (optional) | string
# input        $Value | value of variable (optional) | string
# return       $hVars | set TLE variables as key value pairs | ref.hash
# return       $Value | value of given variable name | string
#========================================================================================
sub vars {
    my $self = shift;
    my (@Params) = @_;

    if (scalar @Params == 0) { # get hash
        return $self->{'Variables'};
    } elsif (scalar @Params == 1) { # get
        return $self->vars->{$Params[0]};
    } elsif (scalar @Params == 2) { # set
        my ($key, $value) = @Params;
        $self->vars->{$key} = $value;
        return $value;
    } else {
        Error('TOO_MANY_PARAMETER');
    }
}

#========================================================================================
# function     logUserFault
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->logUserFault( $Error );
#----------------------------------------------------------------------------------------
# description  Handles an error. This function logs the error and then displays it
#               using the <function viewError> method.
#               This method must be overloaded by any sub class.
#========================================================================================
sub logUserFault {
    die 'logUserFault is not implemented';
}

#========================================================================================
# function     viewError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->viewError( $Error );
#----------------------------------------------------------------------------------------
# description  Displays an error page, which is processed by <function processErrorContent>.
#               This method must be overloaded by any sub class.
#========================================================================================
sub viewError  {
    die 'viewError is not implemented';
}

sub useSSLForShopDomains {
    return EpagesConfig->get('Security', 'UseAlwaysSSLForShopDomains', 0);
}

sub useSSLForProviderDomains {
    return EpagesConfig->get('Security', 'UseAlwaysSSLForProviderDomains', 0);
}

1;