#==============================================================================
# package      DE_EPAGES::Product::API::Basket
# state        public
# description  API for addding products to the basket.
#==============================================================================
package DE_EPAGES::Product::API::Basket;
use base Exporter;
use strict;

our @EXPORT_OK  = qw (
    AddProducts
);

use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Warning qw (Warning );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject Transaction);
use DE_EPAGES::Order::API::Basket qw (GetBasket);

sub ErrorDefinition {
    return {
        'ProductNotVisible' => 'product ( #ProductID ) is not visible cant add to container.',
        'ProductCanNotAddToBasket' => 'can product ( #ProductID ) not add to basket.',
        'ExclusiveShippingMethod' => 'can product ( #ProductID ) not add to basket, because of special shipping methods.',
    };
}

#========================================================================================
# function     AddProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aBaskets = AddProducts($User, $hInfo, $ahProducts);
# example      my @Products;
#               push @Products, { Product => $Product, Quantity => 3 };
#               my $aBaskets = AddProducts( $User, {}, \@Products );
#               my $Basket = $aBaskets->[0];
#----------------------------------------------------------------------------------------
# description  Adds a list of products to a the basket of the user.
#               If the list contains products from different shops, then multiple baskets
#               are created - one for each shop.
#----------------------------------------------------------------------------------------
# input        $User | user | object
# input        $hInfo | basket infos keys:
#               <ul>
#                   <li>LocaleID | Locale, e.g. 'en_US' (optional) | char(5)
#                   <li>CurrencyID | currency, e.g. 'USD' (optional) | char(3)
#                   <li>AddProductStyle | basket action for new added product (optional) | int
#               </ul> | ref.array.hash
# input        $ahProducts | product line info, keys:
#               <ul>
#                   <li>Product - product object
#                   <li>Quantity - product quantity to add (optional)- float
#                   <li>... parameter for $Product->addToContainer()
#               </ul> | ref.array.hash
# return       $aBaskets | baskets of different shops | object
#========================================================================================
sub AddProducts ($$$) {
    my ($User, $hInfo, $ahProducts) = @_;
    my $MaxBasketPositions = LoadRootObject()->get('MaxBasketPositions');
    my $AddProductStyle = defined $hInfo->{'AddProductStyle'} ? $hInfo->{'AddProductStyle'} : undef ;
    delete $hInfo->{'AddProductStyle'};

    my @Baskets;
    Transaction( sub {
        my (%ShopBaskets, %AppointmentBaskets);
        foreach my $hProduct (@$ahProducts) {
            my $Product = $hProduct->{'Product'};

            # can we add the product to the basket?
            if (not $Product->canAddToBasket() or $Product->isMaster ) {
                Warning('ProductCanNotAddToBasket', {'Product'=>$Product, 'ProductID'=>$Product->id});
                next;
            }

            # create basket if not exists
            my $Shop = $Product->getSite();
            my $IsAppointment = $Product->get('IsAppointment');
            my $Basket = $IsAppointment ? $AppointmentBaskets{$Shop->id} : $ShopBaskets{$Shop->id};
            if (not defined $Basket) {
                if ($IsAppointment) {
                    $Basket = GetBasket($User, $Shop, {
                        %$hInfo,
                        Parent => $User->folder('AppointmentBaskets')
                    });
                    $AppointmentBaskets{$Shop->id} = $Basket;
                } else {
                    $Basket = GetBasket($User, $Shop, $hInfo);
                    $ShopBaskets{$Shop->id} = $Basket;
                }
            }
            my $Container = $Basket->container;

            # reach max basket positions?
            if (scalar(@{$Container->get('LineItems')})>=$MaxBasketPositions) {
                Warning('ReachMaxBasketPositions', {'Product'=>$Product, 'ProductID'=>$Product->id});
                next;
            }
            # matches shipping with special product shipping methods
            if ($Shop->featureMaxValue('ShippingMethodsPerProduct')) {
                my $ahShippingMethods = $Product->getWithParent('ShippingMethods');
                if (scalar @$ahShippingMethods) {
                    my @ContainerMethodIDs = map { $_->id } @{$Container->get('ShippingMethods')};
                    # test if method from new product is not existing in one product of the basket
                    if(scalar @ContainerMethodIDs and
                        not grep {ElementOf($_, @ContainerMethodIDs)}
                            map  { $_->id } @$ahShippingMethods) {
                        if (scalar @{$Container->get('LineItems')} ) {
                            Warning('ExclusiveShippingMethod', {'Product'=>$Product, 'ProductID'=>$Product->id});
                        } else {
                            Warning('NoShipping', {'Product'=>$Product, 'ProductID'=>$Product->id});
                        }
                        next;
                    }
                }
            }
            eval {
                my %ProductInfo = %$hProduct;
                delete $ProductInfo{'Product'};
                my $OldShippingMethod = defined $Container->get('Shipping') ? $Container->get('Shipping')->get('ShippingMethod') : undef;
                $ProductInfo{'AddProductStyle'} = $AddProductStyle if defined $AddProductStyle;
                $Product->addToContainer($Basket->container, \%ProductInfo);
                if (defined $OldShippingMethod and $OldShippingMethod->id != $Container->get('Shipping')->get('ShippingMethod')->id) {
                    Warning('ChangedShippingMethod', {'Product'=>$Product, 'ProductID'=>$Product->id});
                }
            };
            if (ExistsError()) {
                my $Error = GetError();
                Error($Error) if $Error->code !~ /Product/;
                Warning($Error);
            }
        }

        @Baskets = (values %ShopBaskets, values %AppointmentBaskets);
        $_->recalculate foreach @Baskets;
    });
    return \@Baskets;
}

1;
