#========================================================================================
# package      DE_EPAGES::Product::API::CSV::ProductTypeExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to export , import producttypes
#========================================================================================
package DE_EPAGES::Product::API::CSV::ProductTypeExportImport;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Product::API::CSV::ProductTypeExportDriver;
use DE_EPAGES::Product::API::CSV::ProductTypeImportHandler;
use DE_EPAGES::Product::API::Constants qw( CSV_WARNING_CODES_PRODUCTTYPE PRODUCT_PAGETYPE
                                           UNITY_PRODUCT_TYPE_ATTRIBUTES_IGNORE);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    my $self = shift;
    return [ @{$self->SUPER::warningCodes}, CSV_WARNING_CODES_PRODUCTTYPE ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::Product::API::CSV::ProductTypeExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::Product::API::CSV::ProductTypeImportHandler';
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->getExportObjects($Shop, $aObjects);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjects       | list of exportable objects | ref.array.integer
# return       $aExportObjects | edited list of exportable objects | ref.array
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjects) = @_;

    my @ObjectIDs;
    my @ProductTypes;
    push @ProductTypes, $Shop->get('ProductClass');
    if( $Shop->existsChild('ProductTypes') ) {
        # exclude mastershop product types
        push @ProductTypes, grep { $_->class->alias eq 'ProductType' } @{$Shop->child('ProductTypes')->children};
    }

    foreach my $ProductType (@ProductTypes) {
        if ($ProductType->existsChild('Attributes') && scalar @{$ProductType->child('Attributes')->children} ) {
            foreach my $Attribute (@{$ProductType->child('Attributes')->children}) {
                if ($Attribute->existsChild('PreDefAttributes') && scalar @{$Attribute->child('PreDefAttributes')->children} ) {
                    push @ObjectIDs, map {$_->id} @{$Attribute->child('PreDefAttributes')->children};
                } else {
                    push @ObjectIDs, $Attribute->id;
                }
            }
        } else {
            push @ObjectIDs, $ProductType->id;
        }
    }

    return \@ObjectIDs;
}

#========================================================================================
# function     getAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aObjectattributes = $Driver->getAttributes($Shop);
#----------------------------------------------------------------------------------------
# description  Return list of exportable attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $aAttributes | list of attributes | ref.array.object
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Shop) = @_;

    my $ProductTypeClass = LoadClassByAlias('ProductType');
    my @Attributes;
    push @Attributes, {
        'Class' => $ProductTypeClass,
        'Attribute' => $ProductTypeClass->attribute($_)
    } foreach qw (Alias Name);

    my $ProductAttributeClass = LoadClassByAlias('ProductAttribute');

    # EPG-26126: # EPG-26126: Add attribute 'IsSearchFilter' to exportlist.
    my @Attr = qw (Alias Name Type IsVisible IsHTML Position Description ShowInOwnTab);

    my $hParams = {
        'Object' => $Shop,
        'Attributes' => \@Attr,
    };
    TriggerHook('API_ProductTypeExportImportAttributes', $hParams);

    push @Attributes, {
        'Class' => $ProductAttributeClass,
        'Attribute' => $ProductAttributeClass->attribute($_)
    } foreach @{$hParams->{'Attributes'}};

    my $PreDefAttributeClass = LoadClassByAlias('PreDefAttribute');
    push @Attributes, {
        'Class' => $PreDefAttributeClass,
        'Attribute' => $PreDefAttributeClass->attribute($_)
    } foreach qw (Alias ValueLocString ValueString Position);

    return \@Attributes;
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);

    my @CSVAttributes;
    foreach my $hAttribute ( @{ $self->getAttributes($Shop) } ) {
        push @CSVAttributes, @{$self->_productTypeAttributes($hAttribute->{'Attribute'}, $hAttribute->{'Class'}, $hLocale, $ahLanguages)};
    }
    # Design
    push @CSVAttributes, @{$self->_templateTypeAttributes($hLocale)};

    my @FilteredCSVAttributes = @CSVAttributes;

    # filter unneeded attributes for unity shops (see: @UNITY_PRODUCT_TYPE_ATTRIBUTES_IGNORE)
    if($Shop->get('IsUnityRegistered')){
        @FilteredCSVAttributes = ();
        foreach my $hAttribute (@CSVAttributes) {
            push @FilteredCSVAttributes, $hAttribute unless ElementOf($hAttribute->{'Alias'}, UNITY_PRODUCT_TYPE_ATTRIBUTES_IGNORE);
        }
    }

    return \@FilteredCSVAttributes;
}

sub _productTypeAttributes {
    my $self = shift;
    my ($Attribute, $Class, $hLocale, $ahLanguages) = @_;
    my @CSVAttributes;
    my %Info;
    my $Alias = $Attribute->alias;
    $Info{$_} = $Attribute->get($_) foreach qw (Type IsObject);
    $Info{'Alias'} = $Alias;
    $Info{'Name'} = $Attribute->get('NameOrAlias', $hLocale->{'LanguageID'});
    $Info{'Attribute'} = $Attribute;
    $Info{'Class'} = $Class;
    $Info{'IsBaseClassAttribute'} = $Class->alias eq 'ProductType' ? 1 : 0;

    if ($Info{'Type'} eq 'LocalizedString') {
        foreach my $hLanguage (@$ahLanguages) {
            $Info{'LanguageID'} = $hLanguage->{LanguageID};
            my $hHeaderStrings = $self->getHeaderStrings( \%Info, $hLocale->{'LanguageID'}, { 'hLanguage' => $hLanguage});
            $Info{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
            $Info{'AliasString'}  = $hHeaderStrings->{'AliasString'};
            push @CSVAttributes, {%Info};
        }
    } else {
        my $hHeaderStrings = $self->getHeaderStrings( \%Info, $hLocale->{'LanguageID'}, {});
        $Info{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
        $Info{'AliasString'}  = $hHeaderStrings->{'AliasString'};
        push @CSVAttributes, {%Info};
    }
    return \@CSVAttributes;
}

sub _templateTypeAttributes {
    my $self = shift;
    my ($hLocale) = @_;

    my @CSVAttributes;
    my %Info;
    foreach my $TemplateTypeAlias (qw (Content-Base Content-Variation Content-Bundles)) {
        my $ObjectPath = '/PageTypes/'.PRODUCT_PAGETYPE.'/TemplateTypes/'.$TemplateTypeAlias;
        my $TemplateType = LoadObjectByPath($ObjectPath);
        $Info{'Alias'} = $TemplateTypeAlias;
        $Info{'Name'} = $TemplateType->get('NameOrAlias', $hLocale->{'LanguageID'});
        $Info{'IsBaseClassAttribute'} = 1;
        my $hHeaderStrings = $self->getHeaderStrings( \%Info, $hLocale->{'LanguageID'}, {});
        $Info{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
        $Info{'AliasString'}  = $hHeaderStrings->{'AliasString'};
        push @CSVAttributes, {%Info};
    }
    return \@CSVAttributes;
}

1;
