#========================================================================================
# package      DE_EPAGES::Product::API::CSV::ProductTypeImportHandler;
# base         DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to import producttypes
# example      use DE_EPAGES::Product::API::CSV::ProductTypeImportHandler;
#               DE_EPAGES::Product::API::CSV::ProductTypeImportHandler->importCSV($FileName, $hFileFormat, $Shop);
#========================================================================================
package DE_EPAGES::Product::API::CSV::ProductTypeImportHandler;
use base qw ( DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler Exporter);

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode ExistsLanguageByCode );
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );
use DE_EPAGES::Product::API::Constants qw( PRODUCT_ATTRIBUTE_TYPES PRODUCT_PAGETYPE );
use DE_EPAGES::Product::API::ProductAttribute qw( CheckProductAttributeAlias );

#ERRORDEFINITION
sub ErrorDefinition {
    my $hSuperErrorDefinition = __PACKAGE__->SUPER::ErrorDefinition();
    return {
        %$hSuperErrorDefinition,
        'NO_MAND_VALUE'         => 'line #Line, column #Attribute : has empty value (but is mandatory).',
        'WRONG_TEMPLATE'        => 'line #Line, column #Attribute : child \'#Value\' does not exist.',
        'ATTRIBUTE_EXISTS'      => 'line #Line, column #Attribute : child \'#Value\' still exists.',
        'WRONG_PRODUCTATTRIBUTE_TYPE' => 'line #Line, column #Attribute : type \'#Value\' does not exist.',
    }
}
#ERRORDEFINITION

#========================================================================================
# function     analyzeAttributeLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hAttributeValues = $self->analyzeAttributeLine();
#----------------------------------------------------------------------------------------
# description  Analyzes the current line in the CSV file and fills the object data hash
#               with information for the actual import.
#               Uses $self->topID('Item') as input, which contains the list of values in
#               the current line.
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | properties of a object of CSV file with following keys
#               <ul>
#                 <li>DirectAttributes  - direct attributes of the object with following keys
#                   <ul>
#                       <li>Alias - alias of the object - string</li>
#                       <li>Class - class object of the object - object</li>
#                       <li>Parent - parent object of the object - object</li>
#                       <li> more direct attributes which should set via $Object->set method,
#                            or can used by $Class->insert($hAttributeValues->{'DirectAttributes'});</li>
#                   </ul> - ref.hash
#                 </li>
#                 <li>DoNotImport -  0 or 1, set to 1 if there are errors to set the parsed values - boolean</li>
#                 <li>Object - object if it is found or undef if it doesn't exist - object</li>
#                 <li>LocalizedAttributes -localized attributes hash with language id as key - ref.hash</li>
#                 <li>Design - attributes hash with template information - ref.hash</li>
#                 <li>ProductAttribute - hash with attributes for productattribute - ref.hash
#                   <ul>
#                       <li>DirectAttributes  - direct attributes of the object - ref.hash</li>
#                       <li>LocalizedAttributes -localized attributes hash with language id as key - ref.hash</li>
#                       <li>PreDefAttribute - hash with predef attribute for productattribute
#                           <ul>
#                               <li>DirectAttributes  - direct attributes of the object - ref.hash</li>
#                               <li>LocalizedAttributes -localized attributes hash with language id as key - ref.hash</li>
#                           </ul>
#                       </li>
#                   </ul>
#                 </li>
#               </ul>
#               | ref.hash
#========================================================================================
sub analyzeAttributeLine {
    my $self = shift;
    my $DoNotImport = 0;

    my $aItem = $self->topID('Item');
    my $hImportAttributes = $self->topID('ImportAttributes');

    # get alias for producttype, productattribute and predefattribute
    my $ProductTypeAlias = $aItem->[$hImportAttributes->{'AliasColumn'}];
    $ProductTypeAlias =~ s/\s+$//; $ProductTypeAlias =~ s/^\s+//;
    my $ProductAttributeAlias = exists $hImportAttributes->{'AliasProductAttributeColumn'} ? $aItem->[$hImportAttributes->{'AliasProductAttributeColumn'}] : '';
    $ProductAttributeAlias =~ s/\s+$//; $ProductAttributeAlias =~ s/^\s+//;
    my $PreDefAttributeAlias = exists $hImportAttributes->{'AliasPreDefAttributeColumn'} ? $aItem->[$hImportAttributes->{'AliasPreDefAttributeColumn'}] : '';
    $PreDefAttributeAlias =~ s/\s+$//; $PreDefAttributeAlias =~ s/^\s+//;

    my $Shop = $self->object;
    my (%AttributeValues, %ProductAttributeValues, %PreDefAttributeValues);
    my $BaseClass = $Shop->get('ProductClass');
    my ($ProductType, $ProductAttribute, $PreDefAttribute);

    # get producttype, if no alias is given use ProductClass of shop.
    unless ($ProductTypeAlias =~ /\S/) {
        $ProductType = $BaseClass;
        $aItem->[$hImportAttributes->{'AliasColumn'}] = $ProductType->alias;
        $AttributeValues{'DirectAttributes'}{'Parent'} = $Shop;
    } else {
        my $ProductTypesFolder = $Shop->folder('ProductTypes');
        $ProductType = $ProductTypesFolder->existsChild($ProductTypeAlias) ? $ProductTypesFolder->child($ProductTypeAlias) : undef;
        $AttributeValues{'DirectAttributes'}{'Parent'} = $ProductTypesFolder;
    }
    # exists productattribute?
    if (defined $ProductType && $ProductAttributeAlias =~ /\S/) {
        my $AttributeFolder = $ProductType->folder('Attributes');
        $ProductAttribute = $AttributeFolder->child($ProductAttributeAlias) if $AttributeFolder->existsChild($ProductAttributeAlias);
        # exists predefattribute?
        if (defined $ProductAttribute && $PreDefAttributeAlias =~ /\S/) {
            my $PreDefFolder = $ProductAttribute->folder('PreDefAttributes');
            $PreDefAttribute = $PreDefFolder->child($PreDefAttributeAlias) if $PreDefFolder->existsChild($PreDefAttributeAlias);
        }
    }

    # loop attributes
    for (my $i=0; $i<scalar(@$aItem) ;$i++) {
        my $hAttribute =  $hImportAttributes->{$i};

        # ignore unknown attribute columns (see also DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler::getUsedAttribute)
        next if exists $hAttribute->{'IGNORE_COLUMN'};


        # no classalias for TemplateTypeAttributes (Content-Base Content-Variation Content-Bundles)
        my $ClassAlias = exists $hAttribute->{'Class'} ? $hAttribute->{'Class'}->alias : '';
        my $Value = $aItem->[$i];
        if( defined $Value ) {
            $Value =~ s/\s+$//; $Value =~ s/^\s+//; $Value = undef if $Value eq '';
        }
        # split attributes to producttype, productattribute and predefattribute and check value
        if ($ClassAlias eq 'ProductAttribute' && $ProductAttributeAlias =~ /\S/) {
            $DoNotImport = $self->_analyzeAttributeLine($hAttribute, $Value, \%ProductAttributeValues, $DoNotImport);
            # check type of productattribute (Localizedstring, PreDefString, Float, ...)
            if ($hAttribute->{'Alias'} eq 'Type') { # missing value for type
                my $ProductAttributeTypes = PRODUCT_ATTRIBUTE_TYPES();
                if (!defined $Value) {
                    Warning( 'NO_MAND_VALUE', {
                        Line => $self->lineNumber,
                        Attribute => $hAttribute->{'AliasString'},
                    });
                    $DoNotImport = 1;
                } elsif (! exists $ProductAttributeTypes->{$Value}) { # unknown type
                    Warning( 'WRONG_PRODUCTATTRIBUTE_TYPE', {
                        Line => $self->lineNumber,
                        Attribute => $hAttribute->{'AliasString'},
                        Value => $Value,
                    });
                    $DoNotImport = 1;
                } else { # set package
                    $ProductAttributeValues{'DirectAttributes'}{'Package'} = $ProductAttributeTypes->{$Value}->{'Package'};
                    $ProductAttributeValues{'DirectAttributes'}{'IsObject'} = $ProductAttributeTypes->{$Value}->{'IsObject'};
                }
            }
        } elsif ($ClassAlias eq 'PreDefAttribute' && $PreDefAttributeAlias =~ /\S/) {
            $DoNotImport = $self->_analyzeAttributeLine($hAttribute, $Value, \%PreDefAttributeValues, $DoNotImport);
        } elsif ($ClassAlias eq 'ProductType' || $ClassAlias eq '') {
            if ($ClassAlias eq '') { #TemplateTypeAttributes
                $AttributeValues{'Design'}{$hAttribute->{'Alias'}} = $Value;
            } else {
                $DoNotImport = $self->_analyzeAttributeLine($hAttribute, $Value, \%AttributeValues, $DoNotImport);
            }
        }
        last if $DoNotImport;
    }
    $AttributeValues{'DoNotImport'}  = $DoNotImport;
    $AttributeValues{'Object'}  = $ProductType;
    $AttributeValues{'DirectAttributes'}{'BaseClass'} = $BaseClass if !defined $ProductType || $BaseClass->id ne $ProductType->id;
    $AttributeValues{'DirectAttributes'}{'Package'} = 'DE_EPAGES::Product::API::Object::Product';
    if (exists $ProductAttributeValues{'DirectAttributes'}{'Alias'}) { # add productattribute values to producttype
        if (exists $PreDefAttributeValues{'DirectAttributes'}{'Alias'}) { # add predefattribute values to productattribute
            $PreDefAttributeValues{'Object'} = $PreDefAttribute;
            $ProductAttributeValues{'PreDefAttribute'} = \%PreDefAttributeValues;
        }
        $ProductAttributeValues{'Object'} = $ProductAttribute;
        $ProductAttributeValues{'DirectAttributes'}{'IsUserDefined'} = 1;
        $AttributeValues{'ProductAttribute'} = \%ProductAttributeValues;
    }
    return \%AttributeValues;
}

# sets the value to $hAttributeValues (fills the object data hash)
sub _analyzeAttributeLine {
    my $self = shift;
    my ($hAttribute, $Value, $hAttributeValues, $DoNotImport) = @_;
    my $AttributeType = $hAttribute->{'Type'};
    if (not defined $Value and exists $hAttribute->{'Attribute'} and $hAttribute->{'Attribute'}->get('IsMandatory')) {
        Warning( 'NO_MAND_VALUE', {
            Line => $self->lineNumber,
            Attribute => $hAttribute->{'AliasString'}
        });
        return 1;
    }
    $hAttributeValues->{'DirectAttributes'}{'Class'} = $hAttribute->{'Class'};
    if ($AttributeType =~ /^Localized/) {
        $hAttributeValues->{'LocalizedAttributes'}{$hAttribute->{'LanguageID'}}{$hAttribute->{'Alias'}} = $Value;
    } else {
        $hAttributeValues->{'DirectAttributes'}{$hAttribute->{'Alias'}} = $Value;
    }
    return $DoNotImport;
}

#========================================================================================
# function     importObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importObject;
#----------------------------------------------------------------------------------------
# description  calls $hAttributeValues = $self->analyzeAttributeLine,
#               return if (not defined $hAttributeValues or $hAttributeValues->{'DoNotImport'})
#               if the object exists, it should be at $hAttributeValues->{'Object'}
#               then a $Object->set($hAttributeValues->{'DirectAttributes'}) is called
#               else $Class must exists $hAttributeValues->{'DirectAttributes'}{'Class'}
#               then calls $Class->insertObject($hAttributeValues->{'DirectAttributes'});
#               then calls foreach language id keys %{ $hAttributeValues->{'LocalizedAttributes'}} :
#               my $hInfo = $hAttributeValues->{'LocalizedAttributes'}{$LanguageID};
#               $Object->set($hInfo, $LanguageID);
#               calls function $self->importSpecialData($Object, $hAttributeValues); to give the possibility
#               to save more data
#========================================================================================
sub importObject {
    my $self = shift;

    my $hAttributeValues = $self->analyzeAttributeLine;
    return if (not defined $hAttributeValues or $hAttributeValues->{'DoNotImport'});

    my $ProductType = $self->_importObject($hAttributeValues); # import / update producttype
    if (defined $ProductType && exists $hAttributeValues->{'ProductAttribute'}) {
        my $hProductAttributeValues = $hAttributeValues->{'ProductAttribute'};
        eval {
            CheckProductAttributeAlias($ProductType, $hProductAttributeValues->{'DirectAttributes'}{'Alias'}, $hProductAttributeValues->{'Object'});
            $hProductAttributeValues->{'DirectAttributes'}{'Parent'} = $ProductType->folder('Attributes');
            my $ProductAttribute = $self->_importObject($hProductAttributeValues); # import / update productattribute
            if (defined $ProductAttribute && exists $hProductAttributeValues->{'PreDefAttribute'} && $ProductAttribute->get('Type') =~ /^PreDef.*/) {
                my $hPreDefAttributeValues = $hProductAttributeValues->{'PreDefAttribute'};
                $hPreDefAttributeValues->{'DirectAttributes'}{'Parent'} = $ProductAttribute->folder('PreDefAttributes');
                $self->_importObject($hPreDefAttributeValues); # import / update predefattribute
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless(ElementOf($Error->code, qw (AliasReserved AliasWithUmlauts ProductAttributeExistsInOtherProductClass ProductAttributeExists)));
            Warning( 'ATTRIBUTE_EXISTS', {
                Line => $self->lineNumber,
                Attribute => 'Alias',
                Value => $hProductAttributeValues->{'DirectAttributes'}{'Alias'},
            });
        }
    }

    $self->importSpecialData($ProductType, $hAttributeValues->{'Design'}) if defined $ProductType;

    $self->_modifyUnityProductType($ProductType) if defined $ProductType and $ProductType->getSite()->get('IsUnityRegistered');

    return;
}

sub _modifyUnityProductType {
    my $self = shift;
    my ($ProductType) = @_;

    # in an unity shop we have only producttypeattribute types LocalizedString and PreDefLocalizedString
    my $aAttributes = $ProductType->get('UserDefinedAttributes');

    #map String and PreDefString to LocalizedString and PreDefLocalizedString
    # and delete all other attributes
    my $ProductAttributeTypes = PRODUCT_ATTRIBUTE_TYPES();
    foreach my $Attribute (@$aAttributes) {
        my $AttributeType = $Attribute->get('Type');

        $Attribute->set({'Type' => 'LocalizedString',
                         'Package' => $ProductAttributeTypes->{'LocalizedString'}->{'Package'}}
                       ) if $AttributeType eq 'String';

        $Attribute->set({'Type' => 'PreDefLocalizedString',
                         'Package' => $ProductAttributeTypes->{'PreDefLocalizedString'}->{'Package'}}
                       ) if $AttributeType eq 'PreDefString';

        # delete all other attributes without the correct type
        $Attribute->delete() unless ElementOf($Attribute->get('Type'), qw(LocalizedString PreDefLocalizedString));
    }

}

sub _importObject {
    my $self = shift;
    my ($hAttributeValues) = @_;

    my $Object = $hAttributeValues->{'Object'};
    my $hDirectAttributes = $hAttributeValues->{'DirectAttributes'};
    my $hOldObjectAttributes;

    my $Class = $hDirectAttributes->{'Class'};
    delete $hDirectAttributes->{'Class'};
    if (defined $Object ) {
        # store former attributes
        foreach my $Attribute ( keys %$hDirectAttributes ) {
            $hOldObjectAttributes->{$Attribute} = $Object->get($Attribute);
        }
        $Object->set( $hDirectAttributes );
    } else {
        eval {
            $Object  = $Class->insertObject($hDirectAttributes);
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $ErrorCode = $Error->code;
            if ($ErrorCode =~ /FeatureLimitExceeded/ ){
                $Error->addVars( { Line => $self->lineNumber } );
                Warning($Error);
                return;
            } else {
                Error($Error);
            }
        }
    }

    # test feature for unity
    my $Shop = $Object->getSite();
    if ($Shop->get('IsUnityRegistered')){
        eval {
            $self->_testAttributeValueFeature($Object, $Shop);
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $ErrorCode = $Error->code;
            if ($ErrorCode =~ /VariationAttributeValuesFeatureLimitExceeded/ ){
                $Error->addVars( { Line => $self->lineNumber } );
                Warning($Error);
                return;
            } else {
                Error($Error);
            }
        }
    }

    # update localized attributes
    foreach my $LanguageID (keys %{ $hAttributeValues->{'LocalizedAttributes'}}) {
        my $hInfo = $hAttributeValues->{'LocalizedAttributes'}{$LanguageID};
        $Object->set($hInfo, $LanguageID);

    }

    return $Object;
}


sub _testAttributeValueFeature {
    my $self = shift;
    my ($Object, $Shop) = @_;

    return unless $Object->instanceOf('ProductAttribute');
    return unless $Object->get('Type') eq 'PreDefLocalizedString';

    # for Unity we test also the max count of attribute values for variation atributes
    my $MaxValue = $Shop->featureMaxValue( 'VariationAttributeValues' );
    my $NewValue = 1 + scalar @{$Object->get('PreDefAttributes')};
    if ($NewValue > $MaxValue) {
        Error( 'VariationAttributeValuesFeatureLimitExceeded', {
            Feature  => 'VariationAttributeValues',
            MaxValue => $MaxValue,
            NewValue => $NewValue,
            Shop => $Shop
        });
    }
}

#========================================================================================
# function     testHeaderColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testHeaderColumn($hAttribute, $hHeader, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  possibility to test  $hHeader, called by getUsedAttribute
#               only dummy function here
#----------------------------------------------------------------------------------------
# input        $hAttribute | used attribute | ref.hash
# input        $hHeader    | current header info | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub testHeaderColumn {
    my $self = shift;
    my ($hAttribute, $hHeader, $ColumnIndex) = @_;

    my $aHeaderParts = $hHeader->{'HeaderParts'};
    my $HeaderString = $hHeader->{'HeaderString'};

    my $Type = $hAttribute->{'Type'};
    if (defined $Type && $Type =~ /^Localized(String|File)$/) {
        if (scalar(@$aHeaderParts) != 3) {
            Error('NO_SHOP_LANG',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'}});
        } elsif (not ExistsLanguageByCode($aHeaderParts->[2])) {
            Error('NO_SHOP_LANG',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'}});
        }
        my $LangID = GetPKeyLanguageByCode($aHeaderParts->[2]);
        if (not $self->object->existsLanguage($LangID)) {
            Error('NO_SHOP_LANG',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'}});
        }
    } elsif (scalar(@$aHeaderParts) != 2) {
        Error('WRONG_FORMAT',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'}});
    }
    return;
}

#========================================================================================
# function     modifyUsedAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->modifyUsedAttributes($hUsedAttributes, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  add keys:
#                 $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Alias'  ? 1 : 0;
#                 $hUsedAttributes->{'AliasColumn'} with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'};
#                 $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Alias'  ? 1 : 0;
#                 $hUsedAttributes->{'AliasProductAttributeColumn'} with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeAliasColumn'};
#                 $hUsedAttributes->{$ColumnIndex}{'IsPreDefAttributeAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Alias'  ? 1 : 0;
#                 $hUsedAttributes->{'AliasPreDefAttributeColumn'} with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsPreDefAttributeAliasColumn'};
#                 $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeTypeColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Type'  ? 1 : 0;
#                 $hUsedAttributes->{'TypeProductAttributeColumn'} with value=$ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeTypeColumn'};
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub modifyUsedAttributes {
    my $self = shift;
    my ( $hUsedAttributes, $ColumnIndex) = @_;
    my $ClassAlias = exists $hUsedAttributes->{$ColumnIndex}{'Class'} ? $hUsedAttributes->{$ColumnIndex}{'Class'}->alias : '';
    if ($ClassAlias eq 'ProductType') {
        $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Alias'  ? 1 : 0;
        $hUsedAttributes->{'AliasColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'};
    } elsif ($ClassAlias eq 'ProductAttribute') {
        $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Alias'  ? 1 : 0;
        $hUsedAttributes->{'AliasProductAttributeColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeAliasColumn'};
        $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeTypeColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Type'  ? 1 : 0;
        $hUsedAttributes->{'TypeProductAttributeColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsProductAttributeTypeColumn'};
    } elsif ($ClassAlias eq 'PreDefAttribute') {
        $hUsedAttributes->{$ColumnIndex}{'IsPreDefAttributeAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Alias'  ? 1 : 0;
        $hUsedAttributes->{'AliasPreDefAttributeColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsPreDefAttributeAliasColumn'};
    }
    return;
}

#========================================================================================
# function     testMissingFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testMissingFields($hUsedAttributes);
#----------------------------------------------------------------------------------------
# description  throws error 'MISSING_FIELD', if field Alias is missing
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
#========================================================================================
sub testMissingFields {
    my $self = shift;
    my ( $hUsedAttributes ) = @_;
    $self->SUPER::testMissingFields($hUsedAttributes);
    if( exists $hUsedAttributes->{'AliasProductAttributeColumn'} && ! exists $hUsedAttributes->{'TypeProductAttributeColumn'} ) {
        Error( 'MISSING_FIELD' => { 'Value' => 'Type', 'FileName' => $self->{'FileName'} });
    }
}

#========================================================================================
# function     baseAttributeClassAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->baseAttributeClassAlias();
#----------------------------------------------------------------------------------------
# description  return Base AttributeClass Alias, here : 'ProductType'
#----------------------------------------------------------------------------------------
# return        $BaseAttributeClassAlias | attribute class alias of normal attributes | string
#========================================================================================
sub baseAttributeClassAlias {
    my $self = shift;
    return 'ProductType';
}

#========================================================================================
# function     importSpecialData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importSpecialData($Object, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  possibility to import special object data, called by importObject
#               saves template design
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hAttributeValues    | attribute values | ref.hash
#========================================================================================
sub importSpecialData {
    my $self = shift;
    my ( $Object, $hAttributeValues ) = @_;

    my $PageType = LoadPageTypeByAlias( PRODUCT_PAGETYPE );

    foreach my $TemplateName (keys %{$hAttributeValues}) {
        my $TemplateFolder = $PageType->templateTypeFolder($TemplateName);

        my $TemplateTypeName = $hAttributeValues->{$TemplateName};
        next unless defined $TemplateTypeName;

        # show select box at base template
        if ($TemplateName eq 'Content-Variation') {
            my $VariationTemplateType = $PageType->templateTypeFolder('Content-Base-SelectVariation')->child(
                $TemplateTypeName eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $Object, 'Content-Base-SelectVariation', $VariationTemplateType->alias );

            # special Content-Variation treatment for new templates (EPG-17552), this will be removed in 6.15.0
            my $VariationTemplateTypeNew = $PageType->templateTypeFolder('Content-PriceBox-SelectVariation')->child(
                $TemplateTypeName eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $Object, 'Content-PriceBox-SelectVariation', $VariationTemplateTypeNew->alias );

            # use not visible Content-Variation-New template type folder
            $PageType->setTemplateType( $Object, 'Content-Variation-New', $TemplateTypeName.'_New' );
        }
        elsif ($TemplateName eq 'Content-Bundles') {
            # special Content-Bundles treatment for new templates (EPG-17552), this will be removed in 6.15.0
            # use not visible Content-Bundles-New template type folder
            $PageType->setTemplateType( $Object, 'Content-Bundles-New', $TemplateTypeName.'_New' );
        }

        $PageType->setTemplateType( $Object, $TemplateFolder->alias, $TemplateTypeName );
    }

    $Object->updatePageCache;
    return;
}

1;
