#========================================================================================
# package      DE_EPAGES::Product::API::Object::LineItemDeposit
# state        public
# base         DE_EPAGES::Order::API::Object::LineItem
# description  object interface for LineItemDeposit.
#========================================================================================
package DE_EPAGES::Product::API::Object::LineItemDeposit;
use base qw( DE_EPAGES::Order::API::Object::LineItem );

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemDeposit = DE_EPAGES::Order::API::Object::LineItemDeposit->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemDeposit');
#               my $LineItemDeposit = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemDeposit' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemDeposit | new object | object
# hook         OBJ_InsertLineItemDeposit | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item eco participation id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertLineItemDeposit', {'Object' => $self, 'LineItemID' => $self->id });
    return $self;
}

#========================================================================================
# function     clone
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Clone = $LineItem->clone( $hInfo );
# syntax       $Clone = $LineItem->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new LineItem into the database as copy from old LineItem.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;
    if (!$hInfo->{'Parent'}->parent->instanceOf('PackingSlip')) {
        return $self->SUPER::clone($hInfo, $Recursive);
    }
    return;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemDeposit->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemDeposit' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemDeposit | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID - line item eco Participation id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemDeposit', {'Object' => $self, 'LineItemID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemDeposit->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemDeposit'
#               and 'OBJ_AfterUpdateLineItemDeposit'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemDeposit | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id of eco participation - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemDeposit | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id of eco participation - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    if ($hValues->{'BasePrice'}) {
        my $Container = $self->get('LineItemContainer');
        if ($Container->parent->parent->alias eq 'EditableOrders') {
                my $PriceQuantity = $self->get('PriceQuantity');

                if(defined $PriceQuantity and $PriceQuantity != 0) {
                    $hValues->{'BasePrice'} = $hValues->{'BasePrice'} / $PriceQuantity;
                }
        }
    }

    TriggerHook('OBJ_BeforeUpdateLineItemDeposit', {'Object'=>$self, 'Values'=> $hValues, 'LineItemID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemDeposit',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     recalculate
#----------------------------------------------------------------------------------------
# syntax       $LineItem->recalculate($Container);
#----------------------------------------------------------------------------------------
# description  Recalculate global value discount item or delete it, if it is not valid anymore.
#----------------------------------------------------------------------------------------
# input        $Container | container | object
#========================================================================================
sub recalculate {
    my $self = shift;
    my ($Container) = @_;

    # set new baseprice e.g we have a gross2net switch
    my $CurrencyID = $Container->get('CurrencyID');
    my $TaxModel = $Container->get('TaxModel');
    my $Deposit = $self->get('Product')->getDeposit($CurrencyID, $TaxModel);
    $self->delete unless $Deposit;
    my $BasePrice = $Deposit / $self->get('Product')->getWithParent('PriceQuantity');

    $self->set({
        'BasePrice' => $BasePrice,
        'LineItemPrice' => $BasePrice * $self->get('Quantity'),
    });
    return;
}

#========================================================================================
# function     isStatic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isStatic = DE_EPAGES::Order::API::Object::LineItemSales->isStatic();
# example      $isStatic = $LineItem->isStatic;
#----------------------------------------------------------------------------------------
# description  Returns true if line item is static. That means the lineitemprice can't
#               be modified by discounts.
#----------------------------------------------------------------------------------------
# return       $isStatic | Is a static line item. | boolean
#========================================================================================
sub isStatic {
    my $self = shift;
    return 1;
}

1;
