#==============================================================================
# package      DE_EPAGES::Product::API::WebService::ProductTypeService
# base         DE_EPAGES::Content::API::WebService::BaseContentService
# state        public
# description  Implements the Product Type WebService.
#==============================================================================
package DE_EPAGES::Product::API::WebService::ProductTypeService;
use base DE_EPAGES::Content::API::WebService::BaseContentService;

use strict;
use DE_EPAGES::Core::API::Error qw (Error ResetError ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(GetLog LogDebug);
use DE_EPAGES::Object::API::Factory qw (Transaction LoadClassByAlias);
use DE_EPAGES::Product::API::Table::SelectedVariation qw (GetPKeySelectedVariationsByAttributeID
                                                          GetPKeySelectedVariationsByPreDefAttributeID);
use DE_EPAGES::Object::API::Table::SelectPreDefAttribute qw (GetPKeySelectPreDefAttributesByAttributeID
                                                             GetPKeySelectPreDefAttributesByPreDefAttributeID);
use DE_EPAGES::Product::API::Constants qw (PRODUCT_PAGETYPE PRODUCT_ATTRIBUTE_TYPES);
use DE_EPAGES::Product::API::ProductAttribute qw (CheckProductAttributeAlias);
use DE_EPAGES::Presentation::API::PageCache qw (UpdatePageCache);
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByAlias);


#ERRORDEFINITION
sub ErrorDefinition {
    return {
    'ProductTypeNoDelete'                     => 'The product type #Alias must not be deleted because products of this type still exist.',
    'BaseProductTypeNoDelete'                 => 'The base product type must no be deleted.',
    'ProductTypeAlreadyExists'                => 'Product type #Alias does already exist.',
    'NoResults_Template'                      => 'Template #Name does not exist for PageType #PageType.',
    'NoResults_Attribute'                     => 'Product attribute #Alias does not exist for product type #ProductType.',
    'ProductsWithThisVariationAttributeExist' => 'Product attribute #Alias of product type #ProductType can not be deleted as variation products with this attribute still exist.',
    'UnkownAttributeType'                     => 'Unknown product attribute type #Type. Use one of #ValidTypes.',
    'NoResults_PreDefAttribute'               => 'Predefined attribute #Alias does not exist for product attribute #ProductAttribute.',
    'NoPreDefAttributeType'                   => 'Attribute #Path can not be used with predefined attributes.',
    'ProductsWithThisPreDefAttributeExist'    => 'Predefined attribute #Alias of product attribute #ProductAttribute can not be deleted as it is still used by variation products.',
    'PreDefAttributeExists'                   => 'A predefined attribute with alias #Alias already exists for product attribute #ProductAttribute.'
    }
}
#ERRORDEFINITION END

sub new { ## no critic qw( RequireArgUnpacking )
    shift->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ProductTypeService/2007/10',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/ProductTypeTypes/2007/10',
        @_,
    ));
}

#========================================================================================
# function     getBaseProductType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getBaseProductType();
#----------------------------------------------------------------------------------------
# description  WebService-Method: getBaseProductType
#               Returns the path of the base product type.
#----------------------------------------------------------------------------------------
# return       $SoapData  | a SOAP::Data string containing the path
#               of the base product type | SOAP::Data
#========================================================================================
sub getBaseProductType {
    my $self = shift;
    return SOAP::Data->type('string')->name('BaseProductType' => $self->site->get('ProductClass')->get('Path'));
}

#========================================================================================
# function     getInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getInfo($aProductTypes, $aAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about a list of product types.
#----------------------------------------------------------------------------------------
# input        $aProductTypes | list of product type paths | ref.array.string
# input        $aAttributes | list of AttributeIDs | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given product types | SOAP::Data
#========================================================================================
sub getInfo {
    my $self = shift;

    # expects a list of product type paths
    my ($aProductTypes, $aAttributes, $aLanguageCodes) = @_;
    LogDebug('aProductTypes=', $aProductTypes);
    LogDebug('aAttributes=', $aAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my @soapProductTypes = ();

    foreach my $productTypePath (@$aProductTypes) {
        my $soapProductType;

        eval {
            # get object
            my $ProductType = $self->loadShopObject($productTypePath, 'ProductType');
            $soapProductType = $self->buildSoapProductType($ProductType, $aAttributes, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductType = SOAP::Data->type( 'TGetInfo_Return')->name('ProductType' => {
                'Path'  => $self->soapString( $productTypePath ),
                'Error' => $self->soapError($Error)
            });
            ResetError();
        }
        push(@soapProductTypes, $soapProductType);

    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TGetInfo_Return') );

    $self->mapType([qw (TGetInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     getAllInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getAllInfo($aAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getAllInfo
#               Returns information about all existing product types of the current shop
#----------------------------------------------------------------------------------------
# input        $aAttributes | list of AttributeIDs | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with
#                            info of all existing product types | SOAP::Data
#========================================================================================
sub getAllInfo {
    my $self = shift;

    # expects a list of product type paths
    my ($aAttributes, $aLanguageCodes) = @_;
    LogDebug('aAttributes=', $aAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $Shop = $self->site;
    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my @soapProductTypes = ();

    foreach my $ProductType ($Shop->get('ProductClass'), @{$Shop->get('ProductTypes')->children}) {
        my $soapProductType;
        eval {
            $soapProductType = $self->buildSoapProductType($ProductType, $aAttributes, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductType = SOAP::Data->type( 'TGetInfo_Return')->name('ProductType' => {
                'Path'  => $self->soapString( $ProductType->get('Path') ),
                'Error' => $self->soapError($Error)
            });
            ResetError();
        }
        push(@soapProductTypes, $soapProductType);
    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TGetInfo_Return') );

    $self->mapType([qw (TGetInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->exists($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: exists
#               Returns exists status of a list of product types.
#----------------------------------------------------------------------------------------
# input        $aProductTypes | list of product type paths | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                exist status of all given product types | SOAP::Data
#========================================================================================
sub exists {
    my $self = shift;

    # expects a list of product type paths
    my ($aProductTypes) = @_;
    LogDebug('aProductTypes=', $aProductTypes);

    my @soapProductTypes = ();

    foreach my $productTypePath (@$aProductTypes) {
        my $soapProductType = ();

        my $hOutProductType = {
            'Path'   => $self->soapString( $productTypePath ),
            'exists' => $self->soapBool( $self->existsShopObject($productTypePath, 'ProductType') )
        };

        # build return object
        $soapProductType = SOAP::Data->type('TExists_Return')->name('ProductType' => $hOutProductType);
        push(@soapProductTypes, $soapProductType);
    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TExists_Return') );

    $self->mapType([qw (TExists_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->delete($aProductTypess);
#----------------------------------------------------------------------------------------
# description  WebService-Method: delete
#               Deletes a list of product types.
#----------------------------------------------------------------------------------------
# input        $aProductTypes | list of product type paths | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                delete status of all given product types | SOAP::Data
#========================================================================================
sub delete {
    my $self = shift;

    # expects a list of product type paths
    my ($aProductTypes) = @_;
    LogDebug('aProductTypes=', $aProductTypes);

    my $Shop = $self->site;
    my @soapProductTypes = ();

    foreach my $productTypePath (@$aProductTypes) {
        my $soapProductType;

        eval {
            # get object
            my $ProductType = $self->loadShopObject($productTypePath, 'ProductType');

            # base product type must not be deleted
            Error('BaseProductTypeNoDelete') if ($ProductType->id == $Shop->get('ProductClass')->id);

            # product type must not be deleted if products of this type exist
            my $dal = $ProductType->packageObject->dal;
            Error('ProductTypeNoDelete', {'Alias' => $ProductType->alias})
                if scalar @{$dal->get('ByClassID',[$ProductType->id])};

            $ProductType->delete;

            my $hOutProductType = {
                'Path'    => $self->soapString( $productTypePath ),
                'deleted' => $self->soapBool( not $ProductType->exists )
            };

            # build return object
            $soapProductType = SOAP::Data->type( 'TDelete_Return')->name('ProductType' => $hOutProductType);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductType = SOAP::Data->type( 'TDelete_Return')->name('ProductType' => {
                'Path'   => $self->soapString( $productTypePath ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapProductTypes, $soapProductType);

    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TDelete_Return') );

    $self->mapType([qw (TDelete_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->update($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates a list of product types.
#----------------------------------------------------------------------------------------
# input        $aProductTypes | list of product type info (path and
#                              data to update) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given product types | SOAP::Data
#========================================================================================
sub update {
    my $self = shift;

    # expects a list of product types
    my ($aProductTypes) = @_;
    LogDebug('aProductTypes=', $aProductTypes);

    my @soapProductTypes = ();

    foreach my $hInProductType (@$aProductTypes) {
        my $soapProductType;
        my $ProductTypePath = $hInProductType->{'Path'};
        eval {
            $self->checkMandatoryFields($hInProductType, ['Path']);

            # get object
            my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');

            Transaction( sub {
                # set Product parameters
                $self->setParameters($ProductType, $hInProductType);

                # set Attributes
                $self->setAttributeValues($ProductType, $hInProductType->{'Attributes'});
            });

            #--- reset page cache if layout settings have been changed
            UpdatePageCache($ProductType) if (scalar(grep { /^Layout\./ } keys %$hInProductType) > 0);

            my $hOutProductType = {
                'Path'    => $self->soapString( $ProductTypePath ),
                'updated' => $self->soapBool( 1 ),
            };

            # build return object
            $soapProductType = SOAP::Data->type( 'TUpdate_Return')->name('ProductType' => $hOutProductType);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductType = SOAP::Data->type( 'TUpdate_Return')->name('ProductType' => {
                'Path'  => $self->soapString( $ProductTypePath ),
                'Error' => $self->soapError($Error)
            });
        }
        push(@soapProductTypes, $soapProductType);

    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TUpdate_Return') );

    $self->mapType([qw (TUpdate_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->create($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: create
#               Creates a list of product types.
#----------------------------------------------------------------------------------------
# input        $aProductTypes | list of product type info (data to update) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given product types | SOAP::Data
#========================================================================================
sub create {
    my $self = shift;

    # expects a list of products
    my ($aProductTypes) = @_;
    LogDebug('aProductTypes=', $aProductTypes);

    my $Shop = $self->site;

    my @soapProductTypes = ();
    my $Class     = LoadClassByAlias('ProductType');
    my $BaseClass = $Shop->get('ProductClass');

    foreach my $hInProductType (@$aProductTypes) {
        my $soapProductType;

        my $Alias = $hInProductType->{'Alias'};

        eval {
            $self->checkMandatoryFields($hInProductType, ['Alias']);

            my $Folder = $Shop->get('ProductTypes');

            # check if exists
            Error('ProductTypeAlreadyExists', {'Alias' => $Alias}) if ($Folder->existsChild($Alias));

            my $ProductTypePath;
            Transaction( sub {
                # create new product type
                my $ProductType = $Class->insertObject( {
                    'Alias'     => $Alias,
                    'BaseClass' => $BaseClass,
                    'Parent'    => $Folder
                } );

                # set product type parameters
                $self->setParameters($ProductType, $hInProductType);

                # set Attributes
                $self->setAttributeValues($ProductType, $hInProductType->{'Attributes'});

                $ProductTypePath = $ProductType->get('Path');
            });

            my $hOutProductType = {
                'Alias'   => $self->soapString( $Alias ),
                'Path'    => $self->soapString( $ProductTypePath ),
                'created' => $self->soapBool( 1 )
            };

            # build return object
            $soapProductType = SOAP::Data->type( 'TCreate_Return')->name('ProductType' => $hOutProductType);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductType = SOAP::Data->type( 'TCreate_Return')->name('ProductType' => {
                'Alias'      => $self->soapString( $Alias ),
                'Error'      => $self->soapError($Error)
            });
            ResetError();
        }
        push(@soapProductTypes, $soapProductType);

    }
    my $result = SOAP::Data->name( 'ProductTypes' )->value( $self->soapList(\@soapProductTypes, 'TCreate_Return') );

    $self->mapType([qw (TCreate_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     getProductAttributeInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getProductAttributeInfo($ProductTypePath, $aAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getProductAttributeInfo
#               Returns information about a list of product attributes for one product type.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given product attributes | SOAP::Data
#========================================================================================
sub getProductAttributeInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes, $aLanguageCodes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);
    my %ProductAttributes = ();

    $self->mapType([qw (TGetProductAttributeInfo_Return type_getProductAttributeInfo_Out)], $self->getServiceTypesUrn);

    eval {
        my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_getProductAttributeInfo_Out')->name('GetProductAttributeInfoOut' => {
            'Error' => $self->soapError($Error)
        });
    }


    my @soapProductAttributes = ();

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            # check if attribute exists
            if (!exists($ProductAttributes{$AttributeAlias})) {
                Error('NoResults_Attribute', {'Alias' => $AttributeAlias, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$AttributeAlias};

            # build return object
            $soapProductAttribute = $self->buildSoapProductAttribute($Attribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type( 'TGetProductAttributeInfo_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $AttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapProductAttributes, $soapProductAttribute);
    }

    my $result = SOAP::Data->type('type_getProductAttributeInfo_Out')->name('GetProductAttributeInfoOut' => {
        'ProductAttributes' => $self->soapList(\@soapProductAttributes, 'TGetProductAttributeInfo_Return')
    });

    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     getAllProductAttributesInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getAllProductAttributesInfo('/Shops/DemoShop/ProductTypes/Tent');
#               $SoapData = $self->getAllProductAttributesInfo('/Shops/DemoShop/ProductTypes/Tent', ['de', 'en']);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getAllProductAttributesInfo
#               Gets all product attributes of the product type.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            infos of all product attributes | SOAP::Data
#========================================================================================
sub getAllProductAttributesInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aLanguageCodes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my $result;

    eval {
        # get object
        my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        $result = SOAP::Data->type('type_getAllProductAttributesInfo_Out')->name('GetAllProductAttributesInfoOut' =>
            {'ProductAttributes' => $self->soapList(
                [map { $self->buildSoapProductAttribute($_, $aLanguages) }
                   @{$ProductType->directAttributes}],
                'TGetAllProductAttributesInfo_Return')
            }
        );
    };
    if (ExistsError()) {
        my $Error = GetError();
        $result = SOAP::Data->type('type_getAllProductAttributesInfo_Out')->name('GetAllProductAttributesInfoOut' => {
            'Error' => $self->soapError($Error)
        });
        ResetError();
    }

    $self->mapType([qw (TGetAllProductAttributesInfo_Return type_getAllProductAttributesInfo_Out)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     existsProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->existsProductAttribute($ProductTypePath, $aProductAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: existsProductAttribute
#               Returns exists status of a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given product attributes | SOAP::Data
#========================================================================================
sub existsProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();
    my %ProductAttributes = ();

    $self->mapType([qw (TExistsProductAttribute_Return type_existsProductAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_existsProductAttribute_Out')->name('ExistsProductAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        my $hOutProductAttribute = {
            'Alias'   => $self->soapString( $AttributeAlias ),
            'exists' => $self->soapBool( exists($ProductAttributes{$AttributeAlias}) )
        };

        # build return object
        $soapProductAttribute = SOAP::Data->type( 'TExistsProductAttribute_Return')->name('ProductAttribute' => $hOutProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }
    return SOAP::Data->type('type_existsProductAttribute_Out')->name('ExistsProductAttributeOut' => {
        'ProductAttributes' => $self->soapList(\@soapProductAttributes, 'TExistsProductAttribute_Return')
    });
}

#========================================================================================
# function     deleteProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->deleteProductAttribute($aProductTypess);
#----------------------------------------------------------------------------------------
# description  WebService-Method: deleteProductAttribute
#               Deletes a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                delete status of all given product attributes | SOAP::Data
#========================================================================================
sub deleteProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();
    my %ProductAttributes = ();
    my $ProductType;

    $self->mapType([qw (TDeleteProductAttribute_Return type_deleteProductAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_deleteProductAttribute_Out')->name('DeleteProductAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            # check if attribute exists
            if (!exists($ProductAttributes{$AttributeAlias})) {
                Error('NoResults_Attribute', {'Alias' => $AttributeAlias, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$AttributeAlias};

            # variation attribute must not be deleted if variation products exist
            if (scalar(@{GetPKeySelectedVariationsByAttributeID($Attribute->id)}) > 0  &&
                scalar(@{GetPKeySelectPreDefAttributesByAttributeID($Attribute->id)}) > 0)
            {
                Error('ProductsWithThisVariationAttributeExist', {'Alias' => $AttributeAlias, 'ProductType' => $ProductTypePath});
            }

            # delete product attribute
            $Attribute->delete;
            delete($ProductAttributes{$AttributeAlias});

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $AttributeAlias ),
                'deleted' => $self->soapBool( not $Attribute->exists )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type( 'TDeleteProductAttribute_Return')->name('ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type( 'TDeleteProductAttribute_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $AttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    return SOAP::Data->type('type_deleteProductAttribute_Out')->name('DeleteProductAttributeOut' => {
        'ProductAttributes' => $self->soapList(\@soapProductAttributes, 'TDeleteProductAttribute_Return')
    });
}

#========================================================================================
# function     updateProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->updateProductAttribute($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: updateProductAttribute
#               Updates a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute info (alias and
#                              data to update) | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given product attributes | SOAP::Data
#========================================================================================
sub updateProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();
    my %ProductAttributes = ();
    my $ProductType;

    $self->mapType([qw (TUpdateProductAttribute_Return type_updateProductAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_updateProductAttribute_Out')->name('UpdateProductAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $hAttribute (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            $self->checkMandatoryFields($hAttribute, ['Alias']);

            # check if attribute exists
            if (!exists($ProductAttributes{$hAttribute->{'Alias'}})) {
                Error('NoResults_Attribute', {'Alias' => $hAttribute->{'Alias'}, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$hAttribute->{'Alias'}};

            Transaction( sub {
                # set product attribute parameters
                $self->setProductAttributeParameters($Attribute, $hAttribute);

                # set Attributes
                $self->setAttributeValues($Attribute, $hAttribute->{'Attributes'});
            });

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $hAttribute->{'Alias'} ),
                'updated' => $self->soapBool( 1 )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type( 'TUpdateProductAttribute_Return')->name('ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TUpdateProductAttribute_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $hAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }

        # build return object
        $soapProductAttribute = SOAP::Data->type('TUpdateProductAttribute_Return')->name('ProductAttribute' => $soapProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    return SOAP::Data->type('type_updateProductAttribute_Out')->name('UpdateProductAttributeOut' => {
        'ProductAttributes' => $self->soapList(\@soapProductAttributes, 'TUpdateProductAttribute_Return')
    });
}

#========================================================================================
# function     createProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->createProductAttribute($ProductTypePath, $aProductAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: createProductAttribute
#               Creates a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute info (alias and data to set)
#               | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given product attributes | SOAP::Data
#========================================================================================
sub createProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();
    my %ProductAttributes = ();
    my $ProductType;

    $self->mapType([qw (TCreateProductAttribute_Return type_createProductAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
        %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_createProductAttribute_Out')->name('CreateProductAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    my $Class = LoadClassByAlias('ProductAttribute');

    foreach my $hAttribute (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            $self->checkMandatoryFields($hAttribute, ['Alias', 'Type']);

            # check attribute alias
            CheckProductAttributeAlias($ProductType, $hAttribute->{'Alias'});

            # check type
            my $ProductAttributeTypes = PRODUCT_ATTRIBUTE_TYPES();
            if (!exists $ProductAttributeTypes->{$hAttribute->{'Type'}}) {
                Error('UnkownAttributeType', {'Type' => $hAttribute->{'Type'}, 'ValidTypes' => join(', ', sort keys %$ProductAttributeTypes)});
            }

            my $AttributePath;
            Transaction( sub {
                # create new product attribute
                my $Attribute = $ProductType->folder('Attributes')->insertChild($Class, {
                    'Alias'           => $hAttribute->{'Alias'},
                    'Type'            => $hAttribute->{'Type'},
                    'Package'         => $ProductAttributeTypes->{$hAttribute->{'Type'}}->{'Package'},
                    'AttributeClass'  => $ProductType,
                    'IsArray'         => $hAttribute->{'Type'} =~ /Multi/ ? 1 :0,
                    'IsObject'        => $ProductAttributeTypes->{$hAttribute->{'Type'}}->{'IsObject'},
                    'IsUserDefined'   => 1,
                    'IsExportable'    => 1,
                    'ExportLevel'     => 1
                } );

                # set product attribute parameters
                $self->setProductAttributeParameters($Attribute, $hAttribute);

                # set Attributes
                $self->setAttributeValues($Attribute, $hAttribute->{'Attributes'});

                $AttributePath = $Attribute->get('Path');
            });

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $hAttribute->{'Alias'} ),
                'Path'    => $self->soapString( $AttributePath ),
                'created' => $self->soapBool( 1 )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type( 'TCreateProductAttribute_Return')->name('ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TCreateProductAttribute_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $hAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }


        # build return object
        $soapProductAttribute = SOAP::Data->type( 'TCreateProductAttribute_Return')->name('ProductAttribute' => $soapProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    return SOAP::Data->type('type_createProductAttribute_Out')->name('CreateProductAttributeOut' => {
        'ProductAttributes' => $self->soapList(\@soapProductAttributes, 'TCreateProductAttribute_Return')
    });
}

#========================================================================================
# function     getPreDefAttributeInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getPreDefAttributeInfo($ProductAttributePath, $aPreDefAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getPreDefAttributeInfo
#               Returns information about a list of predefined attributes for one product attribute.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given predefined attributes | SOAP::Data
#========================================================================================
sub getPreDefAttributeInfo {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes, $aLanguageCodes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);
    my %PreDefAttributes = ();

    $self->mapType([qw (TGetPreDefAttributeInfo_Return type_getPreDefAttributeInfo_Out)], $self->getServiceTypesUrn);

    eval {
        my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_getPreDefAttributeInfo_Out')->name('GetPreDefAttributeInfoOut' => {
            'Error' => $self->soapError($Error)
        });
    }


    my @soapPreDefAttributes = ();

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            # check if attribute exists
            if (!exists($PreDefAttributes{$PreDefAttributeAlias})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $PreDefAttributeAlias, 'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$PreDefAttributeAlias};

            # build return object
            $soapPreDefAttribute = $self->buildSoapPreDefAttribute($PreDefAttribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TGetPreDefAttributeInfo_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $PreDefAttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapPreDefAttributes, $soapPreDefAttribute);
    }

    my $result = SOAP::Data->type('type_getPreDefAttributeInfo_Out')->name('GetPreDefAttributeInfoOut' => {
        'PreDefAttributes' => $self->soapList(\@soapPreDefAttributes, 'TGetPreDefAttributeInfo_Return')
    });

    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     getAllPreDefAttributesInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getAllPreDefAttributesInfo('/Shops/DemoShop/ProductTypes/Jacket/Attributes/Size');
#               $SoapData = $self->getAllPreDefAttributesInfo($ProductAttributePath, ['de', 'en']);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getAllPreDefAttributesInfo
#               Gets all predefined attributes of the product attribute.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            infos of all predefined attributes | SOAP::Data
#========================================================================================
sub getAllPreDefAttributesInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aLanguageCodes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my $result;

    eval {
        # get object
        my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        $result = SOAP::Data->type('type_getAllPreDefAttributesInfo_Out')->name('GetAllPreDefAttributesInfoOut' => {
            'PreDefAttributes' => $self->soapList(
                [map { $self->buildSoapPreDefAttribute($_, $aLanguages) }
                   @{$ProductAttribute->get('PreDefAttributes')}],
                'TGetAllPreDefAttributesInfo_Return')
            }
        );
    };
    if (ExistsError()) {
        my $Error = GetError();
        $result = SOAP::Data->type('type_getAllPreDefAttributesInfo_Out')->name('GetAllPreDefAttributesInfoOut' => {
            'Error' => $self->soapError($Error)
        });
        ResetError();
    }

    $self->mapType([qw (TGetAllPreDefAttributesInfo_Return type_getAllPreDefAttributesInfo_Out)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     existsPreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->existsPreDefAttribute($ProductAttributePath, $aPreDefAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: existsPreDefAttribute
#               Returns exists status of a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given predefined attributes | SOAP::Data
#========================================================================================
sub existsPreDefAttribute {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();
    my %PreDefAttributes = ();

    $self->mapType([qw (TExistsPreDefAttribute_Return type_existsPreDefAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_existsPreDefAttribute_Out')->name('ExistsPreDefAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        my $hOutPreDefAttribute = {
            'Alias'  => $self->soapString( $PreDefAttributeAlias),
            'exists' => $self->soapBool( exists($PreDefAttributes{$PreDefAttributeAlias}))
        };

        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TExistsPreDefAttribute_Return')->name('PreDefAttribute' => $hOutPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }

    return SOAP::Data->type('type_existsPreDefAttribute_Out')->name('ExistsPreDefAttributeOut' => {
        'PreDefAttributes' => $self->soapList(\@soapPreDefAttributes, 'TExistsPreDefAttribute_Return')
    });
}

#========================================================================================
# function     deletePreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->deletePreDefAttribute($aProductTypess);
#----------------------------------------------------------------------------------------
# description  WebService-Method: deletePreDefAttribute
#               Deletes a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product type | string
# input        $aPreDefAttributes | list of product attribute aliases | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                delete status of all given product attributes | SOAP::Data
#========================================================================================
sub deletePreDefAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();
    my %PreDefAttributes = ();
    my $ProductAttribute;

    $self->mapType([qw (TDeletePreDefAttribute_Return type_deletePreDefAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_deletePreDefAttribute_Out')->name('DeletePreDefAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            # check if attribute exists
            if (!exists($PreDefAttributes{$PreDefAttributeAlias})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $PreDefAttributeAlias, 'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$PreDefAttributeAlias};

            # predef attribute must not be deleted if still used for variation products
            if (scalar @{GetPKeySelectedVariationsByPreDefAttributeID($PreDefAttribute->id)} ||
                scalar @{GetPKeySelectPreDefAttributesByPreDefAttributeID($PreDefAttribute->id)})
            {
                Error('ProductsWithThisPreDefAttributeExist', {'Alias' => $PreDefAttributeAlias, 'ProductAttribute' => $ProductAttributePath});
            }

            # delete predefined attribute
            $PreDefAttribute->delete;
            delete($PreDefAttributes{$PreDefAttributeAlias});

            my $hOutPreDefAttribute = {
                'Alias'   => $self->soapString( $PreDefAttributeAlias ),
                'deleted' => $self->soapBool( not $PreDefAttribute->exists )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TDeletePreDefAttribute_Return')->name('PreDefAttribute' => $hOutPreDefAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type( 'TDeletePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $PreDefAttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    return SOAP::Data->type('type_deletePreDefAttribute_Out')->name('DeletePreDefAttributeOut' => {
        'PreDefAttributes' => $self->soapList(\@soapPreDefAttributes, 'TDeletePreDefAttributes_Return')
    });
}

#========================================================================================
# function     updatePreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->updatePreDefAttribute($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: updatePreDefAttribute
#               Updates a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute info (alias and
#                              data to update) | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given predefined attributes | SOAP::Data
#========================================================================================
sub updatePreDefAttribute {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();
    my %PreDefAttributes = ();
    my $ProductAttribute;

    $self->mapType([qw (TUpdatePreDefAttribute_Return type_updatePreDefAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_updatePreDefAttribute_Out')->name('UpdatePreDefAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    foreach my $hPreDefAttribute (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            $self->checkMandatoryFields($hPreDefAttribute, ['Alias']);

            # check if attribute exists
            if (!exists($PreDefAttributes{$hPreDefAttribute->{'Alias'}})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $hPreDefAttribute->{'Alias'}, 'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$hPreDefAttribute->{'Alias'}};

            Transaction( sub {
                # set predefined attribute parameters
                $self->setPreDefAttributeParameters($PreDefAttribute, $hPreDefAttribute);

                # set Attributes
                $self->setAttributeValues($PreDefAttribute, $hPreDefAttribute->{'Attributes'});
            });

            my $hOutPreDefAttribute = {
                'Alias'   => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'updated' => $self->soapBool( 1 )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TUpdatePreDefAttribute_Return')->name('PreDefAttribute' => $hOutPreDefAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TUpdatePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }

        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TUpdatePreDefAttribute_Return')->name('PreDefAttribute' => $soapPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    return SOAP::Data->type('type_updatePreDefAttribute_Out')->name('UpdatePreDefAttributeOut' => {
        'PreDefAttributes' => $self->soapList(\@soapPreDefAttributes, 'TUpdatePreDefAttribute_Return')
    });
}

#========================================================================================
# function     createPreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->createPreDefAttribute($ProductAttributePath, $aPreDefAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: createPreDefAttribute
#               Creates a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute info (alias and data to set)
#               | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given predefined attributes | SOAP::Data
#========================================================================================
sub createPreDefAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();
    my %PreDefAttributes = ();
    my $ProductAttribute;

    $self->mapType([qw (TCreatePreDefAttribute_Return type_createPreDefAttribute_Out)], $self->getServiceTypesUrn);

    eval {
        $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
        Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath}) unless($ProductAttribute->get('Type') =~ /^PreDef/);
        %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};
    };
    if (ExistsError()) {
        my $Error = GetError();
        return SOAP::Data->type('type_createPreDefAttribute_Out')->name('CreatePreDefAttributeOut' => {
            'Error' => $self->soapError($Error)
        });
    }

    my $Class = LoadClassByAlias('PreDefAttribute');

    foreach my $hPreDefAttribute (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            $self->checkMandatoryFields($hPreDefAttribute, ['Alias']);

            # check if predef attribute with this alias exists
            if (exists $PreDefAttributes{$hPreDefAttribute->{'Alias'}}) {
                Error('PreDefAttributeExists', {'Alias' => $hPreDefAttribute->{'Alias'}, 'ProductAttribute' => $ProductAttributePath});
            }

            my ($PreDefAttributePath, $PreDefAttribute);
            Transaction( sub {
                # create new predefined attribute
                $PreDefAttribute = $ProductAttribute->folder('PreDefAttributes')->insertChild($Class, {
                    'Alias' => $hPreDefAttribute->{'Alias'}
                } );

                # set predefined attribute parameters
                $self->setPreDefAttributeParameters($PreDefAttribute, $hPreDefAttribute);

                # set Attributes
                $self->setAttributeValues($PreDefAttribute, $hPreDefAttribute->{'Attributes'});

                $PreDefAttributePath = $PreDefAttribute->get('Path');
            });

            $PreDefAttributes{$hPreDefAttribute->{'Alias'}} = $PreDefAttribute;

            my $hOutProductType = {
                'Alias'   => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Path'    => $self->soapString( $PreDefAttributePath ),
                'created' => $self->soapBool( 1 )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TCreatePreDefAttribute_Return')->name('PreDefAttribute' => $hOutProductType);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TCreatePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }


        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TCreatePreDefAttribute_Return')->name('PreDefAttribute' => $soapPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    return SOAP::Data->type('type_createPreDefAttribute_Out')->name('CreatePreDefAttributeOut' => {
        'PreDefAttributes' => $self->soapList(\@soapPreDefAttributes, 'TCreatePreDefAttribute_Return')
    });
}

#========================================================================================
# function     setParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setParameters($ProductType, $hProductType);
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Sets parameters of a product type object (for create and update product types).
#----------------------------------------------------------------------------------------
# input        $ProductType | product type to update | product type object
# input        $hProductType | data to assign to the product type
#                             <ul>
#                               <li>Name        - LocalizedString
#                               <li>Layout.*    - String
#                             </ul> | ref.hash
#========================================================================================
sub setParameters {
    my $self = shift;
    my ($ProductType, $hInProductType) = @_;

    #--- localized values
    for (qw (Name)) {
        $self->setLocalizedValues($ProductType, $_, $hInProductType->{$_}) if (defined($hInProductType->{$_}));
    }

    # layout
    my $PageType = LoadPageTypeByAlias(PRODUCT_PAGETYPE);
    foreach my $key (grep { /^Layout\./ } keys %$hInProductType) {
        next unless $hInProductType->{$key};
        #--- check if template exists
        my $TemplateName = substr($key, 7);
        my $TemplateType = $PageType->templateTypeFolder($TemplateName)->child($hInProductType->{$key});
        my $TemplateTypeAlias = $TemplateType->alias;

        # show select box at base template
        if ($TemplateName eq 'Content-Variation') {
            my $VariationTemplateType = $PageType->templateTypeFolder('Content-Base-SelectVariation')->child(
                $hInProductType->{$key} eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $ProductType, 'Content-Base-SelectVariation', $VariationTemplateType->alias );

            # special Content-Variation treatment for new templates (EPG-17552), this will be removed in 6.16.0
            my $VariationTemplateTypeNew = $PageType->templateTypeFolder('Content-PriceBox-SelectVariation')->child(
                $hInProductType->{$key} eq 'VariationSelectBox' ? 'SelectBox' : 'NoSelection'
            );
            $PageType->setTemplateType( $ProductType, 'Content-PriceBox-SelectVariation', $VariationTemplateTypeNew->alias );

            # use not visible Content-Variation-New template type folder
            $PageType->setTemplateType( $ProductType, 'Content-Variation-New', $TemplateTypeAlias.'_New' );
        }
        elsif ($TemplateName eq 'Content-Bundles') {
            # special Content-Bundles treatment for new templates (EPG-17552), this will be removed in 6.16.0
            # use not visible Content-Bundles-New template type folder
            $PageType->setTemplateType( $ProductType, 'Content-Bundles-New', $TemplateTypeAlias.'_New' );
        }

        $PageType->setTemplateType( $ProductType, $PageType->templateTypeFolder($TemplateName)->alias, $TemplateTypeAlias );
    }
}

#========================================================================================
# function     setProductAttributeParameters
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setProductAttributeParameters($ProductAttribute, $hProductAttribute);
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Sets parameters of a product attribute (for create and update product attributes).
#----------------------------------------------------------------------------------------
# input        $ProductAttribute | product attribute to update | product attribute object
# input        $hProductAttribute | data to assign to the product attribute
#                             <ul>
#                               <li>Name        - LocalizedString
#                               <li>Description - LocalizedString
#                               <li>Position    - int
#                               <li>IsVisible   - boolean
#                             </ul> | ref.hash
#========================================================================================
sub setProductAttributeParameters {
    my $self = shift;
    my ($ProductAttribute, $hProductAttribute) = @_;

    #--- localized values
    for (qw (Name Description)) {
        $self->setLocalizedValues($ProductAttribute, $_, $hProductAttribute->{$_}) if (defined($hProductAttribute->{$_}));
    }

    #--- default attributes
    my %Info = ();
    for (qw (Position IsVisible IsHTML)) {
        $Info{$_} = $hProductAttribute->{$_} if (defined $hProductAttribute->{$_});
    }
    $ProductAttribute->set(\%Info);
}

#========================================================================================
# function     setPreDefAttributeParameters
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setPreDefAttributeParameters($PreDefAttribute, $hPreDefAttribute);
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Sets parameters of a predefined attribute (for create and update predefined attributes).
#----------------------------------------------------------------------------------------
# input        $PreDefAttribute | predefined attribute to update | predefined attribute object
# input        $hPreDefAttribute | data to assign to the predefined attribute
#                             <ul>
#                               <li>Position       - int
#                               <li>ValueString    - string (for non-localized product attributes)
#                               <li>ValueLocString - LocalizedString (for localized product attributes)
#                             </ul> | ref.hash
#========================================================================================
sub setPreDefAttributeParameters {
    my $self = shift;
    my ($PreDefAttribute, $hPreDefAttribute) = @_;

    my %Info = ();

    #--- localized values
    my $Type = $PreDefAttribute->get('Attribute')->get('Type');
    if ($Type eq 'PreDefLocalizedString' || $Type eq 'PreDefMultiLocalizedString') {
        for (qw (ValueLocString)) {
            $self->setLocalizedValues($PreDefAttribute, $_, $hPreDefAttribute->{$_}) if (defined($hPreDefAttribute->{$_}));
        }
    }
    else {
        $Info{'ValueString'} = $hPreDefAttribute->{'ValueString'} if (exists $hPreDefAttribute->{'ValueString'});
    }

    #--- default attributes
    for (qw(Position)) {
        $Info{$_} = $hPreDefAttribute->{$_} if (defined $hPreDefAttribute->{$_});
    }

    $PreDefAttribute->set(\%Info);
}

#========================================================================================
# function     buildSoapProductType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildSoapProductType($ProductType, $aAttributes, $aLanguages)
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Build SOAP::Data object for ProductType.
#----------------------------------------------------------------------------------------
# input        $ProductType | product type | ProductType object
# input        $aAttributes | list of attribute alias (optional) | ref.array.string
# input        $aLanguages | list of language codes (optional) | ref.array.string
# return       $SoapData  | a SOAP::Data structure representing the product type | SOAP::Data
#========================================================================================
sub buildSoapProductType {
    my $self = shift;
    my ($ProductType, $aAttributes, $aLanguages) = @_;

    my $hOutProductType = {
        'Path'       => $self->soapString( $ProductType->get('Path') ),
        'Alias'      => $self->soapString( $ProductType->alias ),
        'Name'       => $self->soapList($self->soapLocalizedValues($ProductType, 'Name', $aLanguages ), 'TLocalizedValue' ),
        'Attributes' => $self->soapList($self->soapAttributeValues($ProductType, $aAttributes, $aLanguages), 'TAttribute')
    };

    my $hLayout = $self->getLayout($ProductType);
    foreach my $key (keys %$hLayout) {
        $key =~ s/Content-Base-SelectVariation/Content-Variation/;
        $key =~ s/Content-PriceBox-SelectVariation/Content-Variation/;
        $key =~ s/Content-Variation-New/Content-Variation/;
        $key =~ s/Content-Bundles-New/Content-Bundles/;
        $hOutProductType->{"Layout.$key"} = $self->soapString( $hLayout->{$key} );
    }

    return SOAP::Data->type( 'TGetInfo_Return')->name('ProductType' => $hOutProductType);
}

#========================================================================================
# function     getLayout
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getLayout($ProductType)
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Get the templates for the layout options.
#----------------------------------------------------------------------------------------
# input        $ProductType | product type | ProductType object
# return       $hLayout  | a hash with templates (keys: 'Content-Base', 'Content-Variation', 'Content-Bundles') | ref.hash
#========================================================================================
sub getLayout {
    shift;
    my ($ProductType) = @_;

    my %Layout = ();
    foreach my $hType (@{$ProductType->get('IncludeTemplates')}) {
        my $Key = $hType->{'Alias'};
        $Layout{$Key} = $_->{'Alias'} foreach grep { $_->{'IsActive'} } @{$hType->{'TemplateTypes'}};
    }
    return \%Layout;
}

#========================================================================================
# function     buildSoapProductAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->buildSoapProductAttribute($ProductType)
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Build SOAP::Data object for a product attribute.
#----------------------------------------------------------------------------------------
# input        $ProductAttribute | product attribute | ProductAttribute object
# input        $aAttributes | list of attribute alias (optional) | ref.array.string
# input        $aLanguages | list of language codes (optional) | ref.array.string
# return       $SoapData  | a SOAP::Data structure representing the product attribute | SOAP::Data
#========================================================================================
sub buildSoapProductAttribute {
    my $self = shift;
    my ($ProductAttribute, $aLanguages) = @_;

    my $hOut = {
        'Alias'       => $self->soapString( $ProductAttribute->alias ),
        'Path'        => $self->soapString( $ProductAttribute->get('Path') ),
        'Position'    => $self->soapInt( $ProductAttribute->get('Position') ),
        'Type'        => $self->soapString( $ProductAttribute->get('Type') ),
        'IsVisible'   => $self->soapBool( $ProductAttribute->get('IsVisible') ),
        'IsHTML'      => $self->soapBool( $ProductAttribute->get('IsHTML') ),
        'Name'        => $self->soapList($self->soapLocalizedValues($ProductAttribute, 'Name', $aLanguages ), 'TLocalizedValue' ),
        'Description' => $self->soapList($self->soapLocalizedValues($ProductAttribute, 'Description', $aLanguages ), 'TLocalizedValue' )
    };

    return SOAP::Data->type('TGetProductAttributeInfo_Return')->name('ProductAttribute' => $hOut);
}

#========================================================================================
# function     buildSoapPreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildSoapPreDefAttribute($PreDefAttribute)
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Build SOAP::Data object for PreDefAttribute.
#----------------------------------------------------------------------------------------
# input        $PreDefAttribute | predefined attribute | PreDefAttribute object
# input        $aAttributes | list of attribute alias (optional) | ref.array.string
# input        $aLanguages | list of language codes (optional) | ref.array.string
# return       $SoapData  | a SOAP::Data structure representing the predefined attribute | SOAP::Data
#========================================================================================
sub buildSoapPreDefAttribute {
    my $self = shift;
    my ($PreDefAttribute, $aLanguages) = @_;

    my $hOut = {
        'Alias'       => $self->soapString( $PreDefAttribute->alias ),
        'Path'        => $self->soapString( $PreDefAttribute->get('Path') ),
        'Position'    => $self->soapInt( $PreDefAttribute->get('Position') ),
    };

    my $Type = $PreDefAttribute->get('Attribute')->get('Type');
    if ($Type eq 'PreDefLocalizedString' || $Type eq 'PreDefMultiLocalizedString' || $Type eq 'PreDefCustomerString') {
        $hOut->{'ValueLocString'} = $self->soapList($self->soapLocalizedValues($PreDefAttribute, 'ValueLocString', $aLanguages ), 'TLocalizedValue' );
    }
    else {
        $hOut->{'ValueString'} = $self->soapString( $PreDefAttribute->get('ValueString') );
    }

    return SOAP::Data->type('TGetPreDefAttributeInfo_Return')->name('PreDefAttribute' => $hOut);
}

1;
