#==============================================================================
# package      DE_EPAGES::Product::API::WebService::ProductTypeService2
# base         DE_EPAGES::Product::API::WebService::ProductTypeService
# state        public
# description  Implements the Product Type WebService.
#               extends the original ProductTypeService used by ChannelAdvisor
#               as this one does not work with .NET
#==============================================================================
package DE_EPAGES::Product::API::WebService::ProductTypeService2;
use base DE_EPAGES::Product::API::WebService::ProductTypeService;

use strict;
use DE_EPAGES::Core::API::Error qw (Error ResetError ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(GetLog LogDebug LogError);
use DE_EPAGES::Object::API::Factory qw (Transaction LoadClassByAlias);
use DE_EPAGES::Product::API::Table::SelectedVariation qw (GetPKeySelectedVariationsByAttributeID
                                                          GetPKeySelectedVariationsByPreDefAttributeID);
use DE_EPAGES::Object::API::Table::SelectPreDefAttribute qw (GetPKeySelectPreDefAttributesByAttributeID
                                                             GetPKeySelectPreDefAttributesByPreDefAttributeID);
use DE_EPAGES::Product::API::Constants qw (PRODUCT_ATTRIBUTE_TYPES);
use DE_EPAGES::Product::API::ProductAttribute qw (CheckProductAttributeAlias);


sub new { ## no critic qw( RequireArgUnpacking )
    shift->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ProductTypeService2/2007/11',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/ProductTypeTypes/2007/11',
        @_,
    ));
}

#========================================================================================
# function     getProductAttributeInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getProductAttributeInfo($ProductTypePath, $aAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getProductAttributeInfo
#               Returns information about a list of product attributes for one product type.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given product attributes | SOAP::Data
#========================================================================================
sub getProductAttributeInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes, $aLanguageCodes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    #--- may throw an error
    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
    my %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};

    my @soapProductAttributes = ();

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            # check if attribute exists
            if (!exists($ProductAttributes{$AttributeAlias})) {
                Error('NoResults_Attribute', {'Alias' => $AttributeAlias, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$AttributeAlias};

            # build return object
            $soapProductAttribute = $self->buildSoapProductAttribute($Attribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TGetProductAttributeInfo_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $AttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapProductAttributes, $soapProductAttribute);
    }

    $self->mapType([qw (TGetProductAttributeInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TGetProductAttributeInfo_Return'));
}

#========================================================================================
# function     getAllProductAttributesInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getAllProductAttributesInfo('/Shops/DemoShop/ProductTypes/Tent');
#               $SoapData = $self->getAllProductAttributesInfo('/Shops/DemoShop/ProductTypes/Tent', ['de', 'en']);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getAllProductAttributesInfo
#               Gets all product attributes of the product type.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            infos of all product attributes | SOAP::Data
#========================================================================================
sub getAllProductAttributesInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aLanguageCodes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    # get object
    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');

    my @soapProductAttributes = ();
    foreach my $ProductAttribute(@{$ProductType->directAttributes}) {
        my $soapProductAttribute;
        eval {
            $soapProductAttribute = $self->buildSoapProductAttribute($ProductAttribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TGetProductAttributeInfo_Return')->name('ProductAttribute' => {
                'Path'  => $self->soapPath($ProductAttribute),
                'Error' => $self->soapError($Error)
            });
            ResetError();
        }
        push(@soapProductAttributes, $soapProductAttribute);
    }

    $self->mapType([qw (TGetProductAttributeInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TGetProductAttributeInfo_Return'));
}

#========================================================================================
# function     existsProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->existsProductAttribute($ProductTypePath, $aProductAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: existsProductAttribute
#               Returns exists status of a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given product attributes | SOAP::Data
#========================================================================================
sub existsProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();
    my %ProductAttributes = ();

    #--- may throw an error
    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
    %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        my $hOutProductAttribute = {
            'Alias'   => $self->soapString( $AttributeAlias ),
            'exists' => $self->soapBool( exists($ProductAttributes{$AttributeAlias}) )
        };

        # build return object
        $soapProductAttribute = SOAP::Data->type('TExistsProductAttribute_Return')->name(
            'ProductAttribute' => $hOutProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }

    $self->mapType([qw (TExistsProductAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TExistsProductAttribute_Return'));
}

#========================================================================================
# function     deleteProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->deleteProductAttribute($aProductTypess);
#----------------------------------------------------------------------------------------
# description  WebService-Method: deleteProductAttribute
#               Deletes a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute aliases | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                delete status of all given product attributes | SOAP::Data
#========================================================================================
sub deleteProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();

    #--- may throw an error
    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
    my %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};

    foreach my $AttributeAlias (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            # check if attribute exists
            if (!exists($ProductAttributes{$AttributeAlias})) {
                Error('NoResults_Attribute', {'Alias' => $AttributeAlias, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$AttributeAlias};

            # variation attribute must not be deleted if variation products exist
            if (scalar(@{GetPKeySelectedVariationsByAttributeID($Attribute->id)}) > 0  &&
                scalar(@{GetPKeySelectPreDefAttributesByAttributeID($Attribute->id)}) > 0)
            {
                Error('ProductsWithThisVariationAttributeExist', {'Alias' => $AttributeAlias,
                    'ProductType' => $ProductTypePath});
            }

            # delete product attribute
            $Attribute->delete;
            delete($ProductAttributes{$AttributeAlias});

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $AttributeAlias ),
                'deleted' => $self->soapBool( not $Attribute->exists )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type('TDeleteProductAttribute_Return')->name(
                'ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TDeleteProductAttribute_Return')->name(
                'ProductAttribute' => {
                    'Alias'  => $self->soapString( $AttributeAlias ),
                    'Error'  => $self->soapError($Error)
                }
            );
        }
        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    $self->mapType([qw (TDeleteProductAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TDeleteProductAttribute_Return'));
}

#========================================================================================
# function     updateProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->updateProductAttribute($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: updateProductAttribute
#               Updates a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute info (alias and
#                              data to update) | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given product attributes | SOAP::Data
#========================================================================================
sub updateProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();

    #--- may throw an error
    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
    my %ProductAttributes = map { $_->alias => $_ } @{$ProductType->directAttributes};

    foreach my $hAttribute (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            $self->checkMandatoryFields($hAttribute, ['Alias']);

            # check if attribute exists
            if (!exists($ProductAttributes{$hAttribute->{'Alias'}})) {
                Error('NoResults_Attribute', {'Alias' => $hAttribute->{'Alias'}, 'ProductType' => $ProductTypePath});
            }
            my $Attribute = $ProductAttributes{$hAttribute->{'Alias'}};

            Transaction( sub {
                # set product attribute parameters
                $self->setProductAttributeParameters($Attribute, $hAttribute);

                # set Attributes
                $self->setAttributeValues($Attribute, $hAttribute->{'Attributes'});
            });

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $hAttribute->{'Alias'} ),
                'updated' => $self->soapBool( 1 )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type( 'TUpdateProductAttribute_Return')->name(
                'ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProductAttribute = SOAP::Data->type('TUpdateProductAttribute_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $hAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }

        # build return object
        $soapProductAttribute = SOAP::Data->type('TUpdateProductAttribute_Return')->name(
            'ProductAttribute' => $soapProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    $self->mapType([qw (TUpdateProductAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TUpdateProductAttribute_Return'));
}

#========================================================================================
# function     createProductAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->createProductAttribute($ProductTypePath, $aProductAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: createProductAttribute
#               Creates a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductTypePath | path of product type | string
# input        $aProductAttributes | list of product attribute info (alias and data to set)
#               | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given product attributes | SOAP::Data
#========================================================================================
sub createProductAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductTypePath, $aProductAttributes) = @_;
    GetLog->debug("ProductTypePath=$ProductTypePath");
    LogDebug('aProductAttributes=', $aProductAttributes);

    my @soapProductAttributes = ();

    my $ProductType = $self->loadShopObject($ProductTypePath, 'ProductType');
    my $Class = LoadClassByAlias('ProductAttribute');

    foreach my $hAttribute (@$aProductAttributes) {
        my $soapProductAttribute;

        eval {
            $self->checkMandatoryFields($hAttribute, ['Alias', 'Type']);

            # check attribute alias
            CheckProductAttributeAlias($ProductType, $hAttribute->{'Alias'});

            # check type
            my $ProductAttributeTypes = PRODUCT_ATTRIBUTE_TYPES();
            if (!exists $ProductAttributeTypes->{$hAttribute->{'Type'}}) {
                Error('UnkownAttributeType', {'Type' => $hAttribute->{'Type'},
                    'ValidTypes' => join(', ', sort keys %$ProductAttributeTypes)});
            }

            my $AttributePath;
            Transaction( sub {
                # create new product attribute
                my $Attribute = $ProductType->folder('Attributes')->insertChild($Class, {
                    'Alias'           => $hAttribute->{'Alias'},
                    'Type'            => $hAttribute->{'Type'},
                    'Package'         => $ProductAttributeTypes->{$hAttribute->{'Type'}}->{'Package'},
                    'AttributeClass'  => $ProductType,
                    'IsArray'         => $hAttribute->{'Type'} =~ /Multi/ ? 1 :0,
                    'IsObject'        => $ProductAttributeTypes->{$hAttribute->{'Type'}}->{'IsObject'},
                    'IsUserDefined'   => 1,
                    'IsExportable'    => 1,
                    'ExportLevel'     => 1
                } );

                # set product attribute parameters
                $self->setProductAttributeParameters($Attribute, $hAttribute);

                # set Attributes
                $self->setAttributeValues($Attribute, $hAttribute->{'Attributes'});

                $AttributePath = $Attribute->get('Path');
            });

            my $hOutProductAttribute = {
                'Alias'   => $self->soapString( $hAttribute->{'Alias'} ),
                'Path'    => $self->soapString( $AttributePath ),
                'created' => $self->soapBool( 1 )
            };

            # build return object
            $soapProductAttribute = SOAP::Data->type( 'TCreateProductAttribute_Return')->name(
                'ProductAttribute' => $hOutProductAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
            $soapProductAttribute = SOAP::Data->type('TCreateProductAttribute_Return')->name('ProductAttribute' => {
                'Alias'  => $self->soapString( $hAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }


        # build return object
        $soapProductAttribute = SOAP::Data->type( 'TCreateProductAttribute_Return')->name(
            'ProductAttribute' => $soapProductAttribute);

        push(@soapProductAttributes, $soapProductAttribute);

    }
    $ProductType->folder('Attributes')->renumberChildren;

    $self->mapType([qw (TCreateProductAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('ProductAttributes')->value($self->soapList(\@soapProductAttributes,
        'TCreateProductAttribute_Return'));
}

#========================================================================================
# function     getPreDefAttributeInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getPreDefAttributeInfo($ProductAttributePath, $aPreDefAttributes, $aLanguageCodes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getPreDefAttributeInfo
#               Returns information about a list of predefined attributes for one product attribute.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given predefined attributes | SOAP::Data
#========================================================================================
sub getPreDefAttributeInfo {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes, $aLanguageCodes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    #--- may throw an error
    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);
    my %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};

    my @soapPreDefAttributes = ();

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            # check if attribute exists
            if (!exists($PreDefAttributes{$PreDefAttributeAlias})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $PreDefAttributeAlias,
                    'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$PreDefAttributeAlias};

            # build return object
            $soapPreDefAttribute = $self->buildSoapPreDefAttribute($PreDefAttribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TGetPreDefAttributeInfo_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $PreDefAttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapPreDefAttributes, $soapPreDefAttribute);
    }

    $self->mapType([qw (TGetPreDefAttributeInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TGetPreDefAttributeInfo_Return'));
}

#========================================================================================
# function     getAllPreDefAttributesInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getAllPreDefAttributesInfo('/Shops/DemoShop/ProductTypes/Jacket/Attributes/Size');
#               $SoapData = $self->getAllPreDefAttributesInfo($ProductAttributePath, ['de', 'en']);
#----------------------------------------------------------------------------------------
# description  WebService-Method: getAllPreDefAttributesInfo
#               Gets all predefined attributes of the product attribute.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            infos of all predefined attributes | SOAP::Data
#========================================================================================
sub getAllPreDefAttributesInfo {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aLanguageCodes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);

    my @soapPreDefAttributes = ();
    foreach my $PreDefAttribute(@{$ProductAttribute->get('PreDefAttributes')}) {
        my $soapPreDefAttribute;
        eval {
            $soapPreDefAttribute = $self->buildSoapPreDefAttribute($PreDefAttribute, $aLanguages);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TGetPreDefAttributeInfo_Return')->name('PreDefAttribute' => {
                'Path'  => $self->soapPath($PreDefAttribute),
                'Error' => $self->soapError($Error)
            });
            ResetError();
        }
        push(@soapPreDefAttributes, $soapPreDefAttribute);
    }

    $self->mapType([qw (TGetPreDefAttributeInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);
    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TGetPreDefAttributeInfo_Return'));
}

#========================================================================================
# function     existsPreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->existsPreDefAttribute($ProductAttributePath, $aPreDefAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: existsPreDefAttribute
#               Returns exists status of a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute aliases | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given predefined attributes | SOAP::Data
#========================================================================================
sub existsPreDefAttribute {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();

    #--- may throw an error
    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);
    my %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        my $hOutPreDefAttribute = {
            'Alias'  => $self->soapString($PreDefAttributeAlias),
            'exists' => $self->soapBool( exists($PreDefAttributes{$PreDefAttributeAlias}))
        };

        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TExistsPreDefAttribute_Return')->name(
            'PreDefAttribute' => $hOutPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }

    $self->mapType([qw (TExistsPreDefAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TExistsPreDefAttribute_Return'));
}

#========================================================================================
# function     deletePreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->deletePreDefAttribute($aProductTypess);
#----------------------------------------------------------------------------------------
# description  WebService-Method: deletePreDefAttribute
#               Deletes a list of product attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product type | string
# input        $aPreDefAttributes | list of product attribute aliases | ref.array.string
# return       $SoapData      | a SOAP::Data structure with the
#                                delete status of all given product attributes | SOAP::Data
#========================================================================================
sub deletePreDefAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();

    #--- may throw an error
    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);
    my %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};

    foreach my $PreDefAttributeAlias (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            # check if attribute exists
            if (!exists($PreDefAttributes{$PreDefAttributeAlias})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $PreDefAttributeAlias,
                    'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$PreDefAttributeAlias};

            # predef attribute must not be deleted if still used for variation products
            if (scalar @{GetPKeySelectedVariationsByPreDefAttributeID($PreDefAttribute->id)} ||
                scalar @{GetPKeySelectPreDefAttributesByPreDefAttributeID($PreDefAttribute->id)})
            {
                Error('ProductsWithThisPreDefAttributeExist', {'Alias' => $PreDefAttributeAlias,
                    'ProductAttribute' => $ProductAttributePath});
            }

            # delete predefined attribute
            $PreDefAttribute->delete;
            delete($PreDefAttributes{$PreDefAttributeAlias});

            my $hOutPreDefAttribute = {
                'Alias'   => $self->soapString($PreDefAttributeAlias ),
                'deleted' => $self->soapBool( not $PreDefAttribute->exists )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TDeletePreDefAttribute_Return')->name(
                'PreDefAttribute' => $hOutPreDefAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type( 'TDeletePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $PreDefAttributeAlias ),
                'Error'  => $self->soapError($Error)
            });
        }
        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    $self->mapType([qw (TDeletePreDefAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TDeletePreDefAttributes_Return'));
}

#========================================================================================
# function     updatePreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->updatePreDefAttribute($aProductTypes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: updatePreDefAttribute
#               Updates a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute info (alias and
#                              data to update) | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given predefined attributes | SOAP::Data
#========================================================================================
sub updatePreDefAttribute {
    my $self = shift;

    # expects a product attribute path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();

    #--- may throw an error
    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);
    my %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};

    foreach my $hPreDefAttribute (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            $self->checkMandatoryFields($hPreDefAttribute, ['Alias']);

            # check if attribute exists
            if (!exists($PreDefAttributes{$hPreDefAttribute->{'Alias'}})) {
                Error('NoResults_PreDefAttribute', {'Alias' => $hPreDefAttribute->{'Alias'},
                    'ProductAttribute' => $ProductAttributePath});
            }
            my $PreDefAttribute = $PreDefAttributes{$hPreDefAttribute->{'Alias'}};

            Transaction( sub {
                # set predefined attribute parameters
                $self->setPreDefAttributeParameters($PreDefAttribute, $hPreDefAttribute);

                # set Attributes
                $self->setAttributeValues($PreDefAttribute, $hPreDefAttribute->{'Attributes'});
            });

            my $hOutPreDefAttribute = {
                'Alias'   => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'updated' => $self->soapBool( 1 )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TUpdatePreDefAttribute_Return')->name(
                'PreDefAttribute' => $hOutPreDefAttribute);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TUpdatePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }

        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TUpdatePreDefAttribute_Return')->name(
            'PreDefAttribute' => $soapPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    $self->mapType([qw (TUpdatePreDefAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TUpdatePreDefAttribute_Return'));
}

#========================================================================================
# function     createPreDefAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->createPreDefAttribute($ProductAttributePath, $aPreDefAttributes);
#----------------------------------------------------------------------------------------
# description  WebService-Method: createPreDefAttribute
#               Creates a list of predefined attributes.
#----------------------------------------------------------------------------------------
# input        $ProductAttributePath | path of product attribute | string
# input        $aPreDefAttributes | list of predefined attribute info (alias and data to set)
#               | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given predefined attributes | SOAP::Data
#========================================================================================
sub createPreDefAttribute {
    my $self = shift;

    # expects a product type path
    my ($ProductAttributePath, $aPreDefAttributes) = @_;
    GetLog->debug("ProductAttributePath=$ProductAttributePath");
    LogDebug('aPreDefAttributes=', $aPreDefAttributes);

    my @soapPreDefAttributes = ();

    #--- may throw an error
    my $ProductAttribute = $self->loadShopObject($ProductAttributePath, 'ProductAttribute');
    Error('NoPreDefAttributeType', {'Path' => $ProductAttributePath})
        unless($ProductAttribute->get('Type') =~ /^PreDef/);
    my %PreDefAttributes = map { $_->alias => $_ } @{$ProductAttribute->get('PreDefAttributes')};

    my $Class = LoadClassByAlias('PreDefAttribute');

    foreach my $hPreDefAttribute (@$aPreDefAttributes) {
        my $soapPreDefAttribute;

        eval {
            $self->checkMandatoryFields($hPreDefAttribute, ['Alias']);

            # check if predef attribute with this alias exists
            if (exists $PreDefAttributes{$hPreDefAttribute->{'Alias'}}) {
                Error('PreDefAttributeExists', {'Alias' => $hPreDefAttribute->{'Alias'},
                    'ProductAttribute' => $ProductAttributePath});
            }

            my ($PreDefAttributePath, $PreDefAttribute);
            Transaction( sub {
                # create new predefined attribute
                $PreDefAttribute = $ProductAttribute->folder('PreDefAttributes')->insertChild($Class, {
                    'Alias' => $hPreDefAttribute->{'Alias'}
                } );

                # set predefined attribute parameters
                $self->setPreDefAttributeParameters($PreDefAttribute, $hPreDefAttribute);

                # set Attributes
                $self->setAttributeValues($PreDefAttribute, $hPreDefAttribute->{'Attributes'});

                $PreDefAttributePath = $PreDefAttribute->get('Path');
            });

            $PreDefAttributes{$hPreDefAttribute->{'Alias'}} = $PreDefAttribute;

            my $hOutProductType = {
                'Alias'   => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Path'    => $self->soapString( $PreDefAttributePath ),
                'created' => $self->soapBool( 1 )
            };

            # build return object
            $soapPreDefAttribute = SOAP::Data->type( 'TCreatePreDefAttribute_Return')->name(
                'PreDefAttribute' => $hOutProductType);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapPreDefAttribute = SOAP::Data->type('TCreatePreDefAttribute_Return')->name('PreDefAttribute' => {
                'Alias'  => $self->soapString( $hPreDefAttribute->{'Alias'} ),
                'Error'  => $self->soapError($Error)
            });
            ResetError();
        }


        # build return object
        $soapPreDefAttribute = SOAP::Data->type( 'TCreatePreDefAttribute_Return')->name(
            'PreDefAttribute' => $soapPreDefAttribute);

        push(@soapPreDefAttributes, $soapPreDefAttribute);

    }
    $ProductAttribute->folder('PreDefAttributes')->renumberChildren;

    $self->mapType([qw (TCreatePreDefAttribute_Return)], $self->getServiceTypesUrn);
    return SOAP::Data->name('PreDefAttributes')->value($self->soapList(\@soapPreDefAttributes,
        'TCreatePreDefAttribute_Return'));
}

1;
