#========================================================================================
# package      DE_EPAGES::ProductRecommendation::API::Inquiry
# state        public
# description  Sends the product inquiry e-mail.
#========================================================================================
package DE_EPAGES::ProductRecommendation::API::Inquiry;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    SendProductInquiryMail
);

use Email::Address;
use DE_EPAGES::Core::API::File qw ( ExistsFile );
use DE_EPAGES::Core::API::TestParameter qw( TestHash );
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );

#========================================================================================
# function     SendProductInquiryMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendProductInquiryMail( $Product, $hParams );
# example      SendProductInquiryMail( $Product, {
#                   'Sender'            => $Sender,
#                   'SenderEmail'       => $SenderEmail,
#                   'Inquiry'           => $Inquiry,
#                   'Phone'             => $Phone,
#                   'CurrencyID'        => $CurrencyID,
#                   'LocaleID'          => $LocaleID,
#                   'Copy'              => $Copy,
#               });
#----------------------------------------------------------------------------------------
# description  Sends a product.inquiry e-mail to the shop merchant
#               on behalf of $Sender <$SenderEmail>.
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $hParams | options
#                 <ul>
#                   <li>
#                   <li>Sender - (optional) sender name - string</li>
#                   <li>SenderEmail - sender email - string</li>
#                   <li>Inquiry - inquiry text - string</li>
#                   <li>Phone - (optional) phone numer - string</li>
#                   <li>CurrencyID - (optional, default: shop currency) currency id, e.g. 'EUR' - string</li>
#                   <li>LocaleID - (optional, default: shop locale) locale id, e.g. 'de_DE' - string</li>
#                   <li>Copy - (optional) send a copy to the sender - boolean</li>
#                 </ul>| ref.hash
#========================================================================================
sub SendProductInquiryMail {
    my ( $Product, $hParams ) = @_;
    TestHash('hParams', $hParams, {
        'Sender' => 'String',
        'SenderEmail' => 'String;NotNull',
        'Inquiry' => 'String;NotNull',
        'Phone' => 'String',
        'CurrencyID' => 'String',
        'LocaleID' => 'String',
    });

    my $Shop = $Product->getSite();

    my $MailType  = GetShopMailType( $Shop, 'ProductInquiry' );

    my %Vars = %{$hParams};
    $Vars{'Shop'} = $Shop->tleHash;
    $Vars{'SessionCurrencyID'} = $hParams->{'CurrencyID'};
    $Vars{'Email'} = $hParams->{'SenderEmail'};

    my $LocaleID = $hParams->{'LocaleID'} || $Shop->get('LocaleID');
    my $LanguageID = GetInfoLocale($LocaleID)->{LanguageID};
    my %RegionalParams = (
        Locale => $LocaleID,
        Language => GetCodeByLanguageID( $LanguageID ),
    );


    my $FromAddr = Email::Address->new($hParams->{'Sender'}, $hParams->{'SenderEmail'})->format;
    # get "TO" address from mailtype and add customer for a copy...
    my $TO = $MailType->get('TO');
    my @Receipients = ( $TO );
    push @Receipients, $FromAddr if $hParams->{'Copy'};

    foreach my $Receipient (@Receipients) {
        my %MailParams = (
            'TO' => $Receipient,
            'FROM' => $Receipient,
            'REPLY' => $FromAddr,
            'HTMLAllowed' => 1,
        );

        my $Mail = $MailType->getMail( $Product, \%MailParams, \%RegionalParams, \%Vars );

        # attach inline picture
        my $ImageFilename = $Product->getWithParent('ImageSmall');
        if( defined $ImageFilename ) {
            my $ImageFile = $Product->get('PublicPath').'/'.$ImageFilename;
            if( ExistsFile( $ImageFile ) ) {
                $Vars{'Image'} = 1;
                $Vars{'ImageIsLink'} =  $ImageFilename =~ /\/\// ? 1:0;
                $Mail->addHTMLAttachment([ {
                    'filename'  => $ImageFile,
                    'includeid' => '__ePagesPRC__',
                    'type'      => 'image/jpg'
                } ]);
            }
        }

        # send mail
        $MailType->sendMail( $Mail );
    }

    return;
}

1;
