#========================================================================================
# package      DE_EPAGES::Shippo::API::AppStoreShippo
# state        public
# description  Functions related to Shippo's App.
#========================================================================================
package DE_EPAGES::Shippo::API::AppStoreShippo;

use strict;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Shippo::API::Constants qw (
  SHIPPO_DOMAIN
  SHIPPO_INSTALL_TRIGGER
  SHIPPO_UNINSTALL_TRIGGER
);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AppStoreShippo = DE_EPAGES::Shippo::API::AppStoreShippo->new( );
#----------------------------------------------------------------------------------------
# description  AppStoreShippo constructor
#----------------------------------------------------------------------------------------
# return       $AppStoreShippo | AppStoreShippo object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     getShippoAppId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShippoAppId( $Shop );
#----------------------------------------------------------------------------------------
# description  Obtains Shippo App ID.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop. | object
# return       $appId | Shippo App ID or undef if it isn't installed. | String
#========================================================================================
sub getShippoAppId {
    my $self = shift;
    my ($Shop) = @_;

    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    my $hShopParams = {
        'ShopType' => $Shop->get('ShopTypeRef')->alias(),
        'Shop'     => $Shop
    };

    my $AppsClient = $self->_createAppsClient($hShopParams);

    return $AppsClient->findAppIdByUrl( 'APP_SOAP', SHIPPO_DOMAIN );
}

#========================================================================================
# function     isShippoAppInstalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       isShippoAppInstalled( $Shop, $appId );
#----------------------------------------------------------------------------------------
# description  Determines whether Shippo app is intalled or not.
#----------------------------------------------------------------------------------------
# input        $Shop                 | Shop.                              | object
# input        $appId                | appId. Optional parameter.         | String
# return       $isShippoAppInstalled | True = installed, otherwise false. | boolean
#========================================================================================
sub isShippoAppInstalled {
    my $self = shift;
    my ( $Shop, $appId ) = @_;

    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    unless ( $self->_isValid($appId) ) {
        $appId = $self->getShippoAppId($Shop);
    }
    unless ( $self->_isValid($appId) ) {
        return 0;
    }
    my $hShopParams = {
        'ShopType' => $Shop->get('ShopTypeRef')->alias(),
        'Shop'     => $Shop
    };
    my $AppStore = $self->_createAppsClient($hShopParams);
    $self->_LogShippingFulfillment( 'Shippo', 'appID', $appId, 'isShippoAppInstalled' );
    my $hLinks       = $AppStore->getAppLinks($appId);
    my $aLinks       = $hLinks->{'links'};
    my $aLinksLength = scalar @$aLinks;
    for ( my $i = 0 ; $i < $aLinksLength ; $i++ ) {

        if ( $aLinks->[$i]->{'rel'} eq 'uninstallApp' ) {
            return 1;
        }
    }

    return 0;
}

#========================================================================================
# function     installShippoApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       installShippoApp( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Installs Shippo app.
#----------------------------------------------------------------------------------------
# input        $Servlet   | Servlet                            | object
# input        $Shop      | Current shop                       | object
# return       $installed | True = installed, otherwise false. | boolean
#========================================================================================
sub installShippoApp {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    return $self->_updateShippoAppStatus( $Servlet, $Shop, SHIPPO_INSTALL_TRIGGER );
}

#========================================================================================
# function     uninstallShippoApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       uninstallShippoApp( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Uninstalls Shippo app.
#----------------------------------------------------------------------------------------
# input        $Servlet   | Shop Servlet                       | object
# input        $Shop      | Current shop                       | object
# return       $installed | True = installed, otherwise false. | boolean
#========================================================================================
sub uninstallShippoApp {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    return $self->_updateShippoAppStatus( $Servlet, $Shop, SHIPPO_UNINSTALL_TRIGGER );
}

#========================================================================================
# function     _updateShippoAppStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _updateShippoAppStatus( $Servlet, $Shop, $HookName );
#----------------------------------------------------------------------------------------
# description  Installs or uninstalls Shippo App.
#----------------------------------------------------------------------------------------
# input        $Servlet     | Servlet                            | object
# input        $Shop        | Current shop                       | object
# input        $HookName    | Hook which will be triggered       | String
# return       $installed   | True = installed, otherwise false. | boolean
#========================================================================================
sub _updateShippoAppStatus {
    my $self = shift;
    my ( $Servlet, $Shop, $HookName ) = @_;

    $self->_TestObject( 'Servlet', $Servlet, 'DE_EPAGES::Presentation::Servlet::Permission' );
    $self->_TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $AppId = $Servlet->params('appID');
    unless ( $self->_isValid($AppId) ) {
        $AppId = $self->getShippoAppId($Shop);
    }

    $self->_LogShippingFulfillment( 'Shippo', 'appID', $AppId, '_updateShippoAppStatus' );

    if ( $self->_isValid($AppId) ) {
        eval { $self->_TriggerHook( $HookName, { 'Shop' => $Shop, 'AppID' => $AppId, 'AppUrl' => SHIPPO_DOMAIN } ); };
        if ( ExistsError() ) {
            my $Error = GetError();
            GetLog->error($Error);
            return 0;
        }
    }

    return 1;
}

#----------------------------------------------------------------------------------------
# Wrappers
#========================================================================================
# function     _createAppsClient
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createAppsClient( $hShopParams );
#----------------------------------------------------------------------------------------
# description  Wrapper to create an instance of AppsClient.
#----------------------------------------------------------------------------------------
# input        $hShopParams | Shop and ShopType.                            | object
# return       $appId       | Shippo App ID or undef if it isn't installed. | String
#========================================================================================
sub _createAppsClient {
    my $self = shift;

    my ($hShopParams) = @_;

    my $AppsClient = DE_EPAGES::AppStore::API::Object::AppsClient->create($hShopParams);
    $self->_TestObject( 'AppsClient', $AppsClient, 'DE_EPAGES::AppStore::API::Object::AppsClient' );

    return $AppsClient;
}

#----------------------------------------------------------------------------------------
sub _TestObject {
    my $self = shift;

    my ( $Name, $Value, $ClassPackage ) = @_;

    TestObject( $Name, $Value, $ClassPackage );
}

#----------------------------------------------------------------------------------------
sub _isValid {
    my $self = shift;

    my ($Parameter) = @_;

    return isValid($Parameter);
}

#----------------------------------------------------------------------------------------
sub _LogShippingFulfillment {
    my $self = shift;

    my ( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section ) = @_;

    LogShippingFulfillment( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section );

    return;
}

#----------------------------------------------------------------------------------------
sub _TriggerHook {
    my $self = shift;

    my ( $HookName, $hArgs ) = @_;

    TriggerHook( $HookName, $hArgs );

    return;
}

1;
