#========================================================================================
# package      DE_EPAGES::Shippo::API::CredentialsShippo
# state        public
# description  Class package to handle Shippo credentials stuff.
#========================================================================================
package DE_EPAGES::Shippo::API::CredentialsShippo;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Password qw ( GenerateNewPassword );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shippo::API::Constants qw ( SOAP_USER_PASSWORD_LENGTH );
use DE_EPAGES::Shippo::API::User qw (
  CreateShippoUser
  DeleteShippoUser
  GetShippoAuthToken
);

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new($ShippingMethodShippo);
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $ShippingMethodShippo   | Shipping Method Shippo   | ShippingMethodShippo
# return       $self   | self   | DE_EPAGES::Shippo::API::CredentialsShippo
#========================================================================================
sub new {
    my $class = shift;
    my ($ShippingMethodShippo) = @_;

    my $self = bless { 'ShippingMethodShippo' => $ShippingMethodShippo }, $class;

    return $self;
}

#========================================================================================
# function     createShippoCredentials
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CredentialsShippo->createShippoCredentials($Form,$Shop);
#----------------------------------------------------------------------------------------
# description  Create credentials for Shippo.
#----------------------------------------------------------------------------------------
# input        $Form | Form object
# input        $Shop | Shop object
# return       it returns '1' if credentials were created ok
#========================================================================================
sub createShippoCredentials() {
    my $self = shift;
    my ( $Form, $Shop ) = @_;

    my $ShippingProviderMethodShippo = $self->{'ShippingMethodShippo'}->getShippingProviderMethodShippo($Shop);

    my $Connector   = $self->_createConnector($ShippingProviderMethodShippo);
    my $Password    = $self->_GenerateNewPassword(SOAP_USER_PASSWORD_LENGTH);
    my $User        = $self->_CreateShippoUser( $Shop, $Password );
    my $AuthToken   = $self->_GetShippoAuthToken( $User, $Password, $Connector, $Shop);
    my $ProviderKey = $self->_LoadRootObject()->get('ShippoProviderKey');
    my $ShippingFulfillmentType = $ShippingProviderMethodShippo->get('ShippingFulfillmentType');

    my $ShippoID = $Form->value('ShippoID');

    $self->_setShippoID( $ShippingProviderMethodShippo, $ShippoID );

    eval { $Connector->SendShippoAuthCredentials( $ShippingFulfillmentType, $AuthToken, $ShippoID, $ProviderKey ); };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
        DeleteShippoUser($Shop);
        return 0;
    }
    return 1;
}

#========================================================================================
# function     _createConnector
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createConnector($ShippingProviderMethodShippo);
#----------------------------------------------------------------------------------------
# description  Create a connector for Shippo.
#----------------------------------------------------------------------------------------
# input        $ShippingProviderMethodShippo | Shipping Provider Method Shippo object
# return       $Connector | Connector object
#========================================================================================
sub _createConnector {
    my $self = shift;
    my ($ShippingProviderMethodShippo) = @_;

    return DE_EPAGES::Shippo::API::Connector->new($ShippingProviderMethodShippo);
}

#========================================================================================
# function     _setShippoID
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setShippoID($ShippingProviderMethodShippo,$ShippoID);
#----------------------------------------------------------------------------------------
# description  Set the Shippo ID.
#----------------------------------------------------------------------------------------
# input        $ShippingProviderMethodShippo | Shipping Provider Method Shippo object
# input        $ShippoID                     | the Shippo ID
#========================================================================================
sub _setShippoID {
    my $self = shift;
    my ( $ShippingProviderMethodShippo, $ShippoID ) = @_;

    $ShippingProviderMethodShippo->set( { 'ShippoID' => $ShippoID } );
    return;
}

#========================================================================================
# function     _GenerateNewPassword
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GenerateNewPassword();
#----------------------------------------------------------------------------------------
# description  Generate the new password.
#----------------------------------------------------------------------------------------
# return       the new password.
#========================================================================================
sub _GenerateNewPassword {
    my $self = shift;
    return GenerateNewPassword(SOAP_USER_PASSWORD_LENGTH);
}

#========================================================================================
# function     _CreateShippoUser
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CreateShippoUser();
#----------------------------------------------------------------------------------------
# description  Create the Shippo User.
#----------------------------------------------------------------------------------------
# input        $Shop      | Shop object | Object
# input        $Password  | the password | String
# return       the Shippo user.
#========================================================================================
sub _CreateShippoUser {
    my $self = shift;
    my ( $Shop, $Password ) = @_;
    return CreateShippoUser( $Shop, $Password );
}



#========================================================================================
# function     _GetShippoAuthToken
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetShippoAuthToken($User, $Password, $Connector, $Shop);
#----------------------------------------------------------------------------------------
# description  Create the Shippo User.
#----------------------------------------------------------------------------------------
# input        $User      | User      | object
# input        $Password  | Password  | String
# input        $Connector | Connector | object
# input        $Shop      | Shop      | object
# return       $ShippoAuthToken      | ShippoAuthToken      | object
#========================================================================================
sub _GetShippoAuthToken {
    my $self = shift;
    my ( $User, $Password, $Connector, $Shop ) = @_;

    my $UserPath = $User->pathString();
    my $SoapUrl = $Connector->GetShippoSoapURL($Shop);

    $self->_LogShippingFulfillment('Shippo', 'Soap Access Data: ', { 'User' => $UserPath, 'SoapUrl' => $SoapUrl });

    return GetShippoAuthToken( $User->pathString, $Password, $Connector->GetShippoSoapURL($Shop) );
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _LoadRootObject {
    my $self = shift;
    return LoadRootObject();
}

#----------------------------------------------------------------------------------------
sub _LogShippingFulfillment {
    my $self = shift;

    my ( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section ) = @_;

    LogShippingFulfillment( $ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section );

    return;
}

1;
