#========================================================================================
# package      DE_EPAGES::Shippo::API::ShippingMethodShippo
# state        public
# description  Class package to handle shipping method Shippo stuff.
#========================================================================================
package DE_EPAGES::Shippo::API::ShippingMethodShippo;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject Transaction );
use DE_EPAGES::Shipping::API::Tools qw (
    SetLinkShippingProviderMethod
    SetLogoToShippingMethod
);
use DE_EPAGES::Shippo::API::Constants qw (
    SHIPPING_METHOD_NAME_IN_SHOP
);

use strict;
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new();
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self   | self   | DE_EPAGES::Shippo::API::ShippingMethodShippo
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     getShippingMethodShippo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethodShippo = $ShippingMethodShippo->getShippingMethodShippo($Shop);
#----------------------------------------------------------------------------------------
# description  It returns the Shippo shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# return       $ShippingMethodShippo | Shippo shipping method object | object
#========================================================================================
sub getShippingMethodShippo {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethodShippo = $self->_getShippoShippingMethod($Shop);
    if ( !isValid($ShippingMethodShippo) ) {
        $ShippingMethodShippo = $self->_createShippingMethodShippo($Shop);
    }

    return $ShippingMethodShippo;
}

#==================================================================================================================
# function     getShippingProviderMethodShippo
# state        public
#------------------------------------------------------------------------------------------------------------------
# syntax       $ShippingProviderMethodShippo = $ShippingMethodShippo->getShippingProviderMethodShippo($Shop);
#------------------------------------------------------------------------------------------------------------------
# description  It returns the Shipping provider method
#------------------------------------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       $ProviderMethodsFolder | Shippo shipping provider method object | object
#==================================================================================================================
sub getShippingProviderMethodShippo {
    my $self = shift;
    my ($Shop) = @_;

    my $ProviderMethodsFolder = $Shop->folder('ShippingProviderMethods');

    return $ProviderMethodsFolder->existsChild('Shippo') ? $ProviderMethodsFolder->child('Shippo') : undef;
}

#========================================================================================
# function     _getShippoShippingMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getShippoShippingMethod( $Shop );
#----------------------------------------------------------------------------------------
# description  get the shippo Shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       $ShippoShippingMethod | Shippo shipping method object | object
#========================================================================================
sub _getShippoShippingMethod {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethods = $Shop->folder('ShippingMethods')->children();

    foreach my $ShippingMethod (@$ShippingMethods) {
        my $SelectedShippingProviderMethod = $ShippingMethod->get('SelectedShippingProviderMethod');
        if ( isValid($SelectedShippingProviderMethod) ) {
            my $SelectedShippingProviderMethodAlias = $SelectedShippingProviderMethod->get('Class')->get('Alias');
            if ( $SelectedShippingProviderMethodAlias eq 'ShippingProviderMethodShippo' ) {
                return $SelectedShippingProviderMethodAlias;
            }
        }
    }
    return;
}

#========================================================================================
# function     _createShippingMethodShippo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createShippingMethodShippo( $Shop );
#----------------------------------------------------------------------------------------
# description  Create a Shippo Shipping method
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | Shop
# return       Shipping Method Shippo | Shipping method | object
#========================================================================================
sub _createShippingMethodShippo {
    my $self = shift;
    my ($Shop) = @_;

    LogShippingFulfillment( 'Shippo', 'Creating a new Shipping method (Shippo) for', SHIPPING_METHOD_NAME_IN_SHOP );
    my $NewShippingMethodObject;
    Transaction(
        sub {
            # ShippingFullfillmentTypes: Other carriers Shippo
            my $ShippingFullfillmentType = $self->_getShippingFulfillmentTypeShippo();
            my $ShippingProviderMethod = $self->_getShippingProviderMethodShippo( $Shop, $ShippingFullfillmentType );
            LogShippingFulfillment( 'Shippo', ' Getting the FullfillmentType with ID: ', $ShippingFullfillmentType->id );

            # ShippingTypes:Fixed price
            my $ShippingType = $self->_getFixedPriceShippingType($Shop);
            LogShippingFulfillment( 'Shippo', ' Creating the shipping type with ID: ', $ShippingType->id );

            # Create Shipping method
            $NewShippingMethodObject = $self->_newShippingMethod( $Shop, $ShippingType );
            LogShippingFulfillment('Shippo', ' Creating new Shipping Method has been created with id: ',$NewShippingMethodObject->id);

            $self->_setLinkAndLogo( $NewShippingMethodObject, $ShippingFullfillmentType, $ShippingProviderMethod );
        }
    );

    return $NewShippingMethodObject;
}

#========================================================================================
# function     _setLinkAndLogo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setLinkAndLogo( $NewShippingMethodObject, $ShippingFullfillmentType );
#----------------------------------------------------------------------------------------
# description  It sets the link and logo for the shipping method
#----------------------------------------------------------------------------------------
# input        $NewShippingMethodObject  | Shipping method          | object
# input        $ShippingFullfillmentType | Shipping fulfillmentType | object
# input        $ShippingProviderMethod   | Shipping provider method | object
#========================================================================================
sub _setLinkAndLogo {
    my $self = shift;
    my ( $NewShippingMethodObject, $ShippingFullfillmentType, $ShippingProviderMethod ) = @_;

    if ( isValid($NewShippingMethodObject) && isValid($ShippingProviderMethod) ) {
        SetLinkShippingProviderMethod( $NewShippingMethodObject, $ShippingProviderMethod );
    }
    else {
        LogError(' ShippingFullfillmentType or ShippingMethod are not valid');
    }

    SetLogoToShippingMethod( $NewShippingMethodObject, $ShippingFullfillmentType );
    return;
}

#========================================================================================
# function     _newShippingMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _newShippingMethod( $Shop, $ShippingType  );
#----------------------------------------------------------------------------------------
# description  It creates a new shipping method
#----------------------------------------------------------------------------------------
# input        $Shop                    | Shop            | object
# input        $ShippingType            | ShippingType    | object
# return       $NewShippingMethodObject | shipping method | object
#========================================================================================
sub _newShippingMethod {
    my $self = shift;
    my ( $Shop, $ShippingType ) = @_;

    my %Form;

    my $ShopMethods = $Shop->get('ShippingMethods');
    my $TaxMatrix   = $Shop->get('TaxMatrix');
    $Form{'Shop'}      = $Shop;
    $Form{'IsDefault'} = 1 unless scalar @$ShopMethods;
    $Form{'TaxClass'}  = $TaxMatrix->defaultTaxClass unless defined $Form{'TaxClass'};
    $Form{'Alias'}     = SHIPPING_METHOD_NAME_IN_SHOP;

    my $NewShippingMethodObject;
    eval { $NewShippingMethodObject = $ShippingType->newShippingMethod( $Shop, \%Form ); };

    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
    }

    $Shop->child('ShippingMethods')->renumberChildren;

    return $NewShippingMethodObject;
}

#========================================================================================
# function     _getShippingFulfillmentTypeShippo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getShippingFulfillmentTypeShippo( );
#----------------------------------------------------------------------------------------
# description  It returns the Shippo shipping fulfillment type
#----------------------------------------------------------------------------------------
# return       $ShippingFulfillmentTypeShippo | Shippo shipping fulfillment type | object
#========================================================================================
sub _getShippingFulfillmentTypeShippo {
    my $self = shift;

    return LoadObjectByPath('/ShippingFulfillmentTypes/Shippo');
}

#========================================================================================
# function     _getShippingProviderMethodShippo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getShippingProviderMethodShippo( $Shop, $ShippingFulfillmentType );
#----------------------------------------------------------------------------------------
# description  It returns the Shipping Fulfillment Method
#----------------------------------------------------------------------------------------
# input        $Shop                      | Shop                     | object
# input        $ShippingFulfillmentType   | Shipping Fulfillment type | object
# return       $ShippingFulfillmentMethod | fulfillment method       | object
#========================================================================================
sub _getShippingProviderMethodShippo {
    my $self = shift;
    my ( $Shop, $ShippingFulfillmentType ) = @_;

    my $ShippingProviderMethods = $Shop->get('ShippingProviderMethods');

    foreach my $ShippingProviderMethod (@$ShippingProviderMethods) {
        my $ShippingFulfillmentTypeInUse = $ShippingProviderMethod->get('ShippingFulfillmentType')->id();

        if ( $ShippingFulfillmentType->id() eq $ShippingFulfillmentTypeInUse ) {
            return $ShippingProviderMethod;
        }
    }

    my $NewShippingProviderMethod = $self->_newShippingProviderMethodShippo( $Shop, $ShippingFulfillmentType );

    return $NewShippingProviderMethod;
}


#========================================================================================
# function     _newShippingProviderMethodShippo
# state        private
#----------------------------------------------------------------------------------------
# syntax       _newShippingProviderMethodShippo( $Shop,$ShippingFulfillmentType );
#----------------------------------------------------------------------------------------
# description  It creates a new shipping provider method Shippo
#----------------------------------------------------------------------------------------
# input        $Shop                       | Shop                      | object
# input        $ShippingFulfillmentType    | Shipping Fulfillment Type | object
# return       $NewShippingProviderMethod  | Shipping provider method  | object
#========================================================================================
sub _newShippingProviderMethodShippo {
    my $self = shift;
    my ($Shop,$ShippingFulfillmentType) = @_;

    my $ShippingFulfillmentTypeAlias = $ShippingFulfillmentType->alias();
    my $hInfo = { 'Alias' => $ShippingFulfillmentTypeAlias } ;
    my $NewShippingProviderMethod = $ShippingFulfillmentType->newShippingProviderMethod( $Shop, $hInfo );

    return $NewShippingProviderMethod ;
}

#========================================================================================
# function     _getFixedPriceShippingType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getFixedPriceShippingType( $Shop );
#----------------------------------------------------------------------------------------
# description  It returns the Fixed Price Shipping Type
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# return       $ShippingType | shipping type | object
#========================================================================================
sub _getFixedPriceShippingType {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingTypesList = LoadRootObject()->get('ShippingTypes');

    foreach my $ShippingType (@$ShippingTypesList) {
        my $IsActivated             = $ShippingType->get('IsActivated');
        my $ShippingTypeMethodClass = $ShippingType->get('MethodClass')->get('Alias');
        if ( isValid($ShippingType) && $IsActivated ) {
            if ( $ShippingTypeMethodClass eq 'ShippingMethodFixPrice' ) {
                return $ShippingType;
            }
        }
    }
    return;
}

1;
