#========================================================================================
# package      DE_EPAGES::Shippo::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::Shippo::UI::Shop;
use base DE_EPAGES::ExternalShipping::UI::Shop;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shipping::UI::Shop;
use DE_EPAGES::Shippo::API::AppStoreShippo;
use DE_EPAGES::Shippo::API::CredentialsShippo;
use DE_EPAGES::Shippo::API::ShippingMethodShippo;
use DE_EPAGES::Shippo::API::Tools;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;


#========================================================================================
# function     GrantShippoAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->GrantShippoAuthorization($Servlet);
#----------------------------------------------------------------------------------------
# description  ChangeAction function implemented authorise Shippo connections to
#               Merchant's Shop.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub GrantShippoAuthorization {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $redirectUrl = $Shop->get('WebUrlAdminSSL') . '/?ViewAction=MBO-ViewShippingList&ObjectID=' . $Shop->id ;
    $self->_redirectServlet($Servlet, $redirectUrl);

    return;
}

#========================================================================================
# function     ViewShippoAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewShippoAuthorization($Servlet);
#----------------------------------------------------------------------------------------
# description  ViewAction function implemented as part of the landing page for the Shippo
#               redirection performed right after generating a new Shippo ID for the merchant
#               or when uninstalling Shippo
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub ViewShippoAuthorization {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $IsShippoAppAvailable = $self->_isShippoAppAvailable($Shop);
    if ( $self->_validateConnection($Servlet, $Shop)
            && !($self->_validateShippoInstalled($Servlet, $Shop, $IsShippoAppAvailable)) ) {
        $self->_setupShippo($Servlet, $Shop, $IsShippoAppAvailable);
    }

    if ($IsShippoAppAvailable) {
        $self->_validateUninstallRequest($Servlet);
    } else {
        $self->_redirectToShippingList($Servlet, $Shop);
    }
        

    return;
}

#========================================================================================
# function     ViewShippoConsecution
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewShippoConsecution($Servlet);
#----------------------------------------------------------------------------------------
# description  ViewAction function implemented as part of the landing page for the Shippo.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object
#========================================================================================
sub ViewShippoConsecution {
    my $self      = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $appID = $Servlet->params('appID');
    my $cancelButtonViewURL;
    if (isValid($appID) && !$self->_isMaxDeliveryMethodsReached($Servlet, $Servlet->object())) {
        $cancelButtonViewURL = '?ViewAction=MBO-ViewAppDetails&amp;appID='.$appID;
    }
    else {
        $cancelButtonViewURL = $Shop->isUnity() ?
                '?ViewAction=UnityMBO-ViewShippingList' : '?ViewAction=MBO-ViewShippingList';
    }

    $Servlet->vars('cancelButtonViewURL', $cancelButtonViewURL);
    $Servlet->vars('appID', $appID);

    return;
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------

#========================================================================================
# function     _isMaxDeliveryMethodsReached
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isMaxDeliveryMethodsReached( $Servlet,$Shop );
#----------------------------------------------------------------------------------------
# description  Tells us if the maximum number of delivery methods is reached
#----------------------------------------------------------------------------------------
# input        $Servlet                  | Servlet object                        | object
# input        $Shop                     | Shop object                           | object
# return       $ReturnVariable           | Comparison between feature max Value
#                                             and actual value                    | boolean
#========================================================================================
sub _isMaxDeliveryMethodsReached {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $System = $self->_LoadRootObject();
    my $Feature = $System->child('Features')->child('ShippingMethods');

    my $CurrentValue = $self->_featureCurrentValue($Shop, $Feature);
    my $MaxValue = $self->_featureMaxValue($Shop, $Feature);

    if($CurrentValue >= $MaxValue) {
        my $hParams = {
            'ViewAction' => 'MBO-ViewShippingList',
        };
        my $hOptions = {
            'Type' => 'admin',
            'UseSSL' => 1,
        };
        my $URL = BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
        $self->_redirectServlet($Servlet, $URL);
    }

    return;
}

#========================================================================================
# function     _createShippoCredentials
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createShippoCredentials( $Servlet,$Shop,$ShippingMethodShippo );
#----------------------------------------------------------------------------------------
# description  Create credentials for Shippo.
#----------------------------------------------------------------------------------------
# input        $Servlet                  | Servlet object                        | object
# input        $Shop                     | Shop object                           | object
# input        $ShippingMethodShippo     | Shipping Method Shippo                | object
# return       $ShippoCredentialsCreated | Result of Shippo credentials creation | boolean
#========================================================================================
sub _createShippoCredentials {
    my $self = shift;
    my ($Servlet,$Shop,$ShippingMethodShippo) = @_;

    my $Form = $Servlet->form();
    my $CredentialsShippo = DE_EPAGES::Shippo::API::CredentialsShippo->new($ShippingMethodShippo);

    return $CredentialsShippo->createShippoCredentials($Form,$Shop);
}

#========================================================================================
# function     _redirectToShippingList
# state        private
#----------------------------------------------------------------------------------------
# syntax       _redirectToShippingList( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Redirect flow to shipping list URL
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# input        $Shop | Shop object | object
#========================================================================================
sub _redirectToShippingList {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $CurrentViewAction = $Servlet->params('ViewAction');
    my $NextViewAction = 'MBO-ViewShippingList';
    if ($CurrentViewAction =~ m/^Unity.*/) {
        $NextViewAction = 'Unity' . $NextViewAction;
    }
    my $URL = BuildShopUrl($Shop,
        {'ViewAction' => $NextViewAction, 'ObjectId' => $Shop->id()},
        {'Type' => 'admin','UseSSL' => 1},
        $Servlet);

    $self->_redirectServlet($Servlet, $URL);

    return;
}

#========================================================================================
# function     _setupShippo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setupShippo( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Set up credentials and install shippo app store (if necessary).
#               Also, set servlet installation variables accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# input        $Shop | Shop object | object
# input        $IsShippoAppAvailable | wether shippo app is available in store | boolean
#========================================================================================
sub _setupShippo {
    my $self = shift;
    my ($Servlet, $Shop, $IsShippoAppAvailable) = @_;

    my $ShippingMethodShippo = DE_EPAGES::Shippo::API::ShippingMethodShippo->new();
    my $ShippingMethod = $ShippingMethodShippo->getShippingMethodShippo($Shop);

    my $IsInstalledOk = 0;
    if (isValid($ShippingMethod)) {
        my $ShippoCredentialsCreated = $self->_createShippoCredentials($Servlet, $Shop, $ShippingMethodShippo);
        if ($ShippoCredentialsCreated) {
            if ($IsShippoAppAvailable) {
                my $AppStoreShippo = $self->_createShippoAppStore();
                $IsInstalledOk = $AppStoreShippo->installShippoApp($Servlet, $Shop);
            } else {
                $IsInstalledOk = 1;
            }
        }
    }
    $Servlet->vars('IsInstalledOk', $IsInstalledOk);
    $Servlet->vars('ShippoConnectionValidated', $IsInstalledOk);

    return;
}

#========================================================================================
# function     _validateConnection
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsValidConnection = $self->_validateConnection( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Check components for a successful connection to Shippo and set servlet var accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# return       $IsValidConnection | valid connection flag | boolean
#========================================================================================
sub _validateConnection {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $ShippoConnectionValidated = 0;
    if (isValid($Servlet->params('ShippoID'))) {
        $Servlet->vars('ShippoID', $Servlet->params('ShippoID'));
        $ShippoConnectionValidated = 1;
    } else {
        $self->_validateShippoProviderMethod($Servlet, $Shop);
    }
    $Servlet->vars('ShippoConnectionValidated', $ShippoConnectionValidated);

    return $ShippoConnectionValidated;
}


#========================================================================================
# function     _validateShippoProviderMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsValidProviderMethod = $self->_validateShippoProviderMethod( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  it checks if shippo provider method already exists and sets servlet vars accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Shop | shop object | object
# return       $IsValidProviderMethod | provider method validation | boolean
#========================================================================================
sub _validateShippoProviderMethod {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $ShippingMethodShippo = DE_EPAGES::Shippo::API::ShippingMethodShippo->new();
    my $ShippoProviderMethod = $ShippingMethodShippo->getShippingProviderMethodShippo($Shop);

    if (!defined($ShippoProviderMethod)) {
        $Servlet->vars('NoShippoProviderMethod', 1);
        return 0;
    }

    return 1;
}

#========================================================================================
# function     _validateShippoInstalled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsShippoInstalled = $self->_validateShippoInstalled( $Servlet, $Shop );
#----------------------------------------------------------------------------------------
# description  Checks if shippo is installed and sets servlet var accordingly.
#               Check is done differently depending on shippo app availability.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Object | shop object | object
# input        $IsShippoAppAvailable | if shippo app store is available | boolean
# return       $IsShippoInstalled | shippo installed flag | boolean
#========================================================================================
sub _validateShippoInstalled {
    my $self = shift;
    my ($Servlet, $Shop, $IsShippoAppAvailable) = @_;

    my $IsShippoInstalled;
    if ($IsShippoAppAvailable) {
        my $AppStoreShippo = $self->_createShippoAppStore();
        $IsShippoInstalled =  $AppStoreShippo->isShippoAppInstalled($Shop);
        
    } else {
        $IsShippoInstalled = $self->_isInstallRequest($Servlet) ? 0 : 1;
    }
    $Servlet->vars('IsShippoInstalled', $IsShippoInstalled);

    return $IsShippoInstalled;
}

#========================================================================================
# function     _isShippoAppAvailable
# state        public private
#----------------------------------------------------------------------------------------
# syntax       $IsShippoAppAvailable = $self->_isShippoAppAvailable( $Shop );
#----------------------------------------------------------------------------------------
# description  It provides availability for shippo app
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $IsShippoAppAvailable | shippo app availability flag | boolean
#========================================================================================
sub _isShippoAppAvailable {
    my $self = shift;
    my ($Shop) = @_;

    my $Tools = $self->_createTools();
   return $Tools->IsShippoAppAvailable($Shop);
}

#========================================================================================
# function     _validateUninstallRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsUninstall = $self->_validateUninstallRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  It returns if AppStore request is an uninstall or not (install in this case)
#               and sets servlet variables accordingly
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# return       $IsUninstall | is an uninstall or not | boolean
#========================================================================================
sub _validateUninstallRequest {
    my $self = shift;
    my ($Servlet) = @_;

    my $UninstallFlag = $Servlet->params('uninstall');
    if (!defined($UninstallFlag)) {
        $UninstallFlag = 0;
    }
    $Servlet->vars('UninstallShippo', $UninstallFlag);
    return $UninstallFlag;
}

#========================================================================================
# function     _isInstallRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsInstall = $self->_isInstallRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  It returns if request is an install or uninstall
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet object | object
# return       $IsInstall | is an install or not | boolean
#========================================================================================
sub _isInstallRequest {
    my $self = shift;
    my ($Servlet) = @_;

    my $UninstallFlag = $Servlet->params('uninstall');
    if ($UninstallFlag) {
        return 0;
    }
    return 1;
}

#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------

#----------------------------------------------------------------------------------------
sub _createTools {
    my $self = shift;

    return DE_EPAGES::Shippo::API::Tools->new();
}

#----------------------------------------------------------------------------------------
sub _createShippoAppStore {
    my $self = shift;

    return DE_EPAGES::Shippo::API::AppStoreShippo->new();
}

#----------------------------------------------------------------------------------------
sub _featureCurrentValue {
    my $self = shift;
    my ($Shop, $Feature) = @_;

    return $Shop->featureCurrentValue($Feature);
}

#----------------------------------------------------------------------------------------
sub _featureMaxValue {
    my $self = shift;
    my ($Shop, $Feature) = @_;

    return $Shop->featureMaxValue($Feature);
}

#----------------------------------------------------------------------------------------
sub _redirectServlet {
    my $self = shift;
    my ($Servlet, $redirectUrl) = @_;

    $Servlet->redirect($redirectUrl);
    return;
}

#----------------------------------------------------------------------------------------
sub _LoadRootObject {
    my $self = shift;

    return LoadRootObject();
}

1;
