#========================================================================================
# package      DE_EPAGES::Shippo::t::unit::UI_Shop
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::Shippo::t::unit::UI_Shop;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify::Matcher qw ( HashRef Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use strict;

use constant TEST_SHIPPO_APP_ID => '346789406743';
use constant TEST_SHIPPO_ID => '345623908';
use constant TEST_REDIRECT_URL => '/this/is/redirected/built_url';
use constant TEST_SHOP_ID => '23456';
use constant TEST_WEB_URL_ADMIN_SSL => 'https://test.admin.url';


sub test {
    my $self = shift;

    $self->test_ViewShippoAuthorization_ShippoAppNotAvailable_InstallRequest();
    $self->test_ViewShippoAuthorization_ShippoAppNotAvailable_UninstallRequest();
    $self->test_ViewShippoAuthorization_ShippoAppAvailable_InstallRequest();
    $self->test_ViewShippoAuthorization_ShippoAppAvailable_UninstallRequest();
    $self->test_ViewShippoConsecution_NoAppId_GoToShippingList();
    $self->test_ViewShippoConsecution_WithAppId_GoToAppDetails();
    $self->test_ViewShippoConsecution_WithAppId_GoToAppDetails_NoFeaturesLeft();
    $self->test_GrantShippoAuthorization();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoAuthorization_ShippoAppNotAvailable_InstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 0;
    my $IsShippoAppInstalled = undef;
    my $SUT = $self->_createSUT($IsShippoAppAvailable, $IsShippoAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 1;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest);

    $SUT->ViewShippoAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isShippoAppAvailable'),
            "$SubTestName - Checks whether Shippo app is available in AppStore or not (it is NOT)."
    );
    is(
            $MockedServlet->vars('ShippoConnectionValidated'),
            1,
            "$SubTestName - Shippo id has been provided (because this is an install process)."
    );
    ok(
            WasCalled($SUT, '_isInstallRequest'),
            "$SubTestName - Checks if this is an install request (IT IS)."
    );
    is(
            $MockedServlet->vars('IsShippoInstalled'),
            0,
            "$SubTestName - IsShippoInstalled var is set to zero (not yet installed)."
    );
    ok(
            WasCalled($SUT, '_setupShippo'),
            "$SubTestName - Shippo setup private method gets called."
    );
    ok(
            WasCalled($SUT, '_createShippoCredentials'),
            "$SubTestName - Shippo credentials creation private method gets called."
    );
    ok(
            !WasCalled($SUT, '_createShippoAppStore'),
            "$SubTestName - Shippo App not set as installed because is not available."
    );
    is(
            $MockedServlet->vars('IsInstalledOk'),
            1,
            "$SubTestName - Shippo installation whent well."
    );
    ok(
            WasCalled($SUT, '_redirectToShippingList'),
            "$SubTestName - Flow redirected because this is an installation from shipment."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoAuthorization_ShippoAppNotAvailable_UninstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 0;
    my $IsShippoAppInstalled = undef;
    my $SUT = $self->_createSUT($IsShippoAppAvailable, $IsShippoAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 0;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsShippoAppAvailable);

    $SUT->ViewShippoAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isShippoAppAvailable'),
            "$SubTestName - Checks whether Shippo app is available in AppStore or not (it is NOT)."
    );
    ok(
            WasCalled($SUT, '_validateShippoProviderMethod'),
            "$SubTestName - Shippo provider method gets validated because there's no ShippoID available."
    );
    is(
            $MockedServlet->vars('ShippoConnectionValidated'),
            0,
            "$SubTestName - Shippo connection not validated."
    );
    ok(
            !WasCalled($SUT, '_validateShippoInstalled'),
            "$SubTestName - Do NOT validate Shippo installation on unsuccessful connection validation (uninstall)."
    );
    ok(
            WasCalled($SUT, '_redirectToShippingList'),
            "$SubTestName - Flow redirected because this is an installation from shipment."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoAuthorization_ShippoAppAvailable_InstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 1;
    my $IsShippoAppInstalled = 0;
    my $SUT = $self->_createSUT($IsShippoAppAvailable, $IsShippoAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 1;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsShippoAppAvailable);

    $SUT->ViewShippoAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isShippoAppAvailable'),
            "$SubTestName - Checks whether Shippo app is available in AppStore or not (IT IS)."
    );
    is(
            $MockedServlet->vars('ShippoConnectionValidated'),
            1,
            "$SubTestName - Shippo id has been provided (because this is an install process)."
    );
    ok(
            WasCalled($SUT, '_createShippoAppStore'),
            "$SubTestName - Get AppStore module to know whether Shippo App appears as installed or not."
    );
    is(
            $MockedServlet->vars('IsShippoInstalled'),
            0,
            "$SubTestName - IsShippoInstalled var is set to zero (not yet installed)."
    );
    ok(
            WasCalled($SUT, '_setupShippo'),
            "$SubTestName - Shippo setup private method gets called."
    );
    ok(
            WasCalled($SUT, '_createShippoCredentials'),
            "$SubTestName - Shippo credentials creation private method gets called."
    );
    ok(
            WasCalled($SUT, '_createShippoAppStore'),
            "$SubTestName - Shippo App is set as installed."
    );
    is(
            $MockedServlet->vars('IsInstalledOk'),
            1,
            "$SubTestName - Shippo installation whent well."
    );
    ok(
            WasCalled($SUT, '_validateUninstallRequest'),
            "$SubTestName - Call to check and setup variables about Shippo App (if this is an install or uninstall."
    );
    is(
            $MockedServlet->vars('UninstallShippo'),
            0,
            "$SubTestName - This is an install, so uninstall flag for Shippo App should be FALSE."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoAuthorization_ShippoAppAvailable_UninstallRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 1;
    my $IsShippoAppInstalled = 1;
    my $SUT = $self->_createSUT($IsShippoAppAvailable, $IsShippoAppInstalled);
    my $MockedShop = $self->_createShopAuthorization();
    my $IsInstallRequest = 0;
    my $MockedServlet = $self->_createServletAuthorization($MockedShop, $IsInstallRequest, $IsShippoAppAvailable);

    $SUT->ViewShippoAuthorization($MockedServlet);

    ok(
            WasCalled($SUT,'_isShippoAppAvailable'),
            "$SubTestName - Checks whether Shippo app is available in AppStore or not (IT IS)."
    );
    ok(
            WasCalled($SUT, '_validateShippoProviderMethod'),
            "$SubTestName - Shippo provider method gets validated because there's no ShippoID available."
    );
    is(
            $MockedServlet->vars('ShippoConnectionValidated'),
            0,
            "$SubTestName - Shippo connection not validated."
    );
    ok(
            !WasCalled($SUT, '_validateShippoInstalled'),
            "$SubTestName - Do NOT validate Shippo installation on unsuccessful connection validation (uninstall)."
    );
    ok(
            WasCalled($SUT, '_validateUninstallRequest'),
            "$SubTestName - Call to check and setup variables about Shippo App (if this is an install or uninstall."
    );
    is(
            $MockedServlet->vars('UninstallShippo'),
            1,
            "$SubTestName - This is an uninstall, so uninstall flag for Shippo App should be TRUE."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoConsecution_NoAppId_GoToShippingList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $MockedShop = $self->_createShopConsecution();
    my $hasAppIdParam = 0;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewShippoConsecution($MockedServlet);

    ok(
            !WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - No need to check feature resources because there IS NO appID parameter."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewShippingList/,
            "$SubTestName - Cancellation button resolves to shipping list because there IS NO appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoConsecution_WithAppId_GoToAppDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $FeatureCurrentValue = 0;
    my $FeatureMaxValue = 1;
    my $MockedShop = $self->_createShopConsecution($FeatureCurrentValue, $FeatureMaxValue);
    my $hasAppIdParam = 1;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewShippoConsecution($MockedServlet);

    ok(
            WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - Check feature resources because there IS an appID parameter."
    );
    ok(
            !WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - NO redirect call done because of reached/exceeded max features."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewAppDetails/,
            "$SubTestName - Cancellation button resolves to application details because there IS an appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_ViewShippoConsecution_WithAppId_GoToAppDetails_NoFeaturesLeft {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $FeatureCurrentValue = 1;
    my $FeatureMaxValue = 1;
    my $MockedShop = $self->_createShopConsecution($FeatureCurrentValue, $FeatureMaxValue);
    my $hasAppIdParam = 1;
    my $MockedServlet = $self->_createServletConsecution($MockedShop, $hasAppIdParam);

    $SUT->ViewShippoConsecution($MockedServlet);

    ok(
            WasCalled($SUT, '_isMaxDeliveryMethodsReached'),
            "$SubTestName - Check feature resources because there IS an appID parameter."
    );
    ok(
            WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - Redirect call done because of reached/exceeded max features."
    );
    like(
            $MockedServlet->vars('cancelButtonViewURL'),
            qr/MBO-ViewAppDetails/,
            "$SubTestName - Cancellation button resolves to application details because there IS an appID parameter."
    );
}

#----------------------------------------------------------------------------------------
sub test_GrantShippoAuthorization {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $MockedShop = $self->_createShopGrantAuthorization();
    my $MockedServlet = $self->_createServletGrantAuthorization($MockedShop);

    $SUT->GrantShippoAuthorization($MockedServlet);

    ok(
            WasCalled($MockedServlet, 'redirect'),
            "$SubTestName - Servlet redirect done."
    );
}

#----------------------------------------------------------------------------------------
# helpers (Common)
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($IsShippoAppAvailable, $IsShippoAppInstalled) = @_;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Shippo::UI::Shop', $aParameterList);

    # mocks for Authorization tests
    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(
                    Object('DE_EPAGES::Shop::API::Object::Shop'),
                    HashRef(),
                    HashRef(),
                    Object()
            )->thenReturn(TEST_REDIRECT_URL);

    $MockifySut->spy('_isShippoAppAvailable')
            ->whenAny();
    if (defined($IsShippoAppAvailable)) {
        $MockifySut->mockStatic('DE_EPAGES::Shippo::API::Tools::IsShippoAppAvailable')
                ->when(
                        Object('DE_EPAGES::Shippo::API::Tools'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturn($IsShippoAppAvailable);
    } else {
        $MockifySut->mockStatic('DE_EPAGES::Shippo::API::Tools::IsShippoAppAvailable')
                ->when(
                        Object('DE_EPAGES::Shippo::API::Tools'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturnUndef();
    }
    $MockifySut->spy('_validateShippoProviderMethod')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::ShippingMethodShippo::getShippingProviderMethodShippo')
            ->when(
                    Object('DE_EPAGES::Shippo::API::ShippingMethodShippo'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodShippo());

    $MockifySut->spy('_validateShippoInstalled')
            ->whenAny();
    if (defined($IsShippoAppInstalled)) {
        $MockifySut->mockStatic('DE_EPAGES::Shippo::API::AppStoreShippo::isShippoAppInstalled')
                ->when(
                        Object('DE_EPAGES::Shippo::API::AppStoreShippo'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturn($IsShippoAppInstalled);
    } else {
        $MockifySut->mockStatic('DE_EPAGES::Shippo::API::AppStoreShippo::isShippoAppInstalled')
                ->when(
                        Object('DE_EPAGES::Shippo::API::AppStoreShippo'),
                        Object('DE_EPAGES::Shop::API::Object::Shop')
                )->thenReturnUndef();
    }

    $MockifySut->spy('_isInstallRequest')
            ->whenAny();

    $MockifySut->spy('_setupShippo')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::ShippingMethodShippo::getShippingMethodShippo')
            ->when(
                    Object('DE_EPAGES::Shippo::API::ShippingMethodShippo'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodShippo());

    $MockifySut->spy('_createShippoCredentials')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::CredentialsShippo::createShippoCredentials')
            ->when(
                    Object('DE_EPAGES::Shippo::API::CredentialsShippo'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn(1);

    $MockifySut->spy('_createShippoAppStore')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::AppStoreShippo::installShippoApp')
            ->when(
                    Object('DE_EPAGES::Shippo::API::AppStoreShippo'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn(1);

    $MockifySut->spy('_validateUninstallRequest')
            ->whenAny();

    $MockifySut->spy('_redirectToShippingList')
            ->whenAny();

    # additional mocks for Consecution tests
    $MockifySut->spy('_isMaxDeliveryMethodsReached')
            ->whenAny();

    # additional mocks for GrantAuthorization test
    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
            ->whenAny()->thenReturn($self->_createSystem());

    return $MockifySut->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (Authorization)
#----------------------------------------------------------------------------------------
sub _createShopAuthorization {
    my $self = shift;

    my $hAttributes = {
        'ID' => TEST_SHOP_ID,
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createServletAuthorization {
    my $self = shift;
    my ($MockedShop, $IsInstallRequest, $IsShippoAppAvailable) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $aParameters = [ { 'string' => TEST_REDIRECT_URL } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    if ($IsInstallRequest) {
        # when installing, ShippoID must have been provided after contacting Shippo. Otherwise we are deactivating.
        $MockifyServlet->addParams( { 'ShippoID' => TEST_SHIPPO_ID } );
    } else {
        if ($IsShippoAppAvailable) {
            $MockifyServlet->addParams( { 'uninstall' => 1 } );
        }
    }
    
    $MockifyServlet->addVars(
            {
                'IsShippoInstalled' => undef,
                'ShippoConnectionValidated' => undef,
                'UninstallShippo' => undef,
            }
    );
    return $MockifyServlet->getMockObject();
}

sub _createShippingProviderMethodShippo {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyShippingProviderMethodShippo = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shippo::API::Object::ShippingProviderMethodShippo', $hAttributes
    );

    return $MockifyShippingProviderMethodShippo->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (Consecution)
#----------------------------------------------------------------------------------------

sub _createShopConsecution {
    my $self = shift;
    my ($FeatureCurrentValue, $FeatureMaxValue) = @_;

    my $hAttributes = {};

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    my $aParameter = [ {'object' => 'DE_EPAGES::Shop::API::Object::Feature' } ];
    $MockifyShop->addMockWithReturnValueAndParameterCheck('featureCurrentValue', $FeatureCurrentValue, $aParameter);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('featureMaxValue', $FeatureMaxValue, $aParameter);
    $MockifyShop->addMockWithReturnValue('isUnity', 0);

    return $MockifyShop->getMockObject();
}

sub _createServletConsecution {
    my $self = shift;
    my ($MockedShop, $hasAppIdParam) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $aParameters = [ { 'string' => TEST_REDIRECT_URL } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    my $AppID = $hasAppIdParam ? TEST_SHIPPO_APP_ID : undef;
    $MockifyServlet->addParams( { 'appID' => $AppID } );

    return $MockifyServlet->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {};
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );
    my $aParameter = [ { 'string' => 'Features' } ];
    $MockifySystem->addMockWithReturnValueAndParameterCheck('child', $self->_createObject(), $aParameter);

    return $MockifySystem->getMockObject();
}

sub _createObject {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object', $hAttributes
    );
    my $aParameter = [ { 'string' => 'ShippingMethods' } ];
    $MockifyObject->addMockWithReturnValueAndParameterCheck('child', $self->_createFeature(), $aParameter);

    return $MockifyObject->getMockObject();
}

sub _createFeature {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyFeature = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Feature', $hAttributes
    );

    return $MockifyFeature->getMockObject();
}

#----------------------------------------------------------------------------------------
# helpers (GrantAuthorization)
#----------------------------------------------------------------------------------------
sub _createShopGrantAuthorization {
    my $self = shift;

    my $hAttributes = {
        'ID' => TEST_SHOP_ID,
        'WebUrlAdminSSL' => TEST_WEB_URL_ADMIN_SSL
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createServletGrantAuthorization {
    my $self = shift;
    my ($MockedShop) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShop);
    my $ExpectedRedirectUrl = TEST_WEB_URL_ADMIN_SSL . '/?ViewAction=MBO-ViewShippingList&ObjectID=' . TEST_SHOP_ID;
    my $aParameters = [ { 'string' => $ExpectedRedirectUrl } ];
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, $aParameters);

    return $MockifyServlet->getMockObject();
}

__PACKAGE__->runScript();