#========================================================================================
# package      DE_EPAGES::TLE::API::HtmlFormatter
# state        public
#----------------------------------------------------------------------------------------
# description  TLE format handler for HTML texts. This handler supports the following
#               formats:
#               <ul>
#                 <li>[html] escapes <>&'" with the resp. hex encoding (format &#HH;) or HTML entity</li>
#                 <li>[decodehtml] html vice versa</li>
#                 <li>[nohtml] all HTML tags, SCRIPT elements and STYLE elements are removed; multiple spaces are condensed into one</li>
#                 <li>[js] escapes \'" and all control chars with the resp. hex encoding (format \u00HH)</li>
#                 <li>[preline] html formatting and replaces new lines with &lt;br /&gt;</li>
#                 <li>[uri] special characters are escaped as %XX</li>
#                 <li>[url] special characters are escaped as %XX, except '/'</li>
#                 <li>[urljsarray] split url into javascript join array to prevent indexing by search robots</li>
#                 <li>[color] used for color specification in CSS</li>
#                 <li>[correcthtml] closes any open left html tags to get a xhtml valid tag tree</li>
#                 <li>[nomedia] removes <img> tags</li>
#                 <li>[noactivemedia] removes object,embed,noembed,applet and script tags</li>
#                 <li>[noembed] removes object,embed,noembed and applet tags</li>
#               </ul>
#========================================================================================
package DE_EPAGES::TLE::API::HtmlFormatter;

use strict;
use feature 'switch';

use HTML::Entities qw( encode_entities decode_entities );
use URI::Escape qw( uri_escape_utf8 );
use DE_EPAGES::Core::API::Html qw ( CloseOpenHTMLTags RemoveHtmlTags );

sub new { return bless {}, shift; }

sub register {
    my $self = shift;
    my ($TLEProcessor) = @_;
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'html' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'decodehtml' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'preline' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'uri' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'url' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'urljsarray' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'js' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'nohtml' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'color' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'nomedia' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'noactivemedia' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'noembed' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'correcthtml' );
    $TLEProcessor->registerHandler( 'FormatHandler', $self, 'css' );
    $TLEProcessor->registerHandler( 'FunctionHandler', $self, 'DEFAULTFORMAT');
    return;
}

#========================================================================================
# function     DEFAULTFORMAT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("DEFAULTFORMAT", #Format) template #ENDBLOCK
# example      #LOCAL("Value", "<>'")
#                 #Value
#                 #BLOCK("DEFAULTFORMAT", "js")
#                   #Value
#                 #ENDBLOCK
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Switches the default format to 'js'. This function is also used to
#               switch the default formatting between <script ...></script> tags
#----------------------------------------------------------------------------------------
# input        #Format | formatter name | string
#========================================================================================
sub DEFAULTFORMAT { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParameter, $cTemplate) = @_;
    local $Processor->{'Context'}{'DefaultFormatting'} = $aParameter->[0];
    return join '', @{$Processor->replaceTLEs($cTemplate)};
}

#========================================================================================
sub _encodeForJS {
    local $_ = shift;

    # consolidate newlines
    s/\r?\n/\n/g;

    # encode all non alpha numeric characters - prevent xss
    s/([^\w\-])/'\\u' . sprintf('%04X', ord($1))/gex;

    return $_;
}

#========================================================================================
sub Format {
    my $self = shift;
    my ($Processor, $Format, $Value, $TLEName) = @_;

    given ($Format) {

        # --- HTML

        when ('html') {
            return encode_entities($Value, '<>&"\'');
        }
        when ('decodehtml') {
            return decode_entities($Value);
        }
        when ('preline') {
            $Value = encode_entities($Value, '<>&"\'');
            $Value =~ s/\n/<br \/>/g;
            return $Value;
        }
        when ('correcthtml') {
            return CloseOpenHTMLTags($Value);
        }
        when ('nohtml') {
            return RemoveHtmlTags($Value);
        }
        when (['nomedia', 'noactivemedia', 'noembed']) {
            my @MultiMediaTags = qw( object embed noembed applet );

            if ($Format eq 'nomedia') {
                $Value =~ s/<\s*img[^>]+>//igs;
            }
            if ($Format ~~ ['nomedia', 'noactivemedia']) {
                push(@MultiMediaTags, 'script');
            }

            return _RemoveTags($Value, \@MultiMediaTags);
        }

        #--- URL

        when ('uri') {
            return uri_escape_utf8($Value);
        }
        when ('url') {
            $Value = uri_escape_utf8($Value);
            $Value =~ s/%2F/\//ig;
            return $Value;
        }

        #--- JavaScript

        when ('js') {
            $Value = _encodeForJS($Value);

            return $Value;
        }
        when ('urljsarray') {
            $Value = _encodeForJS($Value);

            # split mustn't omit empty elements (parameter '-1') otherwise trailing slashes are thrown away
            return '["' . join('", "', split(m/\\u002F/, $Value, -1)) . '"].join("/")';
        }

        #--- CSS

        when ('color') {
            return 'transparent' unless defined $Value;
            return $Value if $Value =~ /^#/;
            return $Value if $Value !~ /^[0-9a-fA-F]+$/;
            my $Length = length($Value);
            return ($Length == 3 || $Length == 6) ? '#'.$Value : $Value;
        }
        when ('css') {
            return $Value;
        }
    }
}

sub _RemoveTags {
    my ($Value, $aTags) = @_;

    foreach my $MultiMediaTag (@$aTags) {
        my $Found = 1;
        while ($Found) {
            $Found = 0;
            if ($Value =~ /<\s*$MultiMediaTag[^>]*(\/?)>/pxi) { # match html tag
                my $Pre = ${^PREMATCH};
                my $Closed = $1;
                my $Post = ${^POSTMATCH};
                $Found = 1;
                if ($Closed ne '/') {
                    if ($Post =~ /<\/\s*$MultiMediaTag\s*>/pi) {
                        $Value = $Pre . ${^POSTMATCH};
                    }
                    else { # forgot </element>
                        $Value = $Pre.$Post;
                    }
                }
                else {
                    $Value = $Pre.$Post;
                }
            }
        }
    }

    return $Value;
}

1;
